/**
 * 
 * other original works), unless otherwise noted, is licensed under a Creative
 * Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2009 Open-Xchange, Inc. Mail: info@open-xchange.com
 * 
 * @author Matthias Biggeleben <matthias.biggeleben@open-xchange.com>
 * @ignore
 */

var calendarGridInstances = {};
var CG_ROW_HEIGHT = 50; // in px
var CG_PX2EM = 12;
var CG_HEAD_ZOOM = 75; // in % (show hours in header if > x%)
var CG_HOUR_ZOOM = 50; // in % (show hours in grid if > x%)

function getCalendarGrid(id) {
    // no instance for current view?
    if (!calendarGridInstances[id]) {
        // create new instance
        calendarGridInstances[id] = new CalendarGrid();
    }
    // return instance
    return calendarGridInstances[id];
}

function getCalendarGridIfExists(id, callback) {
    var grid = calendarGridInstances[id];
    if (grid) {
        callback(grid);
    }
}

// TODO:
// register for event "something" that is triggered if calendar data is
// changed. Might become unnecessary if this whole thing works with the cache.
// However, CalendarGrid's property "appointments" must be cleared and the grid
// must be repainted [e.g., grid.update()]. This should get it done.

var calendarGridHandlers = {
        
    addTeamMember: function() {
        getCalendarGridIfExists("teamview", function(grid) {
            if (!grid.participantsDialog) {
                // wrapper
                var wrapper = function(list) {
                    // this list is as good as a team
                    grid.addTeam(list);
                };
                grid.participantsDialog = new ParticipantsSmall(
                    null, wrapper, true, true, true, 
                    false, "Select Members", /* i18n */
                    "grid", false, false, true
                );
            }
            grid.participantsDialog.openAddParticipantsWindow();
        });
    },
    
    removeTeamMember: function() {
        getCalendarGridIfExists("teamview", function(grid) {
            grid.removeSelectedRows();
            grid.clearPartial();
            grid.update();
        });
    },
    
    removeAllTeamMembers: function() {
        getCalendarGridIfExists("teamview", function(grid) {
            grid.removeAll();
            grid.clearPartial();
            grid.update();
        });
    },
    
    sortTeamMembers: function() {
        getCalendarGridIfExists("teamview", function(grid) {
            grid.sort();
            grid.clearPartial();
            grid.update();
        });
    },
    
    appointmentSelected: function() {
        getCalendarGridIfExists("teamview", function(grid) {
            // set (magical) global vars
            selectedAppointment = grid.appointmentSelection.getSelectedItems();
            lastSelectedAppointment = selectedAppointment;
            lastUpdateOfCalendarTimestamp = 0;
            // trigger global event
            triggerEvent("Selected", selectedAppointment);
        });
    },
    
    refresh: function() {
        getCalendarGridIfExists("teamview", function(grid) {
            // clear appointments
            grid.invalidateAppointments();
            // update grid
            grid.clearPartial();
            grid.update();
        });
    },

    update: function() {
        getCalendarGridIfExists("teamview", function(grid) {
            // update grid
            grid.clearPartial();
            grid.update();
        });
    },

    clearSelection: function() {
        getCalendarGridIfExists("teamview", function(grid) {
            // clear selection
            grid.appointmentSelection.clear();
        });
    },
    
    gotoToday: function () {
        getCalendarGridIfExists("teamview", function(grid) {
            var now = new Date();
            // update mini calendar
            activeYear = now.getUTCFullYear();
            activeMonth = now.getUTCMonth();
            activeDay = now.getUTCDate();
            if (oMiniCalendar) oMiniCalendar.setSelectedByDate(activeYear, activeMonth, activeDay);
            // update grid
            grid.setGridInterval(activeYear, activeMonth, activeDay);
            grid.clearPartial();
            grid.update();
        });
    },
    
    gotoDay: function(year, month, day) {
        getCalendarGridIfExists("teamview", function(grid) {
            // update mini calendar
            activeYear = year;
            activeMonth = month;
            activeDay = day;
            if (oMiniCalendar) oMiniCalendar.setSelectedByDate(activeYear, activeMonth, activeDay);
            // update grid
            grid.setGridInterval(year, month, day);
            grid.clearPartial();
            grid.update();
        });
    },
    
    setColorLabel: function(labelNumber) {
        lastUpdateOfCalendarTimestamp = (new Date()).getTime();
        calendarAddTag(labelNumber, true);
        getCalendarGridIfExists("teamview", function(grid) {
            grid.invalidateAppointments();
        });
    },
    
    configurationChanged: function() {
        getCalendarGridIfExists("teamview", function(grid) {
            // update team
            if (grid.currentTeam != null) {
                grid.loadTeamByIndex(grid.currentTeam);
            }
            // refresh
            calendarGridHandlers.refresh();
        });
    },
    
    registerAll: function() {
        register("OX_Calendar_Teammember_Add", calendarGridHandlers.addTeamMember);
        register("OX_Calendar_Teammember_Remove", calendarGridHandlers.removeTeamMember);
        register("OX_Calendar_Teammember_Sort", calendarGridHandlers.sortTeamMembers);
        register("OX_Calendar_Teammember_RemoveAll", calendarGridHandlers.removeAllTeamMembers);
        register("OX_Calendar_Appointment_Selected", calendarGridHandlers.appointmentSelected);
        register('LanguageChanged', calendarGridHandlers.update);
        register('OX_Refresh', calendarGridHandlers.refresh);
        register('OX_After_Delete_Appointment', calendarGridHandlers.refresh);
        register('OX_After_Update_Appointment', calendarGridHandlers.refresh);
        register('OX_After_New_Appointment', calendarGridHandlers.refresh);
        register('OX_After_MoveCopy_Appointment', calendarGridHandlers.clearSelection);
        register('OX_Calendar_Today', calendarGridHandlers.gotoToday);
        register('OX_Mini_Calendar_Date_Picked', calendarGridHandlers.gotoDay);
        register("OX_Add_Flag", calendarGridHandlers.setColorLabel);
    },
    
    registerOnce: function() {
        // this one is registered once (and stays registered) to support
        // changing the teams in the config view and getting informed about that
        register("OX_Configuration_Changed", calendarGridHandlers.configurationChanged);
    },
    
    unregisterAll: function() {
        // unregister
        unregister("OX_Calendar_Teammember_Add", calendarGridHandlers.addTeamMember);
        unregister("OX_Calendar_Teammember_Remove", calendarGridHandlers.removeTeamMember);
        unregister("OX_Calendar_Teammember_Sort", calendarGridHandlers.sortTeamMembers);
        unregister("OX_Calendar_Teammember_RemoveAll", calendarGridHandlers.removeAllTeamMembers);
        unregister("OX_Calendar_Appointment_Selected", calendarGridHandlers.appointmentSelected);
        unregister('LanguageChanged', calendarGridHandlers.update);
        unregister('OX_Refresh', calendarGridHandlers.refresh);
        unregister('OX_After_Delete_Appointment', calendarGridHandlers.refresh);
        unregister('OX_After_Update_Appointment', calendarGridHandlers.refresh);
        unregister('OX_After_New_Appointment', calendarGridHandlers.refresh);
        unregister('OX_After_MoveCopy_Appointment', calendarGridHandlers.clearSelection);
        unregister('OX_Calendar_Today', calendarGridHandlers.gotoToday);
        unregister('OX_Mini_Calendar_Date_Picked', calendarGridHandlers.gotoDay);
        unregister("OX_Add_Flag", calendarGridHandlers.setColorLabel);
        // clear current selection
        getCalendarGridIfExists("teamview", function(grid) {
            grid.invalidateAppointments();
        });        
    }
};

calendarGridHandlers.registerOnce();

function calendarGrid_deleteSelectedAppointments() {
    getCalendarGridIfExists("teamview", function(grid) {
        // get selection
        var apps = grid.appointmentSelection.getSelectedItems();
        // define delete routine // user will be asked first (see below)
        var deleteRoutine = function() {
            // 0. clear appointments
            grid.invalidateAppointments();
            // the tricky part here is that we cannot directly delete the appointment
            // but remove the particular team member from the participants list. Thus:
            // 1. we prepare the given appointment data for a server request
            var multiple = [];
            for (var i = 0; i < apps.length; i++) {
                var request = {
                        "action": "get",
                        "module": "calendar",
                        "folder": apps[i].folder,
                        "id": apps[i].id
                };
                // add recurrence_position?
                if (apps[i].recurrence_position) {
                    request.recurrence_position = apps[i].recurrence_position;
                }
                multiple.push(request);
            }
            // 2. we ask the server for appointments details, i.e. the participants
            json.put(AjaxRoot + "/multiple?session=" + session, multiple, null, function(response) {
                // 3. now loop through the response to consolidate appointments.
                // we need to do this because the selection might refer to identical appointments
                // and we want to avoid that the last update overrides the previous ones
                var hash = {};
                var deleteHash = {};
                var timestamp = 0;
                for (var r = 0; r < response.length; r++) {
                    var app = response[r].data;
                    // tidy up
                    grid.tidyUpAppointment(app);
                    // create composite id (just the id & position; not the folder!)
                    var compId = [app.id, app.recurrence_position].join(".");
                    // update timestamp
                    timestamp = Math.max(timestamp, response[r].timestamp);
                    // add to hash
                    hash[compId] = app;
                }
                // now, hash is free of duplicates
                // 4. we loop again through the selected appointments
                for (var i = 0; i < apps.length; i++) {
                    // split up the selection id; format: folder.app_id.app_pos @ user_id:type
                    var keys = apps[i].selectionId.match(/^\w+\.(\w+\.\w+)\@(\w+)\:(\w+)$/);
                    // get the appointment composite key, the participant id, and the type
                    var compId = keys[1];
                    var id = keys[2];
                    var type = keys[3];
                    // found one?
                    if (typeof(hash[compId]) != "undefined") {
                        // 5a. do we delete the owner?
                        if (id == app.created_by) {
                            deleteHash[compId] = hash[compId];
                        } else {
                            // 5b. we look for this participant by looping through the list
                            var app = hash[compId];
                            for (var p = 0; p < app.participants.length; p++) { // fifth element (see columns)
                                var part = app.participants[p];
                                // match?
                                if (part.type == type && part.id == id) {
                                    // 6. delete it
                                    app.participants.splice(p, 1);
                                    break;
                                }
                            }
                        }
                    }
                }
                // clean up "drop list" (these appointments do not require an update, since they will be deleted)
                for (var compId in deleteHash) {
                    delete hash[compId];
                }
                // 7. throw the new data back to the server (delete participants)
                var multipleDrop = [];
                var localTimestamp = timestamp;
                for (var compId in hash) {
                    var app = {
                            "module": "calendar",
                            "action": "update",
                            "folder": hash[compId].folder_id,
                            "id": hash[compId].id,
                            "data": {
                                "participants": hash[compId].participants
                            },
                            "timestamp": localTimestamp
                    };
                    // add an hour to the timestamp (workaround) to prevent update collision errors
                    localTimestamp += 1000 * 60 * 60;
                    // add recurrence_id?
                    if (hash[compId].recurrence_id)
                        app.data.recurrence_id = hash[compId].recurrence_id;
                    // add recurrence_position?
                    if (hash[compId].recurrence_position)
                        app.data.recurrence_position = hash[compId].recurrence_position;
                    multipleDrop.push(app);
                }
                // send first request
                json.put(AjaxRoot + "/multiple?session=" + session + "&continue=true", multipleDrop, null, function(response) {
                    // update timestamp (only available if appointments have been updated)
                    for (var i = 0; i < response.length; i++) {
                        timestamp = Math.max(timestamp, response[i].timestamp || 0);
                    }
                    // prepare 2nd request
                    var multipleDelete = [];
                    for (var compId in deleteHash) {
                        var app = {
                                "module": "calendar",
                                "action": "delete",
                                "data": {
                                    "folder": deleteHash[compId].folder_id,
                                    "id": deleteHash[compId].id,
                                    "pos": deleteHash[compId].recurrence_position
                                },
                                "timestamp": timestamp
                        };
                        multipleDelete.push(app);
                    }
                    // send 2nd request
                    json.put(AjaxRoot + "/multiple?session=" + session + "&continue=true", multipleDelete, null, function(response) {
                        // finally...
                        calendarGridHandlers.refresh();
                    });
                });
            });
        } // end deleteRoutine
        
        // ask first
        if (apps.length == 1) {
            newConfirm(_("Delete Appointment"),_("Are you sure you want to delete the selected item?"),AlertPopup.YESNO,null,null,deleteRoutine,null,null); /* i18n */
        } else {
            newConfirm(_("Delete Appointment"),_("Are you sure you want to delete the selected items?"),AlertPopup.YESNO,null,null,deleteRoutine,null,null); /* i18n */
        }
    });
}

// util
function getCumulativeOffset(elem) {
    var top = 0, left = 0;
    while (elem) {
      top += elem.offsetTop || 0;
      left += elem.offsetLeft || 0;
      elem = elem.offsetParent;
    }
    return { "top": top, "left": left };
}

function CalendarGrid() {
   
   this.domNode = null;
   this.statusValid = false;
   
   // rows
   this.rows = [];
   this.rowIndex = {};
   this.lastSelectedRowIndex = 0;
   this.autoSortEnabled = false;
   this.sortAsc = true;
   this.isSorted = false;
   
   // appointments
   this.appointments = {};
   // create/get selection for appointments
   this.appointmentSelection = getSimpleSelection("gridCalendarAppointments");
   this.appointmentSelection.setChangedEvent("OX_Calendar_Appointment_Selected");
   
   // grid & day interval
   this.gridIntervalStart = 0;
   this.gridIntervalEnd = 0;
   this.gridIntervalLength = 28;
   this.dayIntervalStart = 0;
   this.dayIntervalEnd = 23;
   this.workdayStart = configGetKey("gui.calendar.starttime") || 8;
   this.workdayEnd = configGetKey("gui.calendar.endtime") || 17;
   this.ONE_HOUR = 1000 * 60 * 60;
   this.ONE_DAY = this.ONE_HOUR * 24;
   this.ONE_WEEK = this.ONE_DAY * 7;
   // initialize date
   this.setDayInterval();
   this.setGridInterval();
   this.setGridIntervalLength(28); // 4 weeks
   
   // zoom level
   this.zoom = 100;
   
   // title bar, container, controlBar, autoComplete
   this.titleBar = newnode("div", {
       position: "absolute",
       top: "0px", right: "0px", left: "0px", height: "30px"
   }, { className: "titleBar" });
   this.container = newnode("div", {
       position: "absolute", 
       top: "30px", right: "0px", bottom: "30px", left: "0px" 
   });
   this.dataGrid = null;
   this.controlBar = newnode("div", {
       position: "absolute", 
       right: "0px", bottom: "0px", left: "0px", height: "30px",
       lineHeight: "30px"
   });
   this.autoCompleteContainer = newnode("div", {
       position: "absolute", top: "0px", left: "0px", width: "600px", height: "70px", zIndex: 1000,
       visibility: "hidden", backgroundColor: "white", border: "1px solid #aaa", fontSize: "8pt",
       overflow: "auto", overflowX: "hidden", overflowY: "auto", MozBoxShadow: "#333 5px 5px 8px"
   });
   document.body.appendChild(this.autoCompleteContainer);
   
   // participants dialog
   this.participantsDialog = null;
   this.currentTeam = null;
   
   // selection
   this.rowSelection = {};
   
   // events
   this.selectionChangedEvent = null;
   this.teamMemberChangedEvent = null;
   this.zoomLevelChangedEvent = null;
   
   // status information
   this.firstRun = true;
   this.manualScroll = false;
   this.showDetails = true;
}

/* imports current team of team week (if exists) or loads the default team */
CalendarGrid.prototype.importTeam = function() {
    // cooperate with week view - any teams out there?
    if (hasTeamMembers()) {
        var Self = this, previousTeam = this.currentTeam;
        this.loadTeam(getTeamArray(), function() {
            // reset current team
            Self.currentTeam = previousTeam;
        });
    } else {
        this.loadDefaultTeam();
    }
}

CalendarGrid.prototype.setDayInterval = function(hourStart, hourEnd) {
    if (!isNaN(hourStart) && !isNaN(hourEnd) && hourStart < hourEnd) {
        this.dayIntervalStart = hourStart;
        this.dayIntervalEnd = hourEnd;
    } else {
        this.dayIntervalStart = 0
        this.dayIntervalEnd = 23;
    }
}

CalendarGrid.prototype.scroll2Date = function(date, roundTo) {
    // adjust parameter
    var roundTo = typeof(roundTo) == "undefined" ? this.ONE_HOUR : roundTo;
    // inside time interval?
    var t = Math.floor(date.getTime()/roundTo)*roundTo;
    if (this.insideGridInterval(t)) {
        var x = this.time2pixel(t);
        this.dataGridContainer.scrollLeft = x;
        this.gridHeader.scrollLeft = x;
        this.lastScrollLeft = x;
    }
}

CalendarGrid.prototype.insideGridInterval = function(timestamp) {
    return timestamp >= this.gridIntervalStart.getTime() && timestamp <= this.gridIntervalEnd.getTime();
}

CalendarGrid.prototype.setGridIntervalToWeek = function(timestamp, intervalLength) {
    // timestamp?
    timestamp = timestamp || this.gridIntervalStart.getTime() || (new Date()).getTime();
    // round down to days
    var t = Math.floor(timestamp/this.ONE_DAY)*this.ONE_DAY;
    // check week day
    var w = (new Date(t)).getUTCDay() - 1; // sub 1 since it starts with Sunday (=0)
    // subtract days (so that t points at Monday morning) & create date
    var d = new Date(t - w * this.ONE_DAY);
    // set interval (default length = 4 weeks)
    this.setGridInterval(d.getUTCFullYear(), d.getUTCMonth(), d.getUTCDate(), intervalLength || this.gridIntervalLength);
}

CalendarGrid.prototype.setGridIntervalLength = function(intervalLength) {
    // new value?
    if (this.gridIntervalLength != intervalLength) {
        this.gridIntervalLength = intervalLength;
        if (this.gridIntervalStart) {
            var d = this.gridIntervalStart;
            this.setGridInterval(d.getUTCFullYear(), d.getUTCMonth(), d.getUTCDate());
        }
    }
}

CalendarGrid.prototype.setGridInterval = function(UTCyear, UTCmonth, UTCday, intervalLength) {
    // no date given?
    var year = UTCyear == null || isNaN(UTCyear) ? activeYear : UTCyear;
    var month = UTCmonth == null || isNaN(UTCmonth) ? activeMonth : UTCmonth;
    var day = UTCday == null || isNaN(UTCday) ? activeDay : UTCday;
    // set date
    var utc_start = Date.UTC(year, month, day, this.dayIntervalStart, 0, 0);
    var maxDay = intervalLength || this.gridIntervalLength || this.getMonthLength(year, month);
    this.gridIntervalStart = new Date(utc_start);
    var utc_end = utc_start + Math.max(0, maxDay - 1) * this.ONE_DAY + (this.dayIntervalEnd-this.dayIntervalStart+1) * this.ONE_HOUR - 1;
    this.gridIntervalEnd = new Date(utc_end);
    // invalidate scrolling positions
    this.invalidScrollPositions = true;
    // invalidate appointments
    this.invalidateAppointments();
}

CalendarGrid.prototype.invalidateAppointments = function() {
    // clear selection
    this.appointmentSelection.clear();
    // clear appointments
    this.appointments = {};
};

CalendarGrid.prototype.getAppointments = function(callback) {
    
    // create multiple requests
    var multiple = { requests: [], objects: [] };
    var skipRequest = true;
    
    // get timestamps
    var start = this.gridIntervalStart.getTime();
    var end = this.gridIntervalEnd.getTime();
    
    // loop members
    for (var i = 0; i < this.rows.length; i++) {
        // get row data
        var data = this.rows[i].data;
        // prepare request object
        var req = {};
        req.module = "calendar";
        req.action = "freebusy";
        req.id = data.id;
        req.type = data.type; // 1 = user; 3 = resource
        req.start = start;
        req.end = end;
        // add to request list
        multiple.requests.push(req);
        // appointments already loaded?
        var rowId = data.id + ":" + data.type;
        skipRequest = skipRequest && typeof(this.appointments[rowId]) != "undefined";
    }
    
    // send server request
    if (this.rows.length > 0 && !skipRequest) {
        var Self = this;
        json.put(AjaxRoot + "/multiple?session=" + session, multiple.requests, null, function(responses) {
        	var tFolders = { };
        	// to prevent an empty folder array later we always add the users default calendar
        	tFolders[configGetKey("folder.calendar")] = configGetKey("folder.calendar");
            if (responses && responses.length) {
                // loop responses
                for (var r = 0; r < responses.length; r++) {
                    var response = responses[r].data
                    var row = Self.rows[r];
                    var id = row.rowId;
                    // not yet loaded?
                    if (!Self.appointments[id]) {
                        // create new list for this id
                        Self.appointments[id] = [];
                        // loop data
                        for (var i = 0; i < response.length; i++) {
                            var app = response[i];
                            // tidy up recurrence appointments
                            Self.tidyUpAppointment(app);
                            // add
                            Self.appointments[id].push(app);
                            // store folder_id in hash to check permissions later
                            if (app.folder_id && !tFolders[app.folder_id]) {
                            	tFolders[app.folder_id] = app.folder_id;
                            }
                            // folder_id vs. folder
                            app.folder = app.folder_id;
                        }
                    }
                }
                // now we have the relevant appointments for each row.
                // we can get the appointments of a row by looking up its id
                // and then using this id to find the appointments in the
                // variable "appointments"
            }
            
            // convert folder hash to array
            var folders = [];
            for (var i in tFolders) {
            	folders.push(tFolders[i]);
            }
            // fetch folder permissions
            // @todo: check the response! it looks like the last folder in
            // the array is always undefined! likely a folder cache bug as the
            // server response seems  to be ok.
            oMainFolderTree.fetchAndGetFolders(folders, 
            		function(result) {
		            	// callback?
		                if (callback) callback(result);
            		} 
            );
        });
    } else {
        // callback?
        if (callback) callback();
    }
}

CalendarGrid.prototype.tidyUpAppointment = function(app) {
    // position?
    if (typeof(app.recurrence_position) == "undefined") {
        app.recurrence_position = 0;
    }
    // recurrence?
    if (typeof(app.recurrence_id) == "undefined") {
        app.recurrence_id = app.id;
    }
    // add alias of folder_id
    if (typeof(app.folder_id) == "undefined") {
        app.folder = app.folder_id;
    }
}

CalendarGrid.prototype.triggerTeamMemberChanged = function() {
    // event defined?
    if (this.teamMemberChangedEvent) {
        // patch: cooperate with week view
        importTeam(this.getRowHash());
        // trigger (selected or all)
        triggerEvent(this.teamMemberChangedEvent, this.getSelectedRowsOrAll());
    }
}

CalendarGrid.prototype.add = function(row) {
    if (!this.hasRow(row)) {
        // add to list
        this.rowIndex[row.rowId] = row.index = this.rows.length;
        this.rows.push(row);
        // sort rows
        this.autoSort();
        // invalidate current team
        this.currentTeam = null;
    }
    // trigger event
    this.triggerTeamMemberChanged();
}

CalendarGrid.prototype.insertAt = function(row, index) {
    if (!this.hasRow(row)) {
        // adjust
        index = Math.max(0, Math.min(index, this.rows.length));
        // add to list and update index
        this.rows.splice(index, 0, row);
        this.updateRowIndex();
        // sort rows
        this.autoSort();
        // invalidate current team
        this.currentTeam = null;
    }
    // trigger event
    this.triggerTeamMemberChanged();
}

CalendarGrid.prototype.hasRow = function(row) {
    return typeof(this.rowIndex[row.rowId]) != "undefined";
}

CalendarGrid.prototype.getRowHash = function() {
    var hash = {};
    for (var i = 0; i < this.numRows(); i++) {
        var row = this.rows[i];
        hash[row.rowId] = row;
    }
    return hash;
}

CalendarGrid.prototype.makeFunky = function(hash) {
    var list = [];
    for (var id in hash) {
        var row = hash[id];
        // some storage style? don't ask me
        list.push([0, 0, row.display_name, 0, row.id, 0, 0, row.type]);
    }
    return list;
}

CalendarGrid.prototype.createAppointment = function(startDate, endDate, members) {
    // is all day?
    var allDay = startDate.getTime() < endDate.getTime() && startDate.getUTCHours() == 0 && endDate.getUTCHours() == 0;
    // create date (activefolder is global)
    calendar_createNewAppointment(startDate, activefolder, allDay, {
        "data": this.makeFunky(members), "module": "contacts"
    }, endDate);
}

CalendarGrid.prototype.createAppointmentForSelectedMembers = function(startDate, endDate) {
    // get selected members (or all)
    var rows = this.numSelectedRows() > 0 ? this.getSelectedHash() : this.getRowHash();
    this.createAppointment(startDate, endDate, rows);
}

CalendarGrid.prototype.autoSort = function() {
    if (this.autoSortEnabled) {
        this.sort();
    }
}

CalendarGrid.prototype.sort = function() {
    // sort rows
    var Self = this;
    this.rows.sort(function() {
        // bind to "this"
        return CalendarGrid.prototype.rowSorter.apply(Self, arguments);
    });
    this.isSorted = true;
    // update row index (required after sort)
    this.updateRowIndex();
    // trigger event
    this.triggerTeamMemberChanged();
}

CalendarGrid.prototype.updateRowIndex = function() {
    for (var i = 0; i < this.numRows(); i++) {
        var row = this.rows[i];
        this.rowIndex[row.rowId] = row.index = i;
    }
}

CalendarGrid.prototype.addMultiple = function(list) {
    // loop through list
    for (var i = 0; i < list.length; i++) {
        var row = list[i];
        if (!this.hasRow(row)) {
            // add to list
            this.rows.push(row);
            // invalidate current team
            this.currentTeam = null;
        }
    }
    // sort rows
    this.autoSort();
    // trigger event
    this.triggerTeamMemberChanged();
}

CalendarGrid.prototype.rowSorter = function(a, b) {
    var v = false;
    if (a.type < b.type) {
        v = false;
    } else if (a.type > b.type) {
        v = true;
    } else {
        v = a.display_name.toLowerCase() >= b.display_name.toLowerCase();
    }
    return (this.sortAsc ? v : !v) ? 1 : -1; // Safari needs -1!
}

CalendarGrid.prototype.loadDefaultTeam = function() {
    // exist in configuration?
    if (configContainsKey("gui.calendar.teams")) {
        var teams = configGetKey("gui.calendar.teams");
        // loop teams
        for (var i = 0; i < teams.length; i++) {
            // marked as default?
            if (teams[i].team_default) {
                // load team
                var Self = this;
                this.loadTeam(teams[i].children, function() {
                    Self.currentTeam = i;
                });
                break;
            }
        }
    }
}

CalendarGrid.prototype.loadTeamByIndex = function(index, callback) {
    // exist in configuration?
    if (configContainsKey("gui.calendar.teams")) {
        var teams = configGetKey("gui.calendar.teams");
        // loop teams
        for (var i = 0; i < teams.length; i++) {
            if (i == index) {
                // load team
                var Self = this;
                this.loadTeam(teams[i].children, function() {
                    Self.currentTeam = i;
                    if (callback) callback();
                });
                break;
            }
        }
    }
}

CalendarGrid.prototype.loadTeam = function(team, callback) {
    // remove all
    this.removeAll();
    // load/add team
    this.addTeam(team, callback);
}

CalendarGrid.prototype.addTeam = function(team, callback) {
    
    var Self = this;
    
    // clear
    Self.clearPartial();
    
    // get team members
    var groups = [];
    for (var i = 0; i < team.length; i++) {
        var member = team[i];
        // handle different types
        switch (member.type) {
            case 1: // user
            case 3: // resource
                var row = new CalendarGridRow(member.type, member.id, member.display_name);
                row.data = member;
                this.add(row);
                break;
            case 2: // group
                groups.push(member);
        }
    }

    // local function
    var CG_localCallback = function() {
        // sort rows
        Self.autoSort();
        // lazy update
        setTimeout(function() { Self.update(); }, 50);
        // trigger event
        Self.triggerTeamMemberChanged();
        // callback?
        if (callback) callback();
    };

    var CG_resolvedGroups = function(groups) {
        // "groups" is a hash, so...
        var ids = [];
        for (var id in groups) {
            var group = groups[id];
            // loop through members (also a hash)
            for (var memberId in group.members) {
                ids.push(group.members[memberId]);
            }
        }
        if (ids.length > 0) {
            // ask cache about these users (async)
            internalCache.getUsers(ids, CG_resolvedUsers);
        } else {
            CG_localCallback();
        }
    };
    
    var CG_resolvedUsers = function(users) {
        var rows = [];
        for (var i in users) {
            var user = users[i];
            // add row
            user.type = 1;
            var row = new CalendarGridRow(user.type, user.id, user.display_name);
            row.data = user;
            rows.push(row);
        }
        Self.addMultiple(rows);
        CG_localCallback();
    }
    
    // groups to resolve?
    if (groups.length > 0) {
        // ask cache about these groups (async)
        internalCache.getObjects(groups, CG_resolvedGroups);
    } else {
        CG_localCallback();
    }
}

CalendarGrid.prototype.removeAll = function() {
    // delete rows
    this.rows = [];
    // clear index
    this.rowIndex = {};
    // clear selection
    this.rowSelection = {};
    // invalidate current team
    this.currentTeam = null;
    // trigger selection event?
    if (this.selectionChangedEvent) {
        triggerEvent(this.selectionChangedEvent, this.numSelectedRows());
    }
    // trigger changed event
    this.triggerTeamMemberChanged();
}

CalendarGrid.prototype.removeById = function(id) {
    // find row
    for (var i = 0; i < this.rows.length; i++) {
        if (this.rows[i].rowId == id) {
            this.removeAt(i);
            break;
        }
    }
}

CalendarGrid.prototype.removeAt = function(index) {
    // get row
    var row = this.rows[index];
    // delete from list
    delete this.rowIndex[row.rowId];
    this.rows.splice(index, 1);
    // invalidate current team
    this.currentTeam = null;
    // delete from selection
    for (var id in this.rowSelection[row.rowId]) {
        if (id == row.rowId) {
            delete this.rowSelection[id];
            // trigger event?
            if (this.selectionChangedEvent) {
                triggerEvent(this.selectionChangedEvent, this.numSelectedRows());
            }
        }
    }
    // trigger changed event
    this.triggerTeamMemberChanged();
}

CalendarGrid.prototype.removeSelectedRows = function(index) {
    var i = 0;
    while (i < this.rows.length) {
        var row = this.rows[i];
        // is selected?
        if (this.rowSelection[row.rowId]) {
            this.rows.splice(i, 1);
            delete this.rowIndex[row.rowId];
            delete this.rowSelection[row.rowId];
            // invalidate current team
            this.currentTeam = null;
            continue;
        }
        i++;
    }
    // trigger event?
    if (this.selectionChangedEvent) {
        triggerEvent(this.selectionChangedEvent, this.numSelectedRows());
    }
    // trigger changed event
    this.triggerTeamMemberChanged();
}

CalendarGrid.prototype.clickRow = function(index, e) {
    // event given?
    e = e || { shiftKey: false, ctrlKey: false };
    // shift?
    if (e.shiftKey && this.numSelectedRows() > 0) {
    	// holds shift!
    	this.selectNone();
    	this.selectRows(this.lastSelectedRowIndex, index);
    } else if (this.rows[index]) {
    	// multiple?
        if (!e.ctrlKey) {
            this.selectNone();
        }
    	if (this.rowSelection[this.rows[index].rowId]) {
	    	// <ctrl> and click on selected row
	    	var row = this.rows[index];
	    	row.node.className = "member";
	        delete this.rowSelection[row.rowId];
	    } else {
	    	this.selectRows(index, index);
	    }
    	this.lastSelectedRowIndex = index;
    }
}

CalendarGrid.prototype.selectRows = function(start, end) {
    if (start > end) {
        var tmp = end; end = start; start = tmp;
    }
	for (var i = start; i <= end; i++) {
		var row = this.rows[i];
		row.node.className = "member rowSelected";
		this.rowSelection[row.rowId] = row;
	}
	
	// trigger event?
    if (this.selectionChangedEvent) {
        triggerEvent(this.selectionChangedEvent, this.numSelectedRows());
    }
    // trigger changed event
    this.triggerTeamMemberChanged();
}

CalendarGrid.prototype.selectNone = function() {
    for (var id in this.rowSelection) {
        this.rowSelection[id].node.className = "member";
        delete this.rowSelection[id];
    }
    // trigger changed event
    this.triggerTeamMemberChanged();
}

CalendarGrid.prototype.numSelectedRows = function(row) {
    var count = 0;
    for (var id in this.rowSelection) count++;
    return count;
}

CalendarGrid.prototype.setSelectionChangedEvent = function(eventName) {
    this.selectionChangedEvent = eventName;
}

CalendarGrid.prototype.setTeamMemberChangedEvent = function(eventName) {
    var oldEvent = this.teamMemberChangedEvent;
    this.teamMemberChangedEvent = eventName;
    // fire now?
    if (oldEvent != eventName) {
        this.triggerTeamMemberChanged();
    }
}

CalendarGrid.prototype.numRows = function(row) {
    return this.rows.length;
}

CalendarGrid.prototype.getSelectedHash = function() {
    return this.rowSelection;
}

CalendarGrid.prototype.getSelectedRows = function() {
    var selection = [];
    for (var id in this.rowSelection) {
        selection.push(this.rowSelection[id]);
    }
    return selection;
}

CalendarGrid.prototype.getSelectedRowsOrAll = function() {
    return this.numSelectedRows() > 0 ? this.getSelectedRows() : this.rows;
}

CalendarGrid.prototype.getSelectedIndexes = function() {
    var selection = [];
    for (var id in this.rowSelection) {
        selection.push(this.rowIndex[id]);
    }
    return selection;
}

CalendarGrid.prototype.getSelectedIDs = function() {
    var selection = [];
    for (var id in this.rowSelection) {
        selection.push(id);
    }
    return selection;
}

CalendarGrid.prototype.setParentDOMNode = function(node) {
    // set node
    this.domNode = node;
    // set class
    this.domNode.className += " calendarGrid";
    // add children
    node.appendChild(this.titleBar);
    node.appendChild(this.container);
    node.appendChild(this.controlBar);
    // prevent selection
    node.onselectstart = function() { return false; }
    node.style.MozUserSelect = "none";
}

CalendarGrid.prototype.validate = function() {
    if (!this.statusValid && this.domNode != null) {
        this.draw();
        this.statusValid = true;
    }
}

CalendarGrid.prototype.update = function() {
	if (this.domNode != null) {
		this.drawPartial();
	}
}

CalendarGrid.prototype.validateOrUpdate = function() {
	if (this.domNode != null) {
		if (!this.statusValid) {
			this.validate();
		} else {
			this.update();
		}
	}
}

CalendarGrid.prototype.getMonthLength = function(year, month) {
    var d = Date.UTC(year, month+1, 1);
    d -= this.ONE_DAY;
    d = new Date(d);
    return d.getUTCDate();
}

CalendarGrid.prototype.moveInterval = function(type, steps) {
    // get current interval start
    var start = new Date(this.gridIntervalStart);
    var year = start.getUTCFullYear();
    var month = start.getUTCMonth();
    var day = start.getUTCDate();
    // get sign
    var sign = steps/Math.abs(steps);
    // iterate
    for (var i = 0; i < Math.abs(steps); i++) {
        // consider type
        switch (type) {
            case "MONTH":
                month += sign;
                break;
            case "WEEK":
                day += sign * 7;
                break;
            case "DAY":
                day += sign;
                break;
        }
    }
    // update grid interval
    this.setGridInterval(year, month, day);
    // update grid
    this.clearPartial();
    this.update();
}

CalendarGrid.prototype.draw = function() {

    var Self = this;
    
    // title
    var arrowLeft = newnode("span", { cursor: "pointer" }, null, [
        newtext(" "),
        newnode("img", { 
            width: "4px", height: "7px", margin: "0px 5px 0px 20px"
        }, {
            src: getFullImgSrc("img/arrows/arrow_darkgrey_left.gif"), alt: ""
        }),
        newtext(" ")
    ]);
    addDOMEvent(arrowLeft, "click", function(e) {
        // change interval
        Self.moveInterval("DAY", -Self.gridIntervalLength);
        // update calendar
        activeYear = Self.gridIntervalStart.getUTCFullYear();
        activeMonth = Self.gridIntervalStart.getUTCMonth();
        activeDay = Self.gridIntervalStart.getUTCDate();
        if (oMiniCalendar) oMiniCalendar.setSelectedByDate(activeYear, activeMonth, activeDay);
    });
    this.titleBar.appendChild(arrowLeft);
    this.titleDates = newtext("");
    this.drawDateInterval();
    this.titleBar.appendChild(this.titleDates);
    // right arrow
    var arrowRight = newnode("span", { cursor: "pointer" }, null, [
        newtext(" "),
        newnode("img", { 
            width: "4px", height: "7px", margin: "0px 20px 0px 5px"
        }, {
            src: getFullImgSrc("img/arrows/arrow_darkgrey_right.gif"), alt: ""
        }),
        newtext(" ")
    ]);
    addDOMEvent(arrowRight, "click", function(e) {
        // change interval
        Self.moveInterval("DAY", +Self.gridIntervalLength);
        // update calendar
        activeYear = Self.gridIntervalStart.getUTCFullYear();
        activeMonth = Self.gridIntervalStart.getUTCMonth();
        activeDay = Self.gridIntervalStart.getUTCDate();
        if (oMiniCalendar) oMiniCalendar.setSelectedByDate(activeYear, activeMonth, activeDay);
    });
    this.titleBar.appendChild(arrowRight);

    // --------------------------------------------------------------------------
    
    // container
    this.drawPartial();
    
    // --------------------------------------------------------------------------
    
    // controls
    // 1. slider
    this.zoomSlider = newnode("div", null,
            { className: "zoomSlider" }
    );
    this.zoomLevel = newtext(Self.zoom + " %");
    var zoomLevelBox = newnode("span", {
        fontSize: "8pt", color: "white"
        }, null //, [this.zoomLevel]
    );
    this.zoomSliderBox = newnode("div", null,
            { className: "zoomSliderBox" }, [this.zoomSlider, zoomLevelBox]
    );
    // 2. combo box
    this.zoomCombo = newnode("select", null, { size: 1 });
    // add options
    var zoomLevels = ["","10","50","75","100","200","400","700","1000"];
    for (var i = 0; i < zoomLevels.length; i++) {
        var text = newtext(zoomLevels[i] ? zoomLevels[i] + " %" : "");
        var option = newnode("option", null, { value: zoomLevels[i] }, [text]);
        this.zoomCombo.appendChild(option);
    }
    var zoomComboBox = newnode("div", null,
            { className: "zoomComboBox" }, [this.zoomCombo]
    );
    // add listener to combo box
    addDOMEvent(this.zoomCombo, "mousedown", function() {
        Self.zoomCombo.firstChild.innerHTML = "";
        Self.zoomCombo.firstChild.value = "";
    });
    addDOMEvent(this.zoomCombo, "change", function() {
        var v = Self.zoomCombo.value;
        if (v) {
            Self.zoomCombo.firstChild.innerHTML = "";
            Self.zoomCombo.firstChild.value = "";
            // initialize scroll/zoom fix
            //Self.scrollPositionFix.style.left = Self.roundEm(Self.dataGridContainer.scrollLeft/Self.zoom) + 1 + "em";
            // change zoom level
            Self.changeZoomLevel(v);
        }
    });
    // add controls
    this.controlBar.appendChild(this.zoomSliderBox);
    this.controlBar.appendChild(zoomComboBox);
    
    // update zoom level
    this.updateZoomControls(this.zoom);
    
    // add listener
    this.zooming = false;
    this.zoomSliderLeft = 0;
    this.zoomTimeout = null;
    this.lastZoom = this.zoom;
    addDOMEvent(this.zoomSlider, "mousedown", CalendarGrid_startSlider);
    addDOMEvent(this.zoomSliderBox, "mousedown", CalendarGrid_startSlider);
    function CalendarGrid_startSlider(e) {
        // hide appointments
        try {
            Self.dataGrid.removeChild(Self.appContainer);
        } catch(e) { }
        // remove scroll handler
        removeDOMEvent(Self.dataGridContainer, "scroll", Self.scrollHandler);
        // start zooming
        Self.zooming = true;
        Self.zoomSliderLeft = getCumulativeOffset(Self.zoomSliderBox).left + 10;
        // make first move
        addDOMEvent(document, "mousemove", CalendarGrid_moveSlider);
        CalendarGrid_moveSlider(e);
    };
    addDOMEvent(document, "mouseup", function() {
        if (Self.zooming) {
            // stop zooming
            Self.zooming = false;
            window.setTimeout(function() {
                try {
                    // show appointments
                    Self.dataGrid.appendChild(Self.appContainer);
                    removeDOMEvent(document, "mousemove", CalendarGrid_moveSlider);
                    // add scroll handler
                    addDOMEvent(Self.dataGridContainer, "scroll", Self.scrollHandler);
                } catch(e) { }
            }, 100);
        }
    });
    function CalendarGrid_moveSlider(e) {
        if (Self.zooming) {
            var x = e.clientX - Self.zoomSliderLeft;
            x = Math.max(0, Math.min(180, x)) + 10;
            // snap to grid (10 pixel)
            x = Math.round(x/10) * 10;
            // get zoom level
            var z = x <= 100 ? x : 100 + (x-100) * 10;
            if (!isNaN(z) && z != Self.lastZoom) {
                Self.changeZoomLevel(z);
            }
        }
    }
    
    // 3. "Details" checkbox
    var showDetailsBox = newnode("input", { border: "0px none" }, {
        type: "checkbox", value: "1", defaultChecked: "checked", checked: "checked", id: "calendarGridShowDetails"
    });
    var showDetailsLabel = newnode("label", 0, 0, [newtext(_("Details"))]);
    showDetailsLabel.setAttribute("for", "calendarGridShowDetails");
    var showDetailsDiv = newnode("div", {
        position: "absolute", left: "200px", top: "2px"
    }, 0, [showDetailsBox, showDetailsLabel]);
    // add handler
    if (IE) {
		// IE checkbox fix
		var detailHandlerIE = function() {
			Self.showDetails = showDetailsBox.checked;
			Self.clearPartial();
			Self.update();
		};
		addDOMEvent(showDetailsBox, "click", detailHandlerIE);
	} else {
		var detailHandler = function() {
	    	Self.showDetails = showDetailsBox.checked;
	    	if (!Self.showDetails) {
	    		Self.dataGridContainer.className += " bar";
	    	} else {
	    		Self.dataGridContainer.className = Self.dataGridContainer.className.replace(/\s*bar/g, '');
	    	}
	    };
		addDOMEvent(showDetailsBox, "change", detailHandler);
	}
    this.controlBar.appendChild(showDetailsDiv);
}

CalendarGrid.prototype.changeZoomLevel = function(z) {
    // check value
    z = Math.max(10, Math.min(1000, z));
    // set
    this.zoom = z;
    this.lastZoom = z;
    var Self = this;
    // speed!
    window.setTimeout(function() { Self.updateZoomControls(Self.zoom); }, 0);
    window.clearTimeout(this.zoomTimeout);
    this.zoomTimeout = window.setTimeout(function() {
        // new font size
        var fs = Self.zoom + "px";
        // set new zoom level
        Self.dataGrid.style.fontSize = fs;
        Self.gridHeader.style.fontSize = fs;
        // hide/show hours?
        Self.gridHeader.className = Self.zoom >= CG_HEAD_ZOOM ? "gridHeader zoom1" : "gridHeader zoom0";
        // restore correct scroll position
        Self.dataGridContainer.scrollLeft = Self.scrollPositionFix.offsetLeft;
        Self.gridHeader.scrollLeft = Self.scrollPositionFix.offsetLeft;
        // trigger event?
        if (Self.zoomLevelChangedEvent) {
            triggerEvent(Self.zoomLevelChangedEvent, Self.zoom);
        }
    }, 250);
}

CalendarGrid.prototype.updateZoomControls = function(zoomLevel) {
    // get x
    var x = zoomLevel <= 100 ?
            zoomLevel : Math.round(zoomLevel/10) + 90;
    // update slider
    if (this.zoomSlider) {
        this.zoomSlider.style.left = (x-10) + "px";
    }
    // update combo box
    if (this.zoomCombo) {
        this.zoomCombo.firstChild.innerHTML = "";
        this.zoomCombo.firstChild.value = "";
        this.zoomCombo.value = this.zoom;
        // check
        if (this.zoomCombo.value != this.zoom) {
            this.zoomCombo.firstChild.innerHTML = this.zoom + " %";
            this.zoomCombo.firstChild.value = this.zoom;
            this.zoomCombo.value = this.zoom;
        }
    }
}

CalendarGrid.prototype.setZoomLevelChangedEvent = function(eventName) {
    this.zoomLevelChangedEvent = eventName;
}

CalendarGrid.prototype.snapX2grid = function(x, factor) {
    // snap  x position to grid (pixel; half hours / quarter hours)
    var factor = factor ? factor : (this.zoom <= 400 ? 2 : 4);
    var snapper = 100 * CG_PX2EM * this.zoom / 100 / 24 / factor;
    return Math.round(Math.round(x/snapper)*snapper);
}

CalendarGrid.prototype.time2pixel = function(t) {
    // convert into time (~1200 Pixel = 1 Day if zoom = 100%)
    var x = Math.round((t - this.gridIntervalStart.getTime()) * this.zoom * 100 * CG_PX2EM / this.ONE_DAY / 100 );
    return x;
}

CalendarGrid.prototype.pixel2time = function(x, snap2Grid) {
    // convert into time (1200 Pixel = 1 Day if zoom = 100%)
    var t = this.gridIntervalStart.getTime() + Math.round(this.ONE_DAY * x * 100 / this.zoom / 100 / CG_PX2EM);
    // snap to grid?
    if (snap2Grid) {
        // half hour / quarter hour
        var snapper = this.zoom <= 400 ? this.ONE_HOUR/2 : this.ONE_HOUR/4;
        t = Math.round(t/snapper)*snapper;
    }
    return t;
}

CalendarGrid.prototype.roundEm = function(em) {
    return Math.round(em*1000)/1000;
}

CalendarGrid.prototype.drawDateInterval = function() {
    var startDate = formatDate(this.gridIntervalStart, "date");
    var endDate = formatDate(this.gridIntervalEnd, "date");
    this.titleDates.nodeValue = "" +
        // start date
        startDate + " " +
        // start calendar week
        "(" + _("CW") + " " + formatDateTime("w", new Date(this.gridIntervalStart)) + ")" + /* i18n */
        // different?
        (startDate != endDate ?
            // dash
            " - " + 
            // end date
            endDate + " " +
            // end calendar week
            "(" + _("CW") + " " + formatDateTime("w", new Date(this.gridIntervalEnd)) + ")" /* i18n */
            :
            ""
        );
}

CalendarGrid.prototype.drawPartial = function() {
    
    var Self = this;
    
    // scroll position fix
    if (this.invalidScrollPositions) {
        this.scrollPositionFix = null;
        this.manualScroll = false;
    }
    
    // -------------------------------------------------------------------------
    
    // update title
    this.drawDateInterval();
    
    // -------------------------------------------------------------------------
    
    // current zoom -> pixel
    var fs = Math.round(this.zoom) + "px";
    
    // container
    this.dataGrid = newnode("div", { fontSize: fs }, { "className": "zoom0" });
    this.dataGridContainer = newnode("div", null, { 
        "className": "dataGrid" }, [this.dataGrid]
    );

    var maxRows = this.numRows();
    var maxCols = Math.ceil((this.gridIntervalEnd.getTime() - this.gridIntervalStart.getTime())/this.ONE_DAY);
    var hourStart = this.dayIntervalStart;
    var hourStop = this.dayIntervalEnd;
    var numHours = hourStop - hourStart + 1;
    var hourWidth = CG_PX2EM/numHours;
    
    // set data grid width
    this.dataGrid.style.width = (maxCols * numHours * hourWidth) + "em";
    
    // hours (vertical; one extra row for new members)
    var wdStart = this.workdayStart - 1, wdEnd = this.workdayEnd - 1;
    var hourZ = 0;
    for (var i = 0; i < maxCols; i++) {
        for (var h = hourStart; h <= hourStop; h++) {
            var hourClassName = "hour ";
            hourClassName += h < wdStart || h > wdEnd ? "dark " : "bright ";
            if (i != 0 || h != 0) {
                hourClassName += h == 0 ? "borderColorMidnight " : h < wdStart || h > wdEnd ? "borderColorDark " : "borderColorBright ";
                hourClassName += h == 0 ? "borderThick" : "borderThin";
            }
            var hour = newnode("div", {
                left: this.roundEm((i * CG_PX2EM) + ((h-hourStart) * hourWidth)) + "em",
                width: this.roundEm(hourWidth) + "em",
                height: ((maxRows+1) * CG_ROW_HEIGHT) + "px"
            }, { className: hourClassName }, [newnode("div", { fontSize: "10px" })]);
            this.dataGrid.appendChild(hour);
        }
    }
    // shadows (horizontal; one extra row for new members)
    for (var r = 0; r <= maxRows; r++) {
        if (r % 2 == 1) { // IE ||
            var shadowsPerRow = IE ? Math.ceil(maxCols/2) : 1; // IE opacity fix
            for (var i = 0; i < shadowsPerRow; i++) {
                var shadow = newnode("div", {
                    top: (r * CG_ROW_HEIGHT - 1) + "px",
                    left: this.roundEm(i * maxCols * CG_PX2EM / shadowsPerRow) + "em",
                    width: this.roundEm(maxCols * CG_PX2EM / shadowsPerRow) + "em",
                    backgroundColor: IE && false ? "transparent" : "black"
                }, { className: "rowShadow" }, [newnode("div", { fontSize: "10px" })]);
                this.dataGrid.appendChild(shadow);
            }
        }
    }
    var lastRow = newnode("div", {
        top: ((maxRows+1) * CG_ROW_HEIGHT - 1) + "px",
        width: (maxCols * CG_PX2EM) + "em"
    }, { className: "lastRow" }, [newnode("div", { fontSize: "10px" })]);
    this.dataGrid.appendChild(lastRow);
    
    //--------------------------------------------------------------------------
    
    // add scroll/zoom position fix    
    // this one is required to keep the horizontal scroll position during zoom
    var top = "0px", left = "0px";
    if (this.scrollPositionFix) {
        top = this.scrollPositionFix.style.top || "0px";
        left = this.scrollPositionFix.style.left || "0px";
    }
    this.scrollPositionFix = newnode("div", {
        position: "absolute", "top": top, height: "1px", "left": left,
        width: "1px", zIndex: 0
    });
    this.dataGrid.appendChild(this.scrollPositionFix);
    
    //--------------------------------------------------------------------------
    
    // draw header
    var leftHeaderText = newtext(_("Team Members")); /* i18n */
    var src = !this.isSorted ? "img/dummy.gif" :
        this.sortAsc ? "img/arrows/sort_up.gif" : "img/arrows/sort_down.gif";
    var leftHeaderSort = newnode("img",
        { position: "absolute", right: "2px", top: "8px", width: "16px", height: "16px", border: "0px none" },
        { "src": getFullImgSrc(src), alt: "" }
    );
    var src = this.numRows() > 0 ? "img/menu/remove_teammember.gif" : "img/menu/remove_teammember_d.gif";
    var leftHeaderRemove = newnode("img",
            { position: "absolute", left: "10px", top: "8px", width: "16px", height: "16px", border: "0px none" },
            { "src": getFullImgSrc(src), alt: "" }
        );
    var leftHeader = newnode("div", {
        position: "absolute", top: "0px", left: "0px", width: "163px", height: "30px",
        lineHeight: "30px", fontSize: "9pt", textAlign: "left", paddingLeft: "36px",
        borderRight: "1px solid #ccc", cursor: "pointer"
    }, null, [leftHeaderRemove, leftHeaderText, leftHeaderSort]);
    // add event (sort)
    addDOMEvent(leftHeader, "click", function() {
        // sort
        Self.sort();
        // change sort direction
        Self.sortAsc = !Self.sortAsc;
        // update
        Self.clearPartial();
        Self.update();
    });
    // add event (remove)
    addDOMEvent(leftHeaderRemove, "click", function(e) {
        // remove all members
        Self.removeAll();
        // update
        Self.clearPartial();
        Self.update();
        stopEvent(e);
    });
    this.gridHeader = newnode("div", {
        position: "absolute", top: "0px", left: "200px", right: "0px", height: "30px",
        fontSize: fs, overflow: "hidden", cursor: "ew-resize"
    }, { className: "gridHeader" });
    var header = newnode("div", {
        position: "absolute", top: "0px", right: "0px", left: "0px", height: "28px",
        backgroundColor: "#eee", borderBottom: "1px solid #ccc", borderTop: "1px solid #ccc"
    }, null, [leftHeader, this.gridHeader]);
    
    //--------------------------------------------------------------------------
    
    // add days & hours to time grid header
    var baseTime = this.gridIntervalStart.getTime();
    for (var i = 0, hoursTotal = 0; i < maxCols; i++) {
        // create day
        var dayText = formatDate(baseTime+i*this.ONE_DAY, "dateshortday") + " " +
        	"(" + _("CW") + " " + formatDateTime("w", new Date(baseTime+i*this.ONE_DAY)) + ")"; /* i18n */
    	var day = newnode("div", {
            left: (i * CG_PX2EM) + "em",
            width: CG_PX2EM + "em"
        }, { className: "dayContainer" },
        	[ newnode("div", 0, {className : "day" }, [newtext(dayText)]) ]);
        this.gridHeader.appendChild(day);
        // add hours
        for (var h = hourStart; h <= hourStop; h++) {
            // create hour
            var left = (i * CG_PX2EM) + ((h-hourStart) * hourWidth) + "em";
            var hour = newnode("div", {
                "left": left, "width": hourWidth + "em"
            }, { className: "hourContainer", title: dayText },
            		[newnode("div", 0, {className: "hour"}, [newtext(formatDate(baseTime+i*this.ONE_DAY+h*this.ONE_HOUR, "time"))])]);
            this.gridHeader.appendChild(hour);
            hoursTotal++;
            // repeat current day (for higher zoom levels)
            if (h > hourStart + 3) {
            	var smallday = newnode("div", {
                    "left": left, width: hourWidth + "em"
                }, { className: "smallDayContainer" },
                	[newnode("div", 0, {className : "day"}, [newtext(formatDate(baseTime+i*this.ONE_DAY, "shortdate")) ])]);
                this.gridHeader.appendChild(smallday);
            }
        }
    }
    
    // add extra space on the very right to sync scrolling
    this.gridHeaderScrollBarFix = newnode("div", {
        position: "absolute", left: (hoursTotal * CG_PX2EM) + "em",
        width: "15px", height: "30px"
    });
    this.gridHeader.appendChild(this.gridHeaderScrollBarFix);
    
    // --------------------------------------------------------------------------
    
    // add "drag-scroll" (with "flick" gesture)
    this.gridHeaderScrolling = {
            on: false,
            start: 0,
            x: 0,
            lastX: 0,
            decel: 0, wait4decel: false,
            time: 0
    };
    addDOMEvent(this.gridHeader, "mousedown", function(e) {
        var ghs = Self.gridHeaderScrolling;
        ghs.x = Self.gridHeader.scrollLeft;
        ghs.start = e.clientX;
        ghs.decel = 0;
        ghs.wait4decel = false;
        ghs.on = true;
        ghs.time = (new Date()).getTime();
        // hide hours (since they make this terribly slow)
        Self.gridHeader.className = "gridHeader zoom0";
    });
    addDOMEvent(document, "mouseup", function(e) {
        var ghs = Self.gridHeaderScrolling;
        // turn scrolling off
        ghs.on = false;
        // get last scroll duration
        var duration = (new Date()).getTime() - ghs.time;
        // gesture?
        if (duration < 750 && ghs.wait4decel) {
            ghs.decel = 20 * (e.clientX - ghs.start > 0 ? +1 : -1);
            ghs.x = Self.dataGridContainer.scrollLeft;
            GridCalendarDecelerate();
        } else {
            // show hours
            Self.gridHeader.className = Self.zoom >= CG_HEAD_ZOOM ? "gridHeader zoom1" : "gridHeader zoom0";
        }
    });
    addDOMEvent(this.gridHeader, "mousemove", function(e) {
        var ghs = Self.gridHeaderScrolling;
        if (ghs.on) {
            var x = e.clientX - ghs.start;
            Self.dataGridContainer.scrollLeft = ghs.x - x;
            Self.gridHeader.scrollLeft = ghs.x - x;
            Self.manualScroll = true;
            ghs.wait4decel = true;
        }
    });
    function GridCalendarDecelerate() {
        var ghs = Self.gridHeaderScrolling;
        if (ghs.decel != 0) {
            ghs.x = Math.round(ghs.x-ghs.decel);
            Self.dataGridContainer.scrollLeft = ghs.x;
            Self.gridHeader.scrollLeft = ghs.x;
            ghs.decel += ghs.decel > 0 ? -0.25 : +0.25;
            setTimeout(function() { GridCalendarDecelerate(); }, 5);
        } else {
            // show hours
            Self.gridHeader.className = Self.zoom >= CG_HEAD_ZOOM ? "gridHeader zoom1" : "gridHeader zoom0";
        }
    }
    
    // --------------------------------------------------------------------------
    
    // draw left column container
    this.leftColumn = newnode("div", null, { className: "leftColumn" });
    this.leftColumnScrollBarFix = newnode("div", {
        position: "absolute", top: ((maxRows+1) * CG_ROW_HEIGHT) + "px", left: "0px", 
        width: "199px", height: "15px",
        zIndex: 0
    });
    this.leftColumn.appendChild(this.leftColumnScrollBarFix);
    
    // sync scrolling
    this.scrollSyncTimeout = null;
    this.scrollXAdjusted = false;
    this.scrollYAdjusted = false;
    // longer, "complicated" code for better scroll performance:
    this.scrollHandler = function() {

        // prevent double scrolling // following zooming
        if (Self.gridHeaderScrolling.on || Self.zooming) return;
        
        // update
        setTimeout(function() { Self.gridHeader.scrollLeft = Self.dataGridContainer.scrollLeft; }, 0);
        setTimeout(function() { Self.leftColumn.scrollTop = Self.dataGridContainer.scrollTop; }, 1);
        setTimeout(function() {
            Self.scrollPositionFix.style.left = Self.roundEm(Self.dataGridContainer.scrollLeft/Self.zoom) + "em";
            Self.scrollPositionFix.style.top = Self.dataGridContainer.scrollTop + "px";
        }, 2);
        
        // unimportant things
        setTimeout(function() {
            // remember if even scrolled
            Self.manualScroll = true;
            // adjustments
            if (!Self.scrollXAdjusted) {
                var clientHeightDiff = Self.leftColumn.clientHeight - Self.dataGridContainer.clientHeight;
                Self.gridHeaderScrollBarFix.style.width = clientHeightDiff + "px";
                Self.scrollXAdjusted = true;
            }
            if (!Self.scrollYAdjusted) {
                var clientHeightDiff = Self.leftColumn.clientHeight - Self.dataGridContainer.clientHeight;
                Self.leftColumnScrollBarFix.style.height = clientHeightDiff + "px";
                Self.scrollYAdjusted = true;
            }
        }, 10);
    };
    addDOMEvent(this.dataGridContainer, "scroll", this.scrollHandler);
    
    // --------------------------------------------------------------------------
    
    // draw members
    Self.memberDrag = {};
    for (var r = 0; r < maxRows; r++) {
        var row = this.rows[r];
        var className = "member" + (this.rowSelection[row.rowId] ? " rowSelected" : "");
        var div = newnode("div", 
            { top: (r * CG_ROW_HEIGHT) + "px" },
            { "className": className }
        );
        this.drawLeftColumn(row, r, div);
        this.leftColumn.appendChild(div);
        // remember node
        row.node = div;
        // add listener
        var wrapper = (function(Self, index) {
            return function(e) {
                Self.clickRow(index, e); 
            };
        })(Self, r);
        addDOMEvent(div, "mousedown", wrapper);
    }
    
    // --------------------------------------------------------------------------
    
    // clear
    this.container.innerHTML = "";
    // add
    this.container.appendChild(this.dataGridContainer);
    this.container.appendChild(this.leftColumn);
    this.container.appendChild(header);
    // restore zoom level
    this.gridHeader.className = this.zoom >= CG_HEAD_ZOOM ? "gridHeader zoom1" : "gridHeader zoom0";
    // restore scroll position
    this.dataGridContainer.scrollTop = Self.scrollPositionFix.offsetTop;
    this.dataGridContainer.scrollLeft = Self.scrollPositionFix.offsetLeft;
    this.gridHeader.scrollLeft = Self.scrollPositionFix.offsetLeft;
    this.invalidScrollPositions = false;
    
    //--------------------------------------------------------------------------
    
    // add "new" row (it might set the focus)
    var div = newnode("div", 
        { top: (maxRows * CG_ROW_HEIGHT) + "px" },
        { "className": "member new" }
    );
    this.leftColumn.appendChild(div);
    this.drawLeftNewColumn(div);
    addDOMEvent(div, "click", function() {
        Self.selectNone();
    });
    
    // add "drop zone" for members
    this.memberDropZone = newnode("div", 0, { className: "dropzone" }); 
    this.leftColumn.appendChild(this.memberDropZone);
    
    //--------------------------------------------------------------------------
    
    // add "new appointment" dragger
    this.newAppDragger = newnode("div", {
        position: "absolute", top: "0px", height: ((maxRows+1) * CG_ROW_HEIGHT) + "px",
        backgroundColor: "#555", opacity: "0.5", filter: "alpha(opacity=50)", zIndex: 20,
        visibility: "hidden"
    });
    this.dataGrid.appendChild(this.newAppDragger);
    
    // dragging logic
    this.newAppDrag = 0;
    this.newAppDragX = 0;
    this.newAppDragBase = 0;
    this.newAppDragTimeout = 0;
    addDOMEvent(this.dataGrid, "mousedown", function(e) {
        // ignore context menu
        if (e.button != 2) {
            // potential drag
            Self.newAppDrag = 1;
            // remember position
            Self.newAppDragBase = getCumulativeOffset(Self.dataGridContainer).left;
            Self.newAppDragX = Self.snapX2grid(e.clientX + Self.dataGridContainer.scrollLeft - Self.newAppDragBase);
        }
    });
    addDOMEvent(this.dataGrid, "mouseup", function(e) {
        // create appointment?
        if (Self.newAppDrag == 2) {
            // get positions
            var x1 = Self.newAppDragX;
            var x2 = Self.snapX2grid(e.clientX + Self.dataGridContainer.scrollLeft - Self.newAppDragBase);
            // swap?
            if (x1 > x2) {
            	var tmp = x1; x1 = x2; x2 = tmp;
            }
            if (x2 != x1) {
                var startDate = new Date(Self.pixel2time(x1, true));
                var endDate = new Date(Self.pixel2time(x2, true));
                Self.createAppointmentForSelectedMembers(startDate, endDate);
            }
        }
        // stop dragging
        Self.newAppDrag = 0;
        Self.newAppDragger.style.visibility = "hidden";
        Self.newAppDragger.style.left = "0px";
    });
    addDOMEvent(this.dataGrid, "mousemove", function(e) {
        if (Self.newAppDrag == 1) {
            // show new appointment dragger
            Self.newAppDragger.style.visibility = "visible";
            Self.newAppDrag = 2;
        }
        if (Self.newAppDrag == 2) {
            var x = Self.snapX2grid(e.clientX + Self.dataGridContainer.scrollLeft - Self.newAppDragBase);
            // set width
            var width = x-Self.newAppDragX;
            if (width > 0) {
            	Self.newAppDragger.style.left = Self.newAppDragX + "px";
            	Self.newAppDragger.style.width = Math.max(1, width) + "px";
            } else {
            	Self.newAppDragger.style.left = (Self.newAppDragX+width) + "px";
            	Self.newAppDragger.style.width = Math.max(1, -width) + "px";
            }
        }
    });
    
    
    
    // add mousewheel support
    // get proper event name for mousewheel
    var eventName = navigator.userAgent.indexOf('Gecko') > -1 && 
        navigator.userAgent.indexOf('KHTML') == -1 ? "DOMMouseScroll" : "mousewheel";
    // zoom via mouse wheel
    addDOMEvent(this.dataGridContainer, eventName, function(e) {
        e.delta = 0;
        e.deltaSign = 0;
        if (e.wheelDelta) {
            e.delta = e.wheelDelta;
        } else if (e.detail) {
            e.delta = -e.detail;
        }
        // signed delta
        if (e.delta != 0) {
            e.deltaSign = e.delta/Math.abs(e.delta);
        }
        // set new zoom level
        if (e.shiftKey) {
            Self.changeZoomLevel(Self.zoom + e.deltaSign * 20);
            stopEvent(e);
        } else {
            // always scroll horizontally (vertical scroll via mouse wheel is less important)
            var x = Self.snapX2grid(Self.dataGridContainer.scrollLeft, 1);
            x += Math.round(-e.deltaSign * 100 * CG_PX2EM * Self.zoom / 100 / 24);
            Self.dataGridContainer.scrollLeft = x;
            Self.gridHeader.scrollLeft = x;
            Self.manualScroll = true;
            stopEvent(e);
        }
    });
    
    // -------------------------------------------------------------------------
    
    // "double click to create appointment"
    addDOMEvent(this.dataGridContainer, "dblclick", function(e) {
        // get position
        var x = e.clientX + Self.dataGridContainer.scrollLeft - getCumulativeOffset(Self.dataGridContainer).left;
        var y = e.clientY + Self.dataGridContainer.scrollTop - getCumulativeOffset(Self.dataGridContainer).top;
        // convert pixel into time
        var t = Self.pixel2time(x, true);
        // get row index to get the member
        var r = Math.floor(y / CG_ROW_HEIGHT);
        // valid index?
        if (Self.rows[r]) {
            var row = Self.rows[r];
            // one member only
            var member = {}; member[row.rowId] = row;
            // adjust member selection
            Self.clickRow(r);
            // open "create appointment" dialog (duration: one hour)
            var now = (new Date(t)).getTime();
            var start = new Date(Math.floor(now/Self.ONE_HOUR)*Self.ONE_HOUR);
            var end = new Date(Math.ceil(now/Self.ONE_HOUR)*Self.ONE_HOUR);
            Self.createAppointment(start, end, member);
        }
    });
    
    // -------------------------------------------------------------------------
    
    // update controls
    this.updateZoomControls(this.zoom);
    
    // -------------------------------------------------------------------------
    
    // add appointment container
    this.appContainer = newnode("div", {
        position: "absolute", top: "0px", left: "0px",
        width: (maxCols * CG_PX2EM) + "em",
        height: (maxRows * CG_ROW_HEIGHT) + "px",
        zIndex: 10000
    });
    this.dataGrid.appendChild(this.appContainer);
    
    /* initialize hover */
    if (this.firstRun) {
        var hover = new Hover($("calendar_new_query"), OXAppointmentHover.getContent().node);
        hover.setSize(OXAppointmentHover.contentobject.node);
        /* event got triggered on each mouse move */
        hover.getTarget = function (node) {
            try {
                while (node) {
                    if (node.getAttribute("ox_object_id")) {
                        return node.parentNode ? node : null;
                    }
                    node = node.parentNode;
                }
            } catch (e) { /*see default implementation*/ }
        };
        /* event fired when getTarget found a node with valid attributes */
        hover.onShow = function (node) {
            OXAppointmentHover.actualHover = this;
            var ref       = node.getAttribute("ox_rowId");
            var folder_id = node.getAttribute("ox_folder_id");
            var object_id = node.getAttribute("ox_object_id");
            var rec_pos   = node.getAttribute("ox_rec_pos");
            if (object_id != undefined && folder_id != undefined) {
                // we have a folder and a object id so we can use the regular hover
                OXAppointmentHover.refillContent(object_id, folder_id, rec_pos);
            } else if (ref != undefined && Self.appointments && Self.appointments[ref]) {
                // we don't have read permission so we just show content we are allowed to
                var obj = { };
                for (var i in Self.appointments[ref]) {
                    if (object_id == Self.appointments[ref][i].id) {
                        obj = clone(Self.appointments[ref][i]);
                        break;
                    }
                }
                // disable tabs we don't need
                OXAppointmentHover.slider.getTabById("participants").show(false);
                OXAppointmentHover.slider.getTabById("attachments").show(false);
                OXAppointmentHover.slider.getTabById("others").show(false);
                OXAppointmentHover.refillContentByObject(obj);
            }
        }
        hover.onHide = function (node) {
          // re-enable tabs for later use
          OXAppointmentHover.slider.getTabById("participants").show(true);
          OXAppointmentHover.slider.getTabById("attachments").show(true);
          OXAppointmentHover.slider.getTabById("others").show(true);
        }
        calendarhovers["teamday"] = hover;
        // must be fired once here to enable it, later a change view does it for us
        calendarhovers["teamday"].enable();
        this.hoverInitialized = true;
    }
    
    // required to get the right class names for different appointment types
    var shownAs2Class = { "1": "reserved", "2": "temporary", "3": "absent", "4": "free" };
    
    // get and draw appointments (async.; should be the last thing to do here)
    this.getAppointments(function() { try {
        // get microseconds of start date
        var gridStart = Self.gridIntervalStart.getTime();
        var gridEnd = Self.gridIntervalEnd.getTime();
        // loop rows
        for (var r = 0; r < Self.numRows(); r++) {
            var row = Self.rows[r];
            // look up appointments
            var appointments = Self.appointments[row.rowId];
            var userId = row.id;
            // loop appointments
            for (var i = 0; i < appointments.length; i++) {
                var app = appointments[i];
                var canRead = typeof(app.folder_id) != "undefined";
                var canWrite = false;
                // check object write permissions
                if (canRead && oMainFolderTree.cache.find_folder(app.folder_id)) {
                	var permission = oMainFolderTree.cache.find_folder(app.folder_id).oxfolder.data.own_rights;
                	canWrite = computePerm(permission, 0) >= 2;
                	// @todo: i'm not sure if it's enough to check the own permissions. i assume it fails
                	// as soon as you are member of a group. in this case the permissions array has to 
                	// be checked -> similar to function 'permissionsToViewObjects'.
                }
                var innerDiv = null;
                if (!Self.showDetails) {
                    // bars only
                    var className = "appointment bar " + shownAs2Class[app.shown_as];
                    innerDiv = newnode("div", null, { "className": className });
                } else {
                    // show details
                    var wholeday = !!app.full_time;
                    // create inner div
                    var className = "appointment " + shownAs2Class[app.shown_as] +
                        // read permission?
                        (canWrite ? "" : " norights") +
                        // all day?
                        (wholeday ? " wholeday": "") +
                        // color label?
                        (app.color_label ? " hasColorLabel colorLabel"+app.color_label : "");
                    var appTitle = app.title || "\u2014"; // aka &mdash;
                    // is owner?
                    if (app.created_by == userId) {
                        appTitle += " \u2022"; // aka &bull;
                    }
                    innerDiv = newnode("div", null, { "className": className }, [
                        // add title
                        newnode("b", 0, 0, [newtext(appTitle)]),
                        // add time interval / wholeday
                        newnode("br"),
                        newtext(wholeday ? _("All day") : /* i18n */
                            formatDate(app.start_date, "time") + " - " +
                            formatDate(app.end_date, "time")
                        ),
                        newtext(", "), //newnode("br"),
                        // start date
                        newtext(formatDate(app.start_date, "dateday") +
                            // add end date?
                            (app.end_date-app.start_date > Self.ONE_DAY ? 
                            " - " + formatDate(app.end_date, "dateday") : "")
                        ),
                        // add blank line
                        newnode("br"),
                        newtext("\u00A0") // aka &nbsp;
                    ]);
                }
                // set required attributes to enable hovers
                if (canRead) {
                	innerDiv.setAttribute("ox_folder_id", app.folder_id);
                }
                innerDiv.setAttribute("ox_object_id", app.id);
                if (app.recurrence_position) {
                	innerDiv.setAttribute("ox_rec_pos", app.recurrence_position);
                }
                innerDiv.setAttribute("ox_rowId", row.rowId);
                // now convert microseconds to "em"!
                var top = (r * CG_ROW_HEIGHT + 5 + (wholeday ? 0 : -3));
                var left = ((app.start_date - gridStart) * CG_PX2EM / Self.ONE_DAY);
                var width = (((app.end_date < gridEnd ? app.end_date : gridEnd) - app.start_date) * CG_PX2EM / Self.ONE_DAY);
                var height =  (CG_ROW_HEIGHT - 10 - 1 + (wholeday ? 0 : 6));
                // create outer div
                var div = newnode("div", {
                    "position": "absolute", "top": top + "px", "left": Self.roundEm(left) + "em",
                    "width": Self.roundEm(width) + "em", "height": height + "px",
                    zIndex: (wholeday ? 10001 : 10002)
                }, null, [innerDiv]);
                // add to dom
                Self.appContainer.appendChild(div);
                // add listener
                if (canWrite) {
                    // observe by selection
                    var id = app.folder_id+"."+app.id+"."+app.recurrence_position+"@"+row.rowId;
                    Self.appointmentSelection.observe(innerDiv, id, app);
                    // add double click edit
                    var wrapper = (function(Self, app) {
                        return function(e) {
                            // copy
                            selectedAppointment = app;
                            stopEvent(e);
                            triggerEvent("OX_Calendar_Edit");
                        };
                    })(Self, app);
                    addDOMEvent(innerDiv, "dblclick", wrapper);
                    
                    // add context menu
                    addDOMEvent(innerDiv, "contextmenu", function(e) {
                        globalContextMenus.teamview.display(
                           e.clientX, e.clientY, Self.appointmentSelection.fetch()
                        );
                    });
                }
            }
        }
        
        // "deselect all" click
        addDOMEvent(Self.dataGridContainer, "click", function(e) {
            Self.appointmentSelection.clear();
            stopEvent(e);
        });
        
        // scroll to "now"
        if (!Self.manualScroll) {
            Self.scroll2Date(new Date(), Self.ONE_HOUR);
        }
        
        Self.firstRun = false;
        
    } catch(e) { if (console && console.error) console.error("Unexpected", e); }
    });
}

CalendarGrid.prototype.clearPartial = function() {
    // clear
    if (this.dataGrid && this.appContainer && this.leftColumn) {
        // hide first (responsiveness)
        this.dataGrid.style.visibility = "hidden";
        this.appContainer.style.visibility = "hidden";
        this.leftColumn.style.visibility = "hidden";
        // clear
        this.dataGrid.innerHTML = "";
        this.appContainer.innerHTML = "";
        this.leftColumn.innerHTML = "";
        // show
        this.dataGrid.style.visibility = "visible";
        this.appContainer.style.visibility = "visible";
        this.leftColumn.style.visibility = "visible";
    }
}

CalendarGrid.prototype.drawLeftColumn = function(row, index, parentNode) {
    
    var Self = this;
    
    // icon
    var src = "";
    switch (row.data.type) {
        default: /* no break */
        case 1: src = "img/calendar/user.gif"; break;
        case 3: src = "img/calendar/ressourcen.gif"; break;
    }
    var icon = newnode("img", 0, {
        src: getFullImgSrc(src), alt: ""
    });
    parentNode.appendChild(icon);
    
    // remove member "X"
    var remove = newnode("div", 0, { className: "memberX" }, [newtext("\u00d7")]); // &times;
    parentNode.appendChild(remove);
    addDOMEvent(remove, "click", function(e) {
        // trigger event
        // this row is already selected since the selection routine waits for mousedown
        triggerEvent('OX_Calendar_Teammember_Remove');
    });
    
    // text
    // remove surrounding quoting marks
    row.display_name = row.display_name.replace(/(^"|"$)/g, '');
    // add node
    parentNode.appendChild(newtext(row.display_name));
    
    // add dragger
    addDOMEvent(parentNode, "mousedown", function(e) {
        if (!Self.memberDrag.on) {
            // stand by
            Self.memberDrag.standBy = true;
            Self.memberDrag.startY = e.clientY - parentNode.offsetTop;
            // define listener
            Self.memberDrag.mover = function(e) {
                if (!Self.memberDrag.on && Self.memberDrag.standBy) {
                    Self.memberDrag.on = true;
                    Self.memberDrag.standBy = false;
                    // make clone
                    Self.memberDrag.clone = parentNode.cloneNode(false);
                    // adjust node design
                    parentNode.className += " drag";
                    // add clone
                    parentNode.parentNode.appendChild(Self.memberDrag.clone);
                    // adjust clone design
                    Self.memberDrag.clone.className += " clone";
                    // show drop zone
                    Self.memberDropZone.style.visibility = "visible";
                }
                if (Self.memberDrag.on) {
                    // move node
                    var y = Math.max(0, e.clientY - Self.memberDrag.startY);
                    parentNode.style.top = y + "px";
                    // move drop zone
                    var index = Math.min(Math.round(y/CG_ROW_HEIGHT), Self.numRows());
                    Self.memberDropZone.style.top = (index * CG_ROW_HEIGHT-2) + "px";
                }
            };
            // add event
            Self.memberDrag.stopper = handler;
            addDOMEvent(document, "mouseup", Self.memberDrag.stopper);
            addDOMEvent(document, "mousemove", Self.memberDrag.mover);
        }
    });
    var handler = function(e) {
        // get state
        var wasOn = Self.memberDrag.on;
        // remove event
        removeDOMEvent(document, "mouseup", Self.memberDrag.stopper);
        removeDOMEvent(document, "mousemove", Self.memberDrag.mover);
        // stop
        Self.memberDrag.on = false;
        Self.memberDrag.standBy = false;
        // hide drop zone
        Self.memberDropZone.style.visibility = "hidden";
        // get last position
        var y = Math.max(0, e.clientY - Self.memberDrag.startY);
        if (wasOn) {
            // determine indexes
            var oldIndex = row.index;
            var index = Math.min(Math.round(y/CG_ROW_HEIGHT), Self.numRows()+1);
            // anything to do?
            if (Math.abs(oldIndex-index) >= 1) {
                // remove row
                Self.removeById(row.rowId);
                // adjust index
                index = oldIndex < index ? index-1 : index;
                // insert row
                Self.insertAt(row, index);
            }
            // update
            Self.selectNone();
            Self.clearPartial();
            Self.update();
        }
    };
}

CalendarGrid.prototype.drawLeftNewColumn = function(parentNode) {
    
    var Self = this;
    
    // icon
    var icon = newnode("img", 0, {
        src: getFullImgSrc("img/menu/add_member_participant.gif"), alt: ""
    });
    parentNode.appendChild(icon);
    // open dialog when clicking the icon
    addDOMEvent(icon, "click", function(e) {
        triggerEvent('OX_Calendar_Teammember_Add');
    });
    
    // text
    var inputField = newnode("input",
        { width: "150px" }, { value: "" }
    );
    inputField.type = "text";
    parentNode.appendChild(inputField);
    
    // set focus if visible
    if (this.isShowing()) {
        inputField.focus();
        inputField.select();
        // better safe then sorry (IE)
        Self.leftColumn.scrollTop = Self.dataGridContainer.scrollTop;
        setTimeout(function() {
            Self.leftColumn.scrollTop = Self.dataGridContainer.scrollTop;
        }, 50); // yes, twice!
    }
    
    // add auto complete feature (in place editing)
    var autoCom = new AutoComplete(inputField, this.autoCompleteContainer, 25, configGetKey("minimumSearchCharacters") || 2, 100);
    autoCom.autoAdjustHeight = false;
    autoCom.showDisplayNamesOnly = true;
    autoCom.showSystemUsersOnly = true;
    autoCom.includeResources = true;
    // override onKeyEnter
    autoCom.onSelect = function (elem, st) {
        // get name & id
        var display_name = elem.getAttribute("oval");
        var member = autoCom.nameIndex[display_name];
        if (member) {
            // create member
            var row = new CalendarGridRow(member.type, member.id, member.display_name);
            row.data = member;
            Self.add(row);
            // update grid
            Self.clearPartial();
            Self.update();
        }
    };
}

CalendarGrid.prototype.isShowing = function() {
    var node = this.dataGrid;
    do {
        if (node.style.visibility == "hidden" || node.style.display == "none") {
            return false;
        }
        // climb up
        node = node.parentNode;
    } while (node != document.body);
    return true;
}

function CalendarGridRow(type, id, name, data) {
    this.type = type;
    this.id = id
    // sequenced "id:type" to allow cooperation with team week:
    this.rowId = id + ":" + type;
    this.display_name = name || "New member";
    // keep original data (might be useful in future)
    this.data = data || {};
    // reference to DOM node / makes selection of rows much easier
    this.node = null;
    // current index
    this.index = 0;
}

//------------------------------------------------------------------------------

var simpleSelectionHash = {};

function getSimpleSelection(id) {
    // exists?
    if (typeof(simpleSelectionHash[id]) == "undefined") {
        simpleSelectionHash[id] = new SimpleSelection();
    }
    return simpleSelectionHash[id];
}

function SimpleSelection() {
    this.multiple = true;
    this.domNodes = {};
    this.selectedItems = {};
    this.observedItems = {};
    this.changedEvent = null;
    this.lastSelectedItem = null;
}

SimpleSelection.prototype.setMultiple = function(flag) {
    this.multiple = !!flag;
};

SimpleSelection.prototype.setChangedEvent = function(eventName) {
    this.changedEvent = eventName;
    this.triggerChangedEvent();
};

SimpleSelection.prototype.triggerChangedEvent = function() {
    if (this.changedEvent) {
        triggerEvent(this.changedEvent, this.getSelectedItems());
    }
};

SimpleSelection.prototype.observe = function(DOMNode, id, item) {
    if (DOMNode) {
        // add listener
        var wrapper = (function(Self, id) {
             return function(e) {
                 Self.click(id, e.ctrlKey, e.shiftKey);
             };
        })(this, id);
        addDOMEvent(DOMNode, "click", wrapper);
        addDOMEvent(DOMNode, "click", stopEvent);
        // add second listener (context menu)
        var contextWrapper = (function(Self, id) {
            return function(e) {
                if (!Self.isSelected(id)) {
                    Self.click(id, false, false);
                }
            };
        })(this, id);
        addDOMEvent(DOMNode, "contextmenu", contextWrapper);
        // add to observation list
        this.observedItems[id] = item;
        // add to dom node list
        this.domNodes[id] = DOMNode;
    }
};

SimpleSelection.prototype.click = function(id, multiple, range) {
    // multiple?
    if (!this.multiple || !multiple) {
        // clear selection
        this.clear();
    }
    // range?
    if (range && this.lastSelectedItem) {
        // select range
        this.selectRange(this.lastSelectedItem, id);
    } else {
        // add single item to selection
        this.toggle(id);
        // remember last item
        this.lastSelectedItem = id;
    }
    // trigger event
    this.triggerChangedEvent();
};

SimpleSelection.prototype.selectRange = function(lastId, newId) {
    var Self = this;
    // get grid
    getCalendarGridIfExists("teamview", function(grid) {
        // get user id
        var fromUser = lastId.match(/@(\d+:\d+)$/);
        var toUser = newId.match(/@(\d+:\d+)$/);
        if (fromUser.length && toUser.length) {
            var members = [];
            // get indexes
            var fromIndex = grid.rowIndex[fromUser[1]];
            var toIndex = grid.rowIndex[toUser[1]];
            // swap?
            if (fromIndex > toIndex) {
                var tmp = fromIndex; fromIndex = toIndex; toIndex = tmp;
            }
            // loop through rows
            for (var i = fromIndex; i <= toIndex; i++) {
                members.push(grid.rows[i].rowId);
            }
            // create regex
            var regex = new RegExp("@(" + members.join("|") + ")$");
            // get start time
            var fromDate = Self.observedItems[lastId].start_date;
            var toDate = Self.observedItems[newId].start_date;
            // swap?
            if (fromDate > toDate) {
                var tmp = fromDate; fromDate = toDate; toDate = tmp;
            }
            // now loop through appointments
            for (var id in Self.observedItems) {
                var item = Self.observedItems[id];
                // check item
                if (item.start_date >= fromDate && item.start_date <= toDate && regex.test(id) ) {
                    Self.select(id);
                }
            }
        }
    });
};

SimpleSelection.prototype.select = function(id) {
    this.selectedItems[id] = this.observedItems[id];
    this.selectedItems[id].selectionId = id;
    var node = this.domNodes[id];
    if (node) {
        node.className += " isSelected";
    }
};

SimpleSelection.prototype.deselect = function(id) {
    delete this.selectedItems[id];
    var node = this.domNodes[id];
    if (node) {
        node.className = node.className.replace(/\s?isSelected/, "");
    }
};

SimpleSelection.prototype.isSelected = function(id) {
    return typeof(this.selectedItems[id]) != "undefined";
};

SimpleSelection.prototype.toggle = function(id) {
    if (this.isSelected(id)) {
        this.deselect(id);
    } else {
        this.select(id);
    }
};

SimpleSelection.prototype.clear = function() {
    // deselect items
    for (var id in this.selectedItems) {
        this.deselect(id);
    }
    // trigger event
    this.triggerChangedEvent();
};

SimpleSelection.prototype.numSelected = function() {
    var count = 0;
    for (var id in this.selectedItems) count++;
    return count;
};

SimpleSelection.prototype.getSelectedItems = function() {
    var list = [];
    for (var id in this.selectedItems) {
        list.push(this.selectedItems[id]);
    }
    return list;
};

SimpleSelection.prototype.getSelection = function() {
    var list = [];
    for (var id in this.selectedItems) {
        list.push(id);
    }
    return list;
};

/* to work with context menus */
SimpleSelection.prototype.fetch = function() {
    return this;
};

//------------------------------------------------------------------------------
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Jan Finsel <jan.finsel@open-xchange.com>
 * 
 */

(function() {
	var team = {}; // actually, this is a hash!

    // returns hash
    getTeamList = function() { return team; }; 
    
    // returns array
    getTeamArray = function() {
        var members = [];
        for (var id in team) { members.push(team[id]); }
        return members;
    };
    
    getTeamMember = function(id) { return team[id]; };
    
    setTeam = function(members, silent) {
        team = {};
        addToTeam(members, silent);
    };
    
    hasTeamMembers = function() {
        for (var id in team) return true;
        return false;
    }
    
    // added to work with new scalable team view
    // members must be a hash (key=id:type, value=member)
    importTeam = function(members) {
        team = members;
    };

    removeFromTeam = function(members, silent) {
        for (var i in members)
            delete team[members[i].id + ":" + members[i].type];
        if (!silent) triggerEvent("OX_Teammember_Changed", team);
    };
    
    addToTeam = function(members, silent) {
        var groups = [];
        for (var i in members) {
            var member = members[i];
            switch (member.type) {
                case 1:
                case 3:
                    team[member.id + ":" + member.type] = member;
                    break;
                case 2:
                    groups.push(member);
            }
        }
        if (groups.length) {
            internalCache.getObjects(groups, resolvedGroups(silent));
        } else {
            if (!silent) triggerEvent("OX_Teammember_Changed", team);
        }
    };
    
    function resolvedGroups(silent) {
        return function(groups) {
            var set = {};
            for (var i in groups) {
                var members = groups[i].members;
                for (var j in members) set[members[j]] = true;
            }
            var ids = [];
            for (var i in set) ids.push(Number(i));
            internalCache.getUsers(ids, resolvedUsers(silent));
        };
    }
    
    function resolvedUsers(silent) {
        return function(users) {
            for (var i in users) {
                team[i + ":1"] = {
                    id: users[i].id, type: 1,
                    display_name: users[i].display_name
                };
            }
            if (!silent) triggerEvent("OX_Teammember_Changed", team);
        };
    }

})();

var firstteam=true;
function filldefaultteam() {
    // required to cooperate with scalable team view (prevent overriding its team)
    var currentTeam = getTeamList();
    // check whether or not there are members
	if (currentTeam.length == 0 && firstteam) {
		var teams=new Array();
		if (configContainsKey("gui.calendar.teams")) {
			teams=configGetKey("gui.calendar.teams");
		}
        for(var i=0;i<teams.length;i++) {
			if(teams[i].team_default) {
                setTeam(teams[i].children, true);
				break;
			}
		}
		firstteam=false;
	}
}

var ctd_participants = null;
var team_participants_selected=new Array();
var team_change_menu;

function team_initContent() {
	team_createParticipantsPopUp();
    if (!team_change_menu) {
        team_change_menu =
            new PulldownMenu($("menu_calendar_teammember_change"));
        team_change_menu.onShow = function() {
            this.removeAllItems();
            var teams = configGetKey("gui.calendar.teams");
            if (teams) for (var i = 0; i < teams.length; i++) {
                this.addItem(new MenuItem(teams[i].name, (function(team) {
                    return function() {
                        team_participants_selected = [];
                        setTeam(team);
                    };
                })(teams[i].children)));
            }
        };
    }
}

function team_removeTeammembers() {
	removeFromTeam(team_participants_selected);
}

function team_changeTeammembers() {
    if(ctd_participants) {
        ctd_participants.openAddParticipantsWindow();
    }
}

function team_storeSelectedMember(response) {
	team_participants_selected=response;
	triggerEvent("SubSelected",team_participants_selected.length);
}

function team_createParticipantsPopUp() {
	ctd_participants = new ParticipantsSmall(null,addToTeam,true,true,true,false,"Select Users", /*i18n*/
						false,false,false,true); 
}

var oTeamWeek;
var TWPartPopup;
function goToNextTeamWeek(nDirection) {
    if(nDirection>0)
        oTeamWeek.next();
    else
        oTeamWeek.back();
        
    if (oMiniCalendar) oMiniCalendar.setSelectedByDate(activeYear, activeMonth, activeDay);
}

function teamWeekAddTeammember(oMemberHM) {
    if (oTeamWeek) {
        oTeamWeek.updateMemberList(oMemberHM);
    }
}

function unloadTeamWeek() {
    unregister("OX_Teammember_Changed",teamWeekAddTeammember);
    resizeEvents.unregister("Resized",resizeTeamWeekHeader);    
}

function loadTeamWeek() {
    register("OX_Teammember_Changed",teamWeekAddTeammember);
    if (!oTeamWeek) {
        oTeamWeek = new teamWeek();
    } else {
        oTeamWeek.set_current_week();
        oTeamWeek.updateHeader();
    }   
    resizeTeamWeekHeader();
    resizeEvents.register("Resized",resizeTeamWeekHeader);
    
    var team = getTeamList();
    if (team) {
        oTeamWeek.updateMemberList (team);
    }
    
    var hover=new Hover($("teamWeekBody"),OXAppointmentHover.getContent().node);
	calendarhovers["teamworkweek"]=hover;
	hover.setSize(OXAppointmentHover.contentobject.node);
	
	hover.getTarget = function (node) {
        try {
            while (node) {
                if (node.getAttribute("ox_object_id")) {
                    return node.parentNode ? node : null;
                }
                node = node.parentNode;
            }
        } catch (e) { /*see default implementation*/ }
    };
    /* event fired when getTarget found a node with valid attributes */
    hover.onShow = function (node) {
        OXAppointmentHover.actualHover = this;
        var ref       = node.getAttribute("ox_rowId");
        var folder_id = node.getAttribute("ox_folder_id");
        var object_id = node.getAttribute("ox_object_id");
        var rec_pos   = node.getAttribute("ox_rec_pos");
        if (object_id != undefined && folder_id != undefined) {
            // we have a folder and a object id so we can use the regular hover
            OXAppointmentHover.refillContent(object_id, folder_id, rec_pos);
        } else if (ref != undefined && oTeamWeek && oTeamWeek.app_assoc[ref]) {
            // we don't have read permission so we just show content we are allowed to            
            // disable tabs we don't need
            OXAppointmentHover.slider.getTabById("participants").show(false);
            OXAppointmentHover.slider.getTabById("attachments").show(false);
            OXAppointmentHover.slider.getTabById("others").show(false);
            OXAppointmentHover.refillContentByObject(oTeamWeek.app_assoc[ref]);
        }
    }
    hover.onHide = function (node) {
      // re-enable tabs for later use
      OXAppointmentHover.slider.getTabById("participants").show(true);
      OXAppointmentHover.slider.getTabById("attachments").show(true);
      OXAppointmentHover.slider.getTabById("others").show(true);
    }    
}

function resizeTeamWeekHeader() {
    var e1=$("teamWeekBody");
    var mywidth=e1.offsetWidth;
    if(!mywidth)
    $("teamWeekTableHead").style.width=mywidth+"px";
}

function clickMemberTeamWeek(index, e) {
    if (e.shiftKey && oTeamWeek.getSelected().length >= 0) {
    	// holds shift?    	
    	var startIndex = oTeamWeek.selected_member[oTeamWeek.getSelected()[0].id];
    	oTeamWeek.selectNone(); // unselect all
    	oTeamWeek.selectMember(Math.min(startIndex, index), 
				Math.max(startIndex, index));
    } else if (oTeamWeek.member_assoc["team_week_row_" + index]) {
    	// multiple?
        if (!(Mac ? e.metaKey : e.ctrlKey)) {
        	oTeamWeek.selectNone();
        }
    	if (oTeamWeek.selected_member[oTeamWeek.member_assoc["team_week_row_" + index].id] != null) {
 	    	// <ctrl> and click on selected row	        
	        $ALL("u_team_week_" + index).parentNode.className = "teamWeekColMemberSecond border-color-design";
	        delete oTeamWeek.selected_member[oTeamWeek.member_assoc["team_week_row_" + index].id];
	    } else {
	    	oTeamWeek.selectMember(index, index);
	    }
    }    
}

function dblClickMemberTeamWeek(e) {
    if (!oTeamWeek || !oTeamWeek.selected_member.length) return; 
    function getTarget(node) {
		function getNode(actualnode) {
			if (actualnode.getAttribute("ox_time")) {
				return actualnode;
			}
			return;
		}
		while (node) {
			var element=getNode(node);
			if (element) {
				return element;
			}
			try {
			   node = node.parentNode;
			} catch (e) { 
				node = null;
			} 
		}
		return;
	}
    var date = new Date(Date.UTC(activeYear, activeMonth, activeDay, 0, 0, 0));
	// setting clicked date
    var node = getTarget(e.srcElement || e.target);
	if (node && node.getAttribute("ox_time")) {
		date = new Date(oTeamWeek.monday.getTime()
				+ (864e5 * parseInt(node.getAttribute("ox_time"))));
		date.setUTCHours(new Date().getHours())
		date.setUTCMinutes(0);
	}
    // trigger new window with clicked paramter
	calendar_createNewAppointment(
			date,
			activefolder,false,
			{ "data"  : [[0, 0, oTeamWeek.selected_member[0].display_name, 0, oTeamWeek.selected_member[0].id, 0, 0, oTeamWeek.selected_member[0].type]], 
				  "module": "contacts" });
	stopEvent(e);
}

/////////////////////////////////////////////////////////////////////////
function teamWeek()
{
    this.template = $('team_week_row');
    this.container_id = 'teamWeekTableContent';
    this.container = newnode('tbody',0,{width:'100%'});
    this.membercount = 0;
    this.memberlist;
    
    this.selected_member = {};
    
    this.member_assoc = new Object();
    this.app_assoc = new Object();  
    
    this.full_data_aps = new Object();
    
    this.calendar_tool;
    this.kw;
    this.monday;
    this.friday;
    
    this.set_current_week();
    this.updateHeader();
    
    this.app_collector = new Object();
}


teamWeek.prototype = {
	set_current_week:function ()
	{
	    this.calendar_tool = new calendarTool(activeMonth,activeYear);
	    this.kw = this.calendar_tool.getKW(activeYear,activeMonth,activeDay);
	    var oDateLoc = new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0));
	    this.monday = new Date(getDayInSameWeek(oDateLoc, weekStart) * 864e5);
	    this.friday = new Date(this.monday.getTime());
	    this.friday.setDate(this.friday.getDate()+5);
	},
	next:function ()
	{
	    var oDateLoc = new Date();
	    oDateLoc.setUTCDate(1);
	    oDateLoc.setUTCHours(0);
	    oDateLoc.setUTCFullYear(activeYear);
	    oDateLoc.setUTCMonth(activeMonth);
	    oDateLoc.setUTCDate(activeDay);
	    oDateLoc.setUTCDate(oDateLoc.getUTCDate()+7);
	    activeDay = oDateLoc.getUTCDate();
	    activeMonth = oDateLoc.getUTCMonth();
	    activeYear = oDateLoc.getUTCFullYear();
	    this.set_current_week();        
	    this.updateAppointmentView();
	},  
	back:function ()
	{   
	    var oDateLoc = new Date();
	    oDateLoc.setUTCDate(1);
	    oDateLoc.setUTCHours(0);
	    oDateLoc.setUTCFullYear(activeYear);
	    oDateLoc.setUTCMonth(activeMonth);
	    oDateLoc.setUTCDate(activeDay);
	    oDateLoc.setUTCDate(oDateLoc.getUTCDate()-7);
	    activeDay = oDateLoc.getUTCDate();  
	    activeMonth = oDateLoc.getUTCMonth();
	    activeYear = oDateLoc.getUTCFullYear();
	    this.set_current_week();    
	    this.updateAppointmentView();   
	
	},
	updateHeader:function()
	{
	    removeChildNodes($('teamWeekHead'));
	    var oTextNode = document.createTextNode(this.kw + ' ' + this.calendar_tool.curr_date.getFullYear());
	    $('teamWeekHead').appendChild(oTextNode);
	    var startDate = new Date(this.monday);
	    for(var indx=1;indx<=5;indx++)
	    {
	        var sHeaderId = 'teamWeekCol_header_' + indx;
	        removeChildNodes($(sHeaderId));
	        var oTextNode = document.createTextNode(formatDate(startDate,"dateday"));       
	        $(sHeaderId).appendChild(oTextNode) ;
	        startDate.setUTCDate(startDate.getUTCDate()+1);
	    }
	},
	updateAppointmentView:function()
	{
	    this.clearAppView();
	    this.fetchAppointments(this.memberlist);
	    this.updateHeader();
	},
	clearAppView:function ()
	{   
	    for(var ikey in this.member_assoc)
	    {
	        for(var indx=1;indx<6;indx++)
	        {
	            var sId = 'd'+indx + '_team_week_' + ikey.split('_')[3];
	            var oParentDiv = document.getElementById(sId);
	            while(oParentDiv.childNodes.length>0)
	            {
	                oParentDiv.removeChild(oParentDiv.firstChild);
	            }
	            oParentDiv.innerHTML = '&#xa0;';            
	        }
	    }
	    this.app_collector = new Object();  
	},
	drawAppointments:function (oAppointments,nRowId)
	{
	    var sPostfix = nRowId.split('_')[3];    
	    this.app_collector[sPostfix] = new Array();
	    function duplicate_app(app) {
	        var copyApp = new Object();
	        copyApp["id"] = app.id;
	        copyApp["folder_id"] = app.folder_id;
	        copyApp["start_date"] = app.start_date;
	        copyApp["end_date"] = app.end_date;
	        copyApp["shown_as"] = app.shown_as;                                                         
	        copyApp["full_time"] = app.full_time;
	        copyApp["created_by"] = app.created_by;
	        copyApp["full_time"] = app.full_time;
	        copyApp["title"] = app.title
	        copyApp["recurrence_position"] = app.recurrence_position;
	        return copyApp;
	    }
	    for (var indx=0; indx<oAppointments.data.length; indx++) {	    		    	
	        var tApp = duplicate_app(oAppointments.data[indx]);	        
	        var sIdAp = 'app_d_' + nRowId + '_' + indx;
	        
	        this.app_assoc[sIdAp] = (this.full_data_aps[tApp.id]==undefined) ? tApp : this.full_data_aps[tApp.id];
	        
	        /*
	         * set appointment start day to the start day of this week
	         * if it starts before monday        
	         */
	        if (tApp.start_date < this.monday.getTime()) {
	            tApp.start_date = this.monday.getTime();
	        }
	        /*
	         * set appointment end day to the end day of this week
	         * if it ends after this friday      
	         */
	        if (tApp.end_date > this.friday.getTime()) {
	            tApp.end_date = this.friday.getTime();
	        }
	        
	        var oDateStart = new Date(tApp.start_date);
	        var oDateEnd = new Date(tApp.end_date); 
	        var nStartDay = oDateStart.getUTCDay();
	        var nEndDay = oDateEnd.getUTCDay();
	        var appDuration = Math.ceil(tApp.end_date/864e5) - Math.floor(tApp.start_date/864e5);       
	 
	        if (appDuration == 1) {
	            tApp.assoc_id = sIdAp;
	            this.collectAppointment(nStartDay,sPostfix,tApp);  
	            
	        } else {
	            for (var nNumDay=nStartDay; nNumDay<(nStartDay+appDuration); nNumDay++) {
	            	var tApp2 = duplicate_app(tApp);
	                if (nNumDay == nStartDay) {
	                    var oDateStart = new Date(tApp2.start_date);
	                    oDateStart.setUTCHours(0);
	                    oDateStart.setUTCMinutes(0);
	                    oDateStart.setUTCSeconds(0);    
	                    oDateStart.setUTCDate(oDateStart.getUTCDate()+1);
	                    tApp2.end_date = oDateStart.getTime();
	                    
	                } else if (nNumDay == nEndDay && nStartDay < nEndDay) {
	                    var oDateStart = new Date(tApp2.end_date);
	                    oDateStart.setUTCHours(0);
	                    oDateStart.setUTCMinutes(0);
	                    oDateStart.setUTCSeconds(0);    
	                    tApp2.start_date = oDateStart.getTime();
	                    
	                } else {                
	                    var oDateStart = new Date(tApp2.start_date);
	                    var oDateEnd = new Date(tApp2.start_date);                                        
	                    oDateStart.setUTCHours(0);
	                    oDateStart.setUTCMinutes(0);
	                    oDateStart.setUTCSeconds(0);    
	                    oDateEnd.setHours(0);
	                    oDateEnd.setMinutes(0);
	                    oDateEnd.setSeconds(0);                     
	                    oDateStart.setUTCDate(oDateStart.getUTCDate()+nNumDay);                 
	                    oDateEnd.setUTCDate(oDateStart.getUTCDate()+nNumDay+1);                                     
	                    tApp2.start_date = oDateStart.getTime();              
	                    tApp2.end_date = oDateEnd.getTime();                                  
	                }
	                tApp2.assoc_id = sIdAp;
	                this.collectAppointment(nNumDay,sPostfix,tApp2);
	            }
	        }
	    }
	},
	doDraw:function ()
	{
	    for(var sKey in this.app_collector)
	    {
	        var aAppointments = this.app_collector[sKey];
	        for(var nDay in aAppointments)
	        {
	            var oList = this.getLineList(aAppointments[nDay]);
	            this.drawList(oList,sKey,nDay);
	        }
	    }
	},
	getPrio:function (number)
	{
	    var aPrio = new Array(-1,4,3,1,0,2);    
	    for(var indx=0;indx<aPrio.length;indx++)
	    {
	        if(aPrio[indx] == number)
	            return indx;    
	
	    }
	},
	getLineList:function (aApp)
	{   
	    var oDataStart = new Object();
	    oDataStart.x = 0;
	    oDataStart.type = -1;
	    oDataStart.appid = -1;      
	    var oDataEnd = new Object();    
	    oDataEnd.x = 100;
	    oDataEnd.type = -1;
	    oDataEnd.appid = -1;    
	    var oList = new js_list();
	    oList.push(oDataStart);
	    oList.push(oDataEnd);
	    //Sort all appointments by start date
	    function sort_fn(a,b){
	        if(a.start_date < b.start_date)
	            return -1;
	        else            
	            return 1;   
	    }
	    aApp.sort(sort_fn);
	    for(var nindx=0;nindx<aApp.length;nindx++)
	    {
	
	        var currApp = aApp[nindx];
	        var sAppId = currApp.assoc_id;
	        var oStartDate = new Date(aApp[nindx].start_date);
	        var oEndDate = new Date(aApp[nindx].end_date);      
	
	        var nHoursStart = oStartDate.getUTCHours();
	        var nMinutesStart = oStartDate.getUTCMinutes();
	        var nHoursEnd = oEndDate.getUTCHours();
	        var nMinutesEnd = oEndDate.getUTCMinutes();
	        if(nHoursStart == 0 && nMinutesStart == 0)
	            var currStartX = 0.5;
	        else
	            var currStartX = Math.round(((nHoursStart * 60 + nMinutesStart) * 100) / (24 * 60));
	            
	        if(nHoursEnd == 0 && nMinutesEnd == 0)
	            var currEndX = 99.5;
	        else
	            var currEndX = Math.round(((nHoursEnd * 60 + nMinutesEnd) * 100) / (24 * 60));
	    
	
	        //find start position       
	        var listnode = oList.first;
	        while(listnode.data.x <= currStartX)
	        {           
	            listnode = listnode.next;
	        }   
	    
	        var oDataStart = new Object();
	        oDataStart.x = currStartX;
	        oDataStart.type = currApp.shown_as; 
	        oDataStart.appid = sAppId;
	        var nTypeBefore = listnode.previous.data.type;      
	        var nAppIdBefore = listnode.previous.data.appid;            
	
	        if(this.getPrio(oDataStart.type) < this.getPrio(nTypeBefore))   
	        {
	            if(this.getPrio(listnode.data.type) < this.getPrio(oDataStart.type) 
	                && listnode.data.x < currEndX)
	            {               
	                currStartX = listnode.data.x;
	                nTypeBefore = listnode.data.type;
	                nAppIdBefore= listnode.data.appid;
	                oDataStart.x = currStartX;
	                listnode = listnode.next;
	            }
	            else continue;
	        }
	
	        //if previous point equal current point, remove it      
	        if(listnode.previous.data.x == currStartX)      
	        {   
	            oList.remove(listnode.previous);
	        }
	        oList.insert_before(listnode,oDataStart);
	        //find end position
	        while(listnode.data.x <= currEndX)
	        {   
	        //  nTypeBefore = listnode.data.type;
	            listnode.data.type = oDataStart.type;
	            listnode = listnode.next;
	        }   
	        if(listnode.previous.data.x == currEndX)        
	        {   
	            oList.remove(listnode.previous);
	        }
	        var oDataEnd = new Object();
	        oDataEnd.x = currEndX;
	        if(listnode.data.x == 100)
	        {
	            nTypeBefore = -1;
	        }
	        oDataEnd.type = nTypeBefore;    
	        oDataEnd.appid = nAppIdBefore;      
	        oList.insert_before(listnode,oDataEnd);     
	    }
	    //this.print_list(oList);
	    return oList;
	},
	
	print_list: function (oList) {
	    var ocurrNode = oList.first;
	    var nC = 0;
	    while (ocurrNode.next != null)
	    {
	        ocurrNode = ocurrNode.next;
	        nC++;
	    }
	},
	
	drawList: function (oList, nId, nDay) {
	    var sContainerId = 'd'+nDay+'_team_week_' + nId;
	    var oParentDiv =  document.getElementById(sContainerId);
	    if(oParentDiv == undefined)
	        return;
	    removeChildNodes(oParentDiv);
	    
	    var lItem = oList.first;    
	    var nNumItems = oList.number;
	    var Coeff = (100 - Math.round(nNumItems/2)-1)/100; // compute with space
	    while(lItem.next != null) {       
	        var appid_l = lItem.data.appid;
	        var size = lItem.next.data.x - lItem.data.x;
	        size *= Coeff;
	        var type = lItem.data.type;
	        var oDiv = newnode('div');      
	        oDiv.className = 'colorBlockTWV workTimeBackground';
	        switch (type) {
		        case 1:
		            oDiv.className += " appointmentRESERVED";
		            break;
		        case 2:
		            oDiv.className += " appointmentTEMPORARY";
		            break;
		        case 3:
		            oDiv.className += " appointmentABSENT";
		            break;
		        case 4:
		            oDiv.className += " appointmentFREE";
		            break;
		        default:
		            //oDiv.className += " appointmentShowAsNone";
		            break;
	        }
	        oDiv.style.width = size + "%";
	        oDiv.innerHTML = '&#xa0;';
	        if (appid_l == -1) {
	        	oDiv.removeAttribute("id");
	        } else {
	        	oDiv.id = appid_l;
	        	var app = this.app_assoc[appid_l];
	        	if (app) {
	        		oDiv.setAttribute("ox_rowId", appid_l);
	        		oDiv.setAttribute("ox_object_id", app.id);
	        		if (app.folder_id) oDiv.setAttribute("ox_folder_id", app.folder_id);
	        		if (app.recurrence_position) oDiv.setAttribute("ox_rec_pos", app.recurrence_position);
	        	}
	        }
	        oParentDiv.appendChild(oDiv);
	        var oSpacerDiv = newnode('div');
	        oSpacerDiv.className = 'colorBlockTWV workTimeBackground';
	        oSpacerDiv.style.width = '0.5%';
	        oParentDiv.appendChild(oSpacerDiv);     
	        lItem = lItem.next;
	    }
	},
	
	collectAppointment: function (nDay,sPostfix,oApp) {
	    if (oApp.start_date < this.monday.getTime() && 
	        oApp.end_date < this.monday.getTime() || oApp.start_date > this.friday.getTime() && 
	        oApp.end_date > this.friday.getTime()) {
	        return;
	    }
	    //exclude saturday and sunday
	    if (nDay==0 || nDay==6) {
	        return;
	    }
	    if(!this.app_collector[sPostfix][nDay]) {
	        this.app_collector[sPostfix][nDay] = new Array();
	    }
	    this.app_collector[sPostfix][nDay].push(oApp);
	},
	
	fetchAppointments: function (aUpdtMember) {
	    if (!aUpdtMember || isEmpty(aUpdtMember)) return; // no members requested   
	    var Self = this;
	    var aJsonArray = new Array();
	    for(var sKeyAssoc in aUpdtMember)
	    {
	        var oRequestObject = new Object();
	        oRequestObject['module'] = "calendar";
	        oRequestObject['action'] = 'freebusy';
	        oRequestObject['id'] = aUpdtMember[sKeyAssoc].id;
	        oRequestObject['type'] = aUpdtMember[sKeyAssoc].type;
	        oRequestObject['start'] = this.monday.getTime();
	        oRequestObject['end']= this.friday.getTime();   
	        aJsonArray.push(oRequestObject);
	    }
	
	    function callback(responseobj) {
	        if (responseobj.error) {
	        	newServerError(responseobj, 4);
	        	
	        } else {
	            //fetch full data here
	            function sub_callback(arg){
	                //prepare and set data
	                for(var nFullDataApp=0;nFullDataApp < arg.data.length;nFullDataApp++) {
	                    var tmpdata = arg.data[nFullDataApp];
	                    Self.full_data_aps[tmpdata[0]] = 
	                        new CalendarWorkWeekObject(tmpdata[0],tmpdata[1],tmpdata[2],tmpdata[3],tmpdata[4],tmpdata[5],tmpdata[6],tmpdata[7],tmpdata[8],tmpdata[9],tmpdata[10],tmpdata[11],tmpdata[12],tmpdata[13],tmpdata[14],tmpdata[15],tmpdata[16]);
	                }
	                var nIndx = 0;
	                // iterate through requested member list
	                for (var mkey in aUpdtMember) {
	                    // iterate through internal member list to map the server response              
	                    for (var nkey in Self.member_assoc) {
	                        // check if member already in list and (re-)draw appointment
	                        if (Self.member_assoc[nkey].id == aUpdtMember[mkey].id) {
	                            Self.drawAppointments(responseobj[nIndx],nkey);
	                            break;
	                        }
	                    }
	                    nIndx++;
	                }
	                Self.doDraw();
	            }
	            var calendarwwfields = new Array(1,20, 207, 206, 2, 200, 201, 202, 203, 209, 220, 401, 402, 102, 400, 101);
	            var sAjaxReq = AjaxRoot + "/calendar?action=all&session=" + session + "&start=" + Self.monday.getTime() + "&end=" 
	                                                        + Self.friday.getTime() + "&columns=" +calendarwwfields.toString();         
	            json.get(sAjaxReq,null,sub_callback);
	        }
	    }   
	    json.put(AjaxRoot + "/multiple?session=" + session,aJsonArray,null,callback);
	},
	
	updateMemberList: function (oMemberHM) {
	    var bMemberAddd = false;
	    if (oMemberHM) {
	        this.memberlist = oMemberHM;
	    }
	
	    //remove member
	    for (var sKeyAssoc in this.member_assoc) {
	        if (!this.memberlist[this.member_assoc[sKeyAssoc].id + ":" + this.member_assoc[sKeyAssoc].type]) {
	            this.removeMember(sKeyAssoc, this.member_assoc[sKeyAssoc].id);
	        }
	    }
	    
	    //check member
	    for (var sKey in this.memberlist) {
	        var bFound = false;
	        //search for members
	        for(var sKeyAssoc in this.member_assoc) {
	            if(this.member_assoc[sKeyAssoc].id == sKey.split(':')[0]) {
	                bFound = true;
	                break;
	            }
	        }
	        //add member        
	        if (!bFound) {
	            this.addMember(this.memberlist[sKey]);          
	            bMemberAddd = true;
	        }
	    }
	//  if(bMemberAddd)
	        this.updateAppointmentView(this.member_assoc);
	},
	
	addMember: function (oMember) {
		var Self = this;
	    var oTeml = this.template.cloneNode(true);
	    oTeml.id += '_' + this.membercount; 
	    var arr = oTeml.getElementsByTagName("div");
	    for (var i in arr) {    	
	    	var div = arr[i];
	    	if (div.id == "u_team_week") {
	            div.appendChild(document.createTextNode(oMember.display_name));
	            // prevent moz element selection
	            div.parentNode.onselectstart = function() { return false; };
	            div.parentNode.style.MozUserSelect = "none";
	            var wrapper = (function(index) {
	                return function(e) {
	                    clickMemberTeamWeek(index, e); 
	                };
	            })(Self.membercount);
	            addDOMEvent(div.parentNode, "click", wrapper);
	    	} else if (div.setAttribute) {
	    		div.setAttribute("ox_time", String(i-1));
	    		addDOMEvent(div, "dblclick", dblClickMemberTeamWeek);    		
	    	} 
	    	if (div.id) {
	    		div.id += "_" + this.membercount;
	    	}
	    }
	    this.container.appendChild(oTeml);
	    if(!this.membercount)
	        $(this.container_id).appendChild(this.container);
	    this.member_assoc['team_week_row' + '_' + this.membercount] = oMember;      
	    this.membercount++;
	    resizeTeamWeekHeader();
	},
	
	/**
	 * set selection by index from to
	 * @param from {num} the start index
	 * @param to   {num} the end index
	 */
	selectMember: function (from, to) {
		for (var i = from; i <= to; i++) {
			var row = this.member_assoc["team_week_row_" + i];		
			this.selected_member[row.id] = i;
			$ALL("u_team_week_" + i).parentNode.className += " background-color-selection-elements font-weight-high font-color-selected";
		}
		triggerEvent('OX_Selected_Member', this.getSelected());
	},
	
	getSelected: function() {
	    var selection = [];
	    for (var id in oTeamWeek.selected_member) {
	        selection.push(this.member_assoc["team_week_row_" + oTeamWeek.selected_member[id]]);
	    }
	    return selection;
	},
	
	/**
	 * un-select all
	 */
	selectNone: function() {
	    for (var id in this.selected_member) {
	        $ALL("u_team_week_" + this.selected_member[id]).parentNode.className = "teamWeekColMemberSecond border-color-design";
	        delete this.selected_member[id];
	    }
	},
		
	/**
	 * removes a member from the list
	 * @param row_id {String} the row_id
	 * @param user_id {String} the id of the user
	 */
	removeMember: function(row_id, user_id) {
	    var oSelTr = document.getElementById(row_id);
	    this.container.removeChild(oSelTr);	    
	    delete this.member_assoc[row_id];
	    delete this.selected_member[user_id];
	    triggerEvent("OX_Selected_Member", this.getSelected());
	}

}


///////////////////////////////////////////////////////
//Liste

function list_obj()
{
    this.next;
    this.previous;
    this.data;
}
function js_list()
{
    this.first = null;
    this.last = null;
    this.next_ = null;
    this.previous_ = null;
    this.number = 0;    
}
js_list.prototype = {
push:function (aData)
{
    var oItem = new list_obj();
    oItem.next = null;      
    oItem.previous = this.previous_;    
    oItem.data = aData;
    
    if(this.previous_!=null)
        this.previous_.next = oItem;
    this.previous_ = oItem;
    if(!this.first)
        this.first = oItem;
    this.last = oItem;          
    this.number++;  
},
insert_before:function(oBeforeItem,aData)
{
    var oItem = new list_obj(); 
    oItem.previous = oBeforeItem.previous;
    oItem.next = oBeforeItem;       
    oItem.data = aData;
    oBeforeItem.previous.next = oItem;
    oBeforeItem.previous = oItem;
    this.number++;
},
remove:function(oItem)
{
    oItem.previous.next = oItem.next;   
    oItem.next.previous = oItem.previous;
    oItem = null;
    delete oItem;
    this.number--;
}
}

fileloaded();
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Jan Finsel <jan@open-xchange.org>
 * 
 */
register_Calendar();

var activeDate=new Date();
var activeDay=activeDate.getDate();
var activeMonth=activeDate.getMonth();
var activeYear=activeDate.getFullYear();
var calendarFolderPath;

/*********************************************************************************************************************/
var selectedAppointment=new Object();
var calendarSelection = new Selection();
var lastSelectedAppointment=null;

calendar_serializeid = function(x) {
	return x.folder + "." + x.id + "." + (x.recurrence_position || 0) 
	       + "." + (x.recurrence_id || 0) + "." + (x.created_by || 0);
};
calendar_makeID = function (x) {
	var mObj = { id: x[0], folder: x[1] };	
	if (x[2] != undefined) { 
		mObj["recurrence_position"]=x[2]; 
	}
	if (x[3] != undefined) {
		mObj["recurrence_id"]=x[3];
	}
	if (x[4] != undefined) {
        mObj["created_by"]=x[4];
    }
	return mObj;
}

var calendarhovers=new Object();
function register_Calendar() {
	
	registerView("calendar",null,
	function () { 
		register("OX_New_Search", calendarSearch); 		
	},
	function () { 
		unregister("OX_New_Search", calendarSearch); 
	},
	function () {
		menuclearSearch();
	});
	registerView("calendar/calendar/day",
		function() { 
			showNode("calendarWorkweek"); 
		}, 
		function() { 
			maxappointmentsperday=configGetKey("gui.calendar.day.numberofappointments");
			cwwdaysinweek = 1;
			var oDateLoc = new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0));			    
			startofweek=oDateLoc.getUTCDay();
			if (oMiniCalendar) oMiniCalendar.setSelectedByDate(activeYear, activeMonth, activeDay);
			
			unregister("OX_Create_Object",default_calendar_create_object);
			register("OX_Create_Object",default_calendar_workweek_create_object);		
 			cwwenable();
            displayNameDayView();       
            calendarFolderPath.drawDOMNode("calendarofworkweekview", activefolder);
			showCalendarWorkweek();
			register("OX_Add_Flag", calendarAddTag);
			if (calendarhovers["daywwcustom"] && configGetKey("gui.effects.hover.calendar")) {
				calendarhovers["daywwcustom"].enable();
				calendarhovers["daywwcustom2"].enable(); 
			}
			register("OX_Print", calendar_printDay);
		 }, 
		 function() {
		 	if (calendarhovers["daywwcustom"]) { 
		 		calendarhovers["daywwcustom"].disable(); 
		 		calendarhovers["daywwcustom2"].disable(); 
		 	}
		 	unregister("OX_Create_Object",default_calendar_workweek_create_object);		 	
 			register("OX_Create_Object",default_calendar_create_object);
 			cwwdisable();
 			calendarFolderPath.clear();
		 	unregister("OX_Add_Flag", calendarAddTag);
		 	unregister("OX_Print", calendar_printDay);
		 },
		function() { 
			hideNode("calendarWorkweek"); 
		},
		function() { 
			var oDateLoc = new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0));	
			startofweek=oDateLoc.getUTCDay();
			displayNameDayView();
			calendarFolderPath.drawDOMNode("calendarofworkweekview", activefolder);		
			showCalendarWorkweek();
		}
	);
	registerView("calendar/calendar/workweek",
		function() { 
			showNode("calendarWorkweek"); 
		}, 
		function() { 
 			maxappointmentsperday=configGetKey("gui.calendar.workweek.numberofappointments");
			cwwdaysinweek = configGetKey("gui.calendar.workweek.countdays");
			startofweek=configGetKey("gui.calendar.workweek.startday");
			unregister("OX_Create_Object",default_calendar_create_object);
			register("OX_Create_Object",default_calendar_workweek_create_object);	
			calendarFolderPath.drawDOMNode("calendarofworkweekview", activefolder);	 	
			displayNameDayView();		
 			cwwenable();
			showCalendarWorkweek();		
			if(calendarhovers["daywwcustom"] && configGetKey("gui.effects.hover.calendar")) { 
				calendarhovers["daywwcustom"].enable(); 
				calendarhovers["daywwcustom2"].enable(); 
			}
			register("OX_Add_Flag", calendarAddTag);
			register("OX_Print", calendar_printWorkweek);
		}, 
		function() {
		    unregister("OX_Print", calendar_printWorkweek);
 			if (calendarhovers["daywwcustom"]) { 
 				calendarhovers["daywwcustom"].disable(); 
 				calendarhovers["daywwcustom2"].disable(); 
 			}
			unregister("OX_Create_Object",default_calendar_workweek_create_object);		 	
 			register("OX_Create_Object",default_calendar_create_object);
 			calendarFolderPath.clear();
			cwwdisable()
		 	unregister("OX_Add_Flag", calendarAddTag);	
		},
        function() { 
			hideNode("calendarWorkweek");
		},
		function() {
			displayNameDayView();
			calendarFolderPath.drawDOMNode("calendarofworkweekview", activefolder);	
			showCalendarWorkweek();
		}
	);
	registerView("calendar/calendar/custom",
		function() { showNode("calendarWorkweek"); }, 
		function() { 
 			maxappointmentsperday=configGetKey("gui.calendar.custom.numberofappointments");
			cwwdaysinweek = configGetKey("gui.calendar.custom.countdays");
			var oDateLoc = new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0));	
			startofweek=oDateLoc.getUTCDay();
			unregister("OX_Create_Object",default_calendar_create_object);
			register("OX_Create_Object",default_calendar_workweek_create_object);
			calendarFolderPath.drawDOMNode("calendarofworkweekview", activefolder);		 	
			displayNameDayView();		
 			cwwenable();
			showCalendarWorkweek();
			if (calendarhovers["daywwcustom"] && configGetKey("gui.effects.hover.calendar")) { 
				calendarhovers["daywwcustom"].enable(); 
				calendarhovers["daywwcustom2"].enable(); 
			}		
			register("OX_Add_Flag", calendarAddTag);
		 }, 
		 function() {
 			if (calendarhovers["daywwcustom"]) { 
 				calendarhovers["daywwcustom"].disable(); 
 				calendarhovers["daywwcustom2"].disable(); 
 			}
			unregister("OX_Create_Object",default_calendar_workweek_create_object);		 	
 			register("OX_Create_Object",default_calendar_create_object);
			cwwdisable()
			calendarFolderPath.clear();
		 	unregister("OX_Add_Flag", calendarAddTag);	
		 },
		function() { 
			hideNode("calendarWorkweek");
		},
		function() {
			var oDateLoc = new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0));	
			startofweek=oDateLoc.getUTCDay();
			calendarFolderPath.drawDOMNode("calendarofworkweekview", activefolder);
			displayNameDayView();		
			showCalendarWorkweek();
		}
	);
	var firstmonthview=false;
	registerView("calendar/calendar/month",
		function() { 
			showNode("calendarMonth");
			$("calendarMonth").style.display="block"; 
		}, 
		function() {
			if(!firstmonthview) { 
				firstmonthview=true;
				var hover=new Hover($("calmain"),OXAppointmentHover.getContent().node);
				calendarhovers["month"]=hover;	
				hover.setSize(OXAppointmentHover.contentobject.node);
				hover.getTarget = function (node) {
                    try {
    					while (node) {
    						if(node.oxcalobj) {
                                return node.parentNode ? node : null;
                            }
    						node = node.parentNode;
    					}
                    } catch (e) { /*see default implementation*/ }
                };
				hover.onShow = function (node) {
					OXAppointmentHover.actualHover=this;
					var splitid=node.oxcalobj.split("-");
					return OXAppointmentHover.refillContent(splitid[1],splitid[0],splitid[2]);	
				} 
			} 
			if(configGetKey("gui.effects.hover.calendar") && calendarhovers["month"]) {
				calendarhovers["month"].enable();
			}
			calendarFolderPath.drawDOMNode("calendarofmonthview", activefolder);
			displayNameDayView();	  
			register('OX_Refresh',refreshMonthView);
		  	register("OX_Add_Flag", calendarAddTag);
			resizeEvents.register("Resized",resizeWeekSections);
			unregister("OX_Create_Object",default_calendar_create_object);
			register("OX_Create_Object",default_calendar_workweek_create_object);		
		  	loadMonthView();
		  	register("OX_Print", calendar_printMonth);
		 }, 
		 function() {
            unregister("OX_Print", calendar_printMonth);
		 	if (calendarhovers["month"]) { 
		 		calendarhovers["month"].disable(); 
		 	}
		 	unregister('OX_Refresh',refreshMonthView);
		 	unregister("OX_Add_Flag", calendarAddTag);
			register("OX_Create_Object",default_calendar_create_object);
			unregister("OX_Create_Object",default_calendar_workweek_create_object);		 	
		 	resizeEvents.unregister("Resized",resizeWeekSections);
		 	calendarFolderPath.clear();
		 },
		function() { 
			hideNode("calendarMonth");
			$("calendarMonth").style.display="none";
		},
		function() {
			calendarFolderPath.drawDOMNode("calendarofmonthview", activefolder);
			displayNameDayView();
			refreshMonthView(true);		
            resizeEvents.register("Resized",resizeWeekSections);	
		}
	);
	registerView("calendar/calendar/week",
		function() { 
			showNode("calendarWeek");
			$("calendarWeek").style.display=""; }, 
		function() { 
			calendarFolderPath.drawDOMNode("calendarofweekview", activefolder);
			displayNameDayView(); 	
			unregister("OX_Create_Object",default_calendar_create_object);
			register("OX_Create_Object",default_calendar_workweek_create_object);		
			objWeekCalendarTool.init();
			register("OX_Add_Flag", calendarAddTag);
			register("OX_Print", calendar_printWeek);
		 }, 
		 function() {
            unregister("OX_Print", calendar_printWeek);
		 	if(calendarhovers["week"]) { 
		 		calendarhovers["week"].disable(); 
		 	}
			register("OX_Create_Object",default_calendar_create_object);
			unregister("OX_Create_Object",default_calendar_workweek_create_object);	
		 	unregister("OX_Add_Flag", calendarAddTag);
		 	calendarFolderPath.clear(); 	
		},
		function() { 
			hideNode("calendarWeek");
			$("calendarWeek").style.display="none"; },
		function() {
			calendarFolderPath.drawDOMNode("calendarofweekview", activefolder);
			displayNameDayView();
			objWeekCalendarTool.init();
		}
	);
	
	var calendarTeamViewChangeTeamPopup = null;
	// store team view specific settings in ui config
	var storeTeamViewSettings = function(param) {
		var grid = getCalendarGrid("teamview");
		if (typeof(grid) != undefined) {
			configSetKey("gui.calendar.team.zoom", grid.zoom);
		}
	}
	
	/*
	 * TeamView
	 */
	var teamView_show = function(intervalLength, zoom, view) {
	    return function() {
    	    showNode("calendarTeamDay");
            var grid = getCalendarGrid("teamview");
//            if (configContainsKey("gui.calendar.team.zoom")) {
//                grid.zoom = configGetKey("gui.calendar.team.zoom");
//            }
            grid.zoom = zoom;
            grid.setGridIntervalLength(intervalLength);
            grid.setParentDOMNode($("teamViewCalendarGrid"));
            grid.setSelectionChangedEvent("SubSelected");
            grid.setTeamMemberChangedEvent("OX_Teammember_Changed");
            grid.setZoomLevelChangedEvent("OX_Zoom_Level_Changed");
            // is month?
            if (view == "MONTH") {
                grid.setGridIntervalToWeek(null, intervalLength);
            }
            grid.validateOrUpdate();
            register("OX_Zoom_Level_Changed", storeTeamViewSettings);
            calendarGridHandlers.registerAll();
	    };
	};
	
	var teamView_enter = function() {
	 // update current team
        var grid = getCalendarGrid("teamview");
        grid.importTeam();
        // create menu pulldown (if not exists)
        if (!calendarTeamViewChangeTeamPopup) {
            calendarTeamViewChangeTeamPopup = new PulldownMenu($("menu_calendar_teammember_change"));
            calendarTeamViewChangeTeamPopup.onShow = function() {
                this.removeAllItems();
                var teams = configGetKey("gui.calendar.teams");
                if (teams) for (var i = 0; i < teams.length; i++) {
                    this.addItem(new MenuItem(teams[i].name, (function(i) {
                        return function() {
                            var grid = getCalendarGrid("teamview");
                            // delay for better gui responsiveness
                            setTimeout(function() {
                                grid.loadTeamByIndex(i);
                            }, 10);
                        };
                    })(i)));
                }
            };
        }
        if (configGetKey("gui.effects.hover.calendar") && calendarhovers["teamday"]) {
            calendarhovers["teamday"].enable();
        }
    };
    
    var teamView_leave = function() {
        if (calendarhovers["teamday"]) {
            calendarhovers["teamday"].disable();
        }
    };
    
    var teamView_hide = function() {
        hideNode("calendarTeamDay");
        $("calendarTeamDay").style.display = "none";
        unregister("OX_Zoom_Level_Changed", storeTeamViewSettings);
        calendarGridHandlers.unregisterAll();
    };
	
	registerView("calendar/team/day",
	        teamView_show(1, 200, "DAY"),
	        teamView_enter,
	        teamView_leave,
	        teamView_hide,
	        null
	        );
	
	registerView("calendar/team/week",
            teamView_show(7, 100, "WEEK"),
            teamView_enter,
            teamView_leave,
            teamView_hide,
            null
            );
	
	registerView("calendar/team/month",
	        teamView_show(28, 75, "MONTH"),
            teamView_enter,
            teamView_leave,
            teamView_hide,
            null
	        );

    registerView("calendar/team/custom",
            function() {
                // config not available now
                var days = configGetKey("gui.calendar.custom.countdays");
                teamView_show(days, 100)();
            },
            teamView_enter,
            teamView_leave,
            teamView_hide,
            null
            );

	function calendarteamweekrefresh() {
		displayNameDayView();
		loadTeamWeek();
	}
	
	registerView("calendar/team/workweek",
		function() { 
			showNode("calendarTeamWeek");
			if(ctd_participants == null) {
				$("calendarTeamWeek").style.display=""; 
				team_initContent();
			}}, 
		function() { 
			// register
			register("OX_Calendar_Teammember_Add", team_changeTeammembers);
			register("OX_Calendar_Teammember_Remove", team_removeTeammembers);
			register("OX_Selected_Member", team_storeSelectedMember);
			register("OX_Refresh", calendarteamweekrefresh);
			filldefaultteam();
			displayNameDayView();
			loadTeamWeek();
			if (configGetKey("gui.effects.hover.calendar") && calendarhovers["teamworkweek"]) {
				calendarhovers["teamworkweek"].enable();
			}
		 }, 
		 function() {
		 	unloadTeamWeek();
		 	if (calendarhovers["teamworkweek"]) {
				calendarhovers["teamworkweek"].disable();
			}
		 },
		function() { 
			// register
			unregister("OX_Refresh", calendarteamweekrefresh);
			unregister("OX_Calendar_Teammember_Add", team_changeTeammembers);
			unregister("OX_Calendar_Teammember_Remove", team_removeTeammembers);
			unregister("OX_Selected_Member", team_storeSelectedMember);
			hideNode("calendarTeamWeek");
			$("calendarTeamWeek").style.display="none";
		},
		function() { 
			displayNameDayView();
			loadTeamWeek(); 
		}
	);
	registerView("calendar/list",
		function() { 
			showNode("calendarList");
			$("calendarList").style.display=""; 
		}, 
		function() {			
			register("OX_Print",calendar_printList); 
			register("OX_Add_Flag", calendarAddTag);
		}, 
		 function() {
		 	calendarFolderPath.clear();
		 	disableCalendarGrid();
		 	unregister("OX_Print",calendar_printList);
		 	unregister("OX_Add_Flag", calendarAddTag);
		 },
		function() { 
			hideNode("calendarList");
			$("calendarList").style.display="none";
		}
	);
	registerView("calendar/list/day",
		null, 
		function() {
			calendarFolderPath.drawDOMNode("calendaroflistview", activefolder);
			cwwdaysinweek = 1; 
			if(!calendarGrid){
				initCalenderLiveGridOnLoad();
			}
			initLiveGridCalendar("day");
		 }, 
		 null,
		 null,
		 function() {
		 	calendarFolderPath.drawDOMNode("calendaroflistview", activefolder);
		 	displayNameDayView();
		 	initLiveGridCalendar("day"); 
		 }
	);
	registerView("calendar/list/workweek",
		null, 
		function() {
			calendarFolderPath.drawDOMNode("calendaroflistview", activefolder);
			cwwdaysinweek = configGetKey("gui.calendar.workweek.countdays");
            startofweek=configGetKey("gui.calendar.workweek.startday");
			if(!calendarGrid){
				initCalenderLiveGridOnLoad();
			}
			initLiveGridCalendar("workweek");
		 }, 
		 null,
		 null,
		 function() { 		 	
		 	calendarFolderPath.drawDOMNode("calendaroflistview", activefolder);
		 	displayNameDayView();
		 	initLiveGridCalendar("workweek");
		 }
	);
	registerView("calendar/list/month",
		null, 
		function() {
			calendarFolderPath.drawDOMNode("calendaroflistview", activefolder);
			if(!calendarGrid){
				initCalenderLiveGridOnLoad();
			}
			initLiveGridCalendar("month");
		 }, 
		 null,
		 null,
		 function() {
		 	calendarFolderPath.drawDOMNode("calendaroflistview", activefolder);
		 	displayNameDayView();
		 	initLiveGridCalendar("month"); 
		 }
	);
	registerView("calendar/list/week",
		null, 
		function() {
			calendarFolderPath.drawDOMNode("calendaroflistview", activefolder);
			if(!calendarGrid){
				initCalenderLiveGridOnLoad();
			}
			initLiveGridCalendar("week");
		 }, 
		 null,
		 null,
		 function() {
		 	calendarFolderPath.drawDOMNode("calendaroflistview", activefolder);
		 	displayNameDayView();
		 	initLiveGridCalendar("week"); 
		 }
	);
	registerView("calendar/list/search",
		null, 
		function() { 
			if (!calendarGrid){
				initCalenderLiveGridOnLoad();
			}
			initLiveGridCalendar("search");
			register("OX_Delete_Search", calendarDeleteSearch);			
			if (lastCalendarSearch) {			   
               setTimeout(function() {
               	    calendarFolderPath.nodeId="calendaroflistview";
                	calendarFolderPath.drawSearch(lastCalendarSearch.module, lastCalendarSearch.searchstring);
                },0);
            }			
		 }, 
		 function() {
		 	calendarFolderPath.clear();	 	
	 		unregister("OX_Delete_Search", calendarDeleteSearch);
		 },
		 null
	);
	registerView("calendar/detail",
	    // SHOW
		function() { 
			showNode("calendarAppointmentDetail");
			$("calendarAppointmentDetail").style.display="";
	    },
		// ENTER
		function() { 
			register("OX_Refresh",reloadAppDetailView);
			register("OX_AcceptDeny_Changed",reloadAppDetailView);
			register("OX_Confirmation_Change",caldetail_change_Confirm);
			register("OX_Add_Flag", calendarAddTag);
			// added "lastSelectedAppointment" so that this also works with the
			// team view. This view (correctly) clears its selection when leaving
			// the view. so: last selected appointment or normal selection?
			var selectedApp = selectedAppointment || lastSelectedAppointment;
			// flatten array
			if(selectedApp.length) selectedApp = selectedApp[0];
			// id?
			if(selectedApp.id) {
			    // restore selectedAppointment, since the next function uses global magic
			    selectedAppointment = selectedApp;
				detailAppointmentOnload();
			}
			if(typeof selectedAppointment == 'undefined' || selectedAppointment == '') {
				triggerEvent("OX_New_Error", 4, _("No Appointment selected."));
			}
        },
        // LEAVE
        function() {
        	unregister("OX_Refresh",reloadAppDetailView);
        	unregister("OX_AcceptDeny_Changed",reloadAppDetailView);
        	unregister("OX_Confirmation_Change",caldetail_change_Confirm);
        	unregister("OX_Add_Flag", calendarAddTag);
        	objAppDetailParticipants = null;
        },
		function() { 			
			hideNode("calendarAppointmentDetail");
			$("calendarAppointmentDetail").style.display="none";
		}
	);
	registerView("calendar/detail/overview",
		function() {
			register("OX_Print",calendar_printDetail);
			$("tabAppointmentDetail1").style.display="block";
			$("panelAppointmentDetail1").className="tabPanelFirstHi tabPanelHiColor background-color-content font-style-lable border-color-content-default";			
		},
		null, 
		null,
		function() {
			unregister("OX_Print",calendar_printDetail);
			$("tabAppointmentDetail1").style.display="none";
			$("panelAppointmentDetail1").className="tabPanelFirst tabPanelColors font-color-disabled background-color-additional-content border-color-design font-weight-default";			
		}
	);
	registerView("calendar/detail/participant",
		function() {
			$("tabAppointmentDetail2").style.display="block";
			$("panelAppointmentDetail2").className="tabPanelHi tabPanelHiColor background-color-content font-style-lable border-color-content-default";			
		}, 
		function() { 
			refAppDetailParticipantTab.enableGrid();
			currentAppointmentDetailTab = 'PARTICIPANT';
		 }, 
		function() {
			if(a_d_participantGrid && a_d_participantGrid.storage) {
				refAppDetailParticipantTab.disableGrid();
			}
		},
		function() {
			$("tabAppointmentDetail2").style.display="none";
			$("panelAppointmentDetail2").className="tabPanel tabPanelColors font-color-disabled background-color-additional-content border-color-design font-weight-default";			
		}
	);
	var attachmentGridLoaded = false;
	registerView("calendar/detail/attachment",
		function() {
			$("tabAppointmentDetail3").style.display="block";
			$("panelAppointmentDetail3").className="tabPanelHi tabPanelHiColor background-color-content font-style-lable border-color-content-default";			
		}, 
		function() {
			if(!attachmentGridLoaded)
			{
				makeDetailAppointmentAttachmentGrid();
				attachmentGridLoaded = true;
			}
			else
				gridatt.enableGrid();
			gridatt.getAttachments(selectedAppointment.id,selectedAppointment.folder);				
		}, 
		 function() {
		 	if(attachmentGridLoaded)
				gridatt.disableGrid();
		},
 		function() {
			$("tabAppointmentDetail3").style.display="none";
			$("panelAppointmentDetail3").className="tabPanel tabPanelColors font-color-disabled background-color-additional-content border-color-design font-weight-default";			
		}

	);				
}
var ctd_tv_members;
register("Loaded",function() {
	// preparing currently selected members for later use in new appointment, see US532
	register("OX_Teammember_Changed",
			function(members) { 
				var _mem = [];
				for (var i in members) {
					_mem.push([0, 0, members[i].display_name, 0, members[i].id, 0, 0, members[i].type]);
				}
				ctd_tv_members = {};				
				ctd_tv_members["data"] = _mem;
				ctd_tv_members["module"] = "contacts";
			}
	);
	
    calendarFolderPath = new FolderPath("calendar");

    var ids = ["arrowForCWVOptions", "arrowForCWWVOptions",
        "arrowForCMVOptions", "arrowForTDVOptions", "arrowForTWWVOptions",
        "arrowForCLVOptions", "arrowForCDetailVOptions"];
    for (var i in ids) {
        var menu = new PulldownMenu($(ids[i]));
        menu.addItem((new MenuItem("Options"/*i18n*/, showOptions))
                      .setIcon("img/folder/settings.gif"));
    }

    function showOptions() { 
        var elem=currentpath2.join("/");
        if(elem.indexOf("calendar/calendar") == 0 || elem.indexOf("calendar/detail") == 0 || elem.indexOf("calendar/list") == 0) { 
            triggerEvent("OX_Switch_View","configuration/calendar/preferences");
        } else if(elem.indexOf("calendar/team") == 0) { 
            triggerEvent("OX_Switch_View","configuration/calendar/teams");
        }   
    }
});

/*********************************************************************************************************************/
var calendar_activeallfolders=true;
registerPrintView("calendar/list/day");
registerPrintView("calendar/list/workweek");
registerPrintView("calendar/list/month");
registerPrintView("calendar/list/week");
registerPrintView("calendar/list/search");
registerPrintView("calendar/detail/overview");

function calendar_printList() {
	globalprint.printLiveGrid(calendarGrid);
}

//Templates
register("OX_Configuration_Loaded", 
	function() {
		// just enable template printing if bundle is installed and enabled
		if (configContainsKey("modules.calendar.printing") && 
				configGetKey("modules.calendar.printing") == true) {
            registerPrintView("calendar/calendar/day");
			registerPrintView("calendar/calendar/workweek");
			registerPrintView("calendar/calendar/week");
			registerPrintView("calendar/calendar/month");
		}
	}
)

function calendar_printDay() {
    var start = Date.UTC(activeYear, activeMonth, activeDay);
    calendar_print("cp_dayview_table.tmpl", start, start + 864e5); // ms/day
}

function calendar_printWorkweek() {
    var date = new Date(Date.UTC(activeYear, activeMonth, activeDay));
    var start = getDayInSameWeek(date, startofweek) * 864e5; // ms/day
    var numdays = configGetKey("gui.calendar.workweek.countdays");
    calendar_print("cp_weekview_table.tmpl", start,
                   start + numdays * 864e5, numdays); // ms/day
}

function calendar_printWeek() {
    var date = new Date(Date.UTC(activeYear, activeMonth, activeDay));
    var start = getDayInSameWeek(date, weekStart) * 864e5; // ms/day
    calendar_print("cp_weekview_table.tmpl", start, start + 6048e5); // ms/week
}

function calendar_printMonth() {
    calendar_print("cp_monthview_table.tmpl",
        Date.UTC(activeYear, activeMonth, 1),
        Date.UTC(activeYear, activeMonth + 1, 1));
}

function calendar_print(template, start, end, numdays) {
    ox.JSON.get(AjaxRoot + "/printCalendar?start=" + start + "&end=" + end +
        "&template=" + template +
        (calendar_getAllFoldersAttribute() ? "" : "&folder=" + activefolder) +
        "&work_day_start_time=" + getWWStartTime() +
        "&work_day_end_time=" + getWWEndTime() +
        (numdays ? "&work_week_day_amount=" + numdays : "") +
        "&session=" + session, function(reply) {
            var w = newWindow("newInfoItemHidden.html",
                "location=no,menubar=no,resizable=yes, scrollbars=yes,status=no,toolbar=no");
            w.document.open();
            w.document.write(reply);
            w.document.close();
        }, null, true);
}

function default_calendar_create_object(module,folder) {
	if (module=="calendar") {
		var d=new Date();
		var myinterval=15;
		d.setUTCDate(1);
		d.setUTCMonth(activeMonth);
		d.setUTCFullYear(activeYear);
		d.setUTCDate(activeDay);
		d.setUTCHours(d.getHours());
		var mymin=d.getUTCMinutes();
		if(mymin % myinterval) {
			mymin=mymin - (mymin % myinterval)+ myinterval;
		}
		d.setUTCMinutes(mymin);
		calendar_createNewAppointment(d, folder, false, 
				(currentpath2.join("/").match(/^calendar\/team\//) ? ctd_tv_members : null));	
	}
}

function calendar_createNewAppointment(startdate,infolder,wholeday,oFromMail,endDate) {	
	var checkfolder;
	var wholeday=wholeday;
	var folder_id_tocheck  = (infolder)?infolder:((activefolder)?activefolder:config.folder.calendar);
	oMainFolderTree.cache.get_folder(folder_id_tocheck,function (checkfolder){
		calendar_createNewAppointmentCb(checkfolder);
	});

	function calendar_createNewAppointmentCb(myfolder) {		
		if (myfolder.oxfolder.data.module != "calendar") {
			triggerEvent("OX_New_Error",4,_("Can not create appointment in a folder other than appointment folder"));
			
		} else {
			var wholeDay=wholeday;
			if(!startdate) {
				startdate=new Date();
			}
			// Private folder or shared then add owner to list of participants
			if(myfolder.oxfolder.data.type == 1 || myfolder.oxfolder.data.type == 3) {
				var userId = myfolder.oxfolder.data.created_by;
			} else {
				var userId = configGetKey('identifier');
			}
			// end date given?
            endDate = endDate ? endDate.getTime() : "";
            var uri = "newAppointment.html#modul=new&folder_id="+infolder+"&view="+currentCalendarView+"&start_date="+startdate.getTime()+"&end_date="+endDate+"&folderOwner="+userId;
            // is all day?
			if (wholeDay){
			    uri += "&full_time="+wholeDay;
			}
			// open window
			var windowRef = newWindow(uri, getWindowOptions());
			if (oFromMail) {
			    // to add participants for example
				windowRef.sFromModule = oFromMail.module;
				windowRef.oFromMail = oFromMail.data;
			}				
		}
	}
}

var detailAppointmentLoading = false;
	
function reloadAppDetailView(opt_fn, target_folder) {
	
	function cb_reloadAppDetail() {
		detailAppointmentLoading = false;		
		if(currentCalendarTagProgress)
			endProgressCalendarTagging();
		if(opt_fn)
			opt_fn();
	}
	
	if(!detailAppointmentLoading) {
		detailAppointmentLoading = true;	
		
		triggerEvent("OX_Switch_View", "calendar/detail/overview");	
		
		if(selectedAppointment.length && selectedAppointment.length == 1) {
			selectedAppointment = selectedAppointment[0];
		}
		if(selectedAppointment.id)
		{			
			if(target_folder && activefolder != target_folder) {
				var cb_setActiveFolder = function(node) {
					
					triggerEvent("Selected", [selectedAppointment]);
					detailAppointmentOnload(cb_reloadAppDetail);
				}
				selectedAppointment.folder = target_folder;
				setActiveFolder(target_folder,cb_setActiveFolder);
			} else {
				detailAppointmentOnload(cb_reloadAppDetail);
			}
												
		}
	}
}

registerModule("calendar", "Calendar",30); /*i18n*/
registerModuleView("calendar","Calendar",5,{x:0,y:0,ww:1,hw:1});
//TODO
register("OX_New_OXObject",function(mymodule,folder) {
	if(mymodule== "calendar") {
		calendar_createNewAppointment(new Date(Date.UTC(activeYear, activeMonth, activeDay,0,0,0)),folder,false);
	}
});
register("OX_Calendar_Edit", openAppointmentPopup);
register("OX_Calendar_Delete", deleteAppointment);
var currentCalendarRange = "";
var currentCalendarView = "CALENDAR";

	//   1 object_id
	//  20 folder_id
	// 100 categories
	// 101 private flag
	// 200 title
	// 201 start_date
	// 202 end_date
	// 203 description
	// 209 recurrence type 0=none 1=daily 2=weekly 3=monthly 4=yearly
	// 217 notification
	// 220 participents
	// 400 location
	// 401 full_time
	// 402 shown_as	
	// 403 colorlabel
	

fieldsToGet = new Array(1, 20, 207, 206, 2, 200, 201, 202, 203, 220, 401, 402, 209, 400, 101, 102);

function calObj(objFromServer){
	
	var count = 0;
	var startDate;
	this.id = objFromServer[count++];	// objectID			
	this.folderID = objFromServer[count++];	// folderID
	
	for(var q = 2; q < fieldsToGet.length; q++){
		switch (fieldsToGet[q]) {
			case 100:
				this.categories = objFromServer[count++];	// categories
				break;
			case 101:
				this.private_flag = objFromServer[count++];
				break;
			case 102:
				this.color = objFromServer[count++];	// color label
				break;
			case 200:
				this.title = objFromServer[count++];	// title				
				break;
			case 201:
				startDate = new Date(objFromServer[count]);
				this.startDate = new Date(objFromServer[count++]);
				this.start = parseInt(((startDate.getUTCHours() * 60) + startDate.getUTCMinutes())/(60/hoursplit));
				this.starthuman = formatDate(startDate, "time");	// human readable start time
				if((this.starthuman.length - (this.starthuman.lastIndexOf(":"))) < 3)
					this.starthuman = this.starthuman + "0";
				if(this.starthuman.length < 5)
					this.starthuman = "0" + this.starthuman;
				break;
			case 202:
				var endDate = new Date(objFromServer[count]);
				this.endDate = new Date(objFromServer[count++]);
				this.endhuman = formatDate(endDate, "time");	// human readable end time
				var activeDate = (new Date(activeYear, activeMonth, activeDay)).getTime();
				var startDateTmp = (new Date(startDate.getUTCFullYear(), startDate.getUTCMonth(), startDate.getUTCDate())).getTime();
				var endDateTmp = (new Date(endDate.getUTCFullYear(), endDate.getUTCMonth(), endDate.getUTCDate())).getTime();
				if(startDateTmp != endDateTmp){
					if(startDateTmp == activeDate && endDateTmp > activeDate){
						endDate.setUTCMonth(activeMonth);
						endDate.setUTCDate(activeDay);
						endDate.setUTCHours(24);
						endDate.setUTCMinutes(0);
					} else if(startDateTmp < activeDate && endDateTmp > activeDate){
							if(startDate.getUTCDate() != 31){
								startDate.setUTCMonth(activeMonth);
								startDate.setUTCDate(activeDay);
							} else {
								startDate.setUTCDate(activeDay);
								startDate.setUTCMonth(activeMonth);
							}
							startDate.setUTCHours(0);
							startDate.setUTCMinutes(0);
							startDate.setUTCSeconds(0);
							if(endDate.getUTCDate() != 31){					
								endDate.setUTCMonth(activeMonth);
								endDate.setUTCDate(activeDay);
							} else {
								endDate.setUTCDate(activeDay);
								endDate.setUTCMonth(activeMonth);
							}
							endDate.setUTCHours(24);
							endDate.setUTCMinutes(0);
							endDate.setUTCSeconds(0);
							this.start = startDate.getUTCHours() + "." + (parseInt((startDate.getUTCMinutes()/15), 10)*25);	// startdate to calculating
					} else if(startDateTmp < activeDate && endDateTmp == activeDate) {
							if(startDate.getUTCDate() != 31){
								startDate.setUTCMonth(activeMonth);
								startDate.setUTCDate(activeDay);
							} else {
								startDate.setUTCDate(activeDay);
								startDate.setUTCMonth(activeMonth);
							}
							startDate.setUTCHours(0);
							startDate.setUTCMinutes(0);
							this.start = startDate.getUTCHours() + "." + (parseInt((startDate.getUTCMinutes()/15), 10)*25);	// startdate to calculating
					}
				}
				this.start = Number(this.start);
				var minutes = ((endDate.getTime() - startDate.getTime())/60000);
				this.end = minutes/(60/hoursplit); // duration for calculating;
				//TODO calculate the correct length dependent on the hoursplit
				if(this.end > 24*hoursplit)
					this.end = 24*hoursplit-this.start;
				if(this.end == 0)
					this.end += 1;
				if((this.endhuman.length - (this.endhuman.lastIndexOf(":"))) < 3)
					this.endhuman = this.endhuman.substring(0, this.endhuman.indexOf(":")) + ":0" + this.endhuman.substring(this.endhuman.indexOf(":")+1);					
				break;
			case 203:
				this.discription = objFromServer[count++];	// discription
				break;
			case 207:
				this.recurrence_position = objFromServer[count++];
				break;				
			case 209:
				this.recurrence_type = objFromServer[count++];
				break;				
			case 217:
				this.notification = objFromServer[count++];
				break;
			case 220:
				this.participants = objFromServer[count++];
				break;
			case 400:
				this.locat = objFromServer[count++];		// location
				break;
			case 401:
				this.fulltime = objFromServer[count++];	// hole day appointment
				break;	
			case 402:
				this.shown = objFromServer[count++];	// 
				break;
			case 206:
                this.recurrence_id = objFromServer[count++]; 
                break;
            case 2:
                this.created_by = objFromServer[count++];
            default:            
				break;
		}
	}
}	

function displayNameDayView(){
	oMainFolderTree.cache.get_folder(activefolder,displayNameDayViewCb);
}

function calendar_getAllFoldersAttribute() {	
	return configGetKey("gui.calendar.allfolders") && calendar_activeallfolders;
}

function displayNameDayViewCb(oFolder) {
	calendar_activeallfolders = false;
	if (oFolder.oxfolder.data.type == 1 || oFolder.oxfolder.data.type == 3){
        calendar_activeallfolders = oFolder.oxfolder.data.type == 1 ? true : false;
    }	
	if (oFolder.oxfolder.data.created_by) {
		internalCache.getUsers([oFolder.oxfolder.data.created_by], function(cbObj){
			if(oFolder.oxfolder.data.type == 1 || oFolder.oxfolder.data.type == 3) {
				if (oFolder.oxfolder.data.type == 1) {
					$("allAppointmentListViewDiv").style.visibility = "visible";
					$("allAppointmentsWeekViewDiv").style.visibility = "visible";
					$("allAppointmentsWorkWeekViewDiv").style.visibility = "visible";
					$("allAppointmentsMonthViewDiv").style.visibility = "visible";
					calendar_activeallfolders=true;
				} else {
					$("allAppointmentListViewDiv").style.visibility = "hidden";
					$("allAppointmentsWeekViewDiv").style.visibility = "hidden";
					$("allAppointmentsWorkWeekViewDiv").style.visibility = "hidden";
					$("allAppointmentsMonthViewDiv").style.visibility = "hidden";
					calendar_activeallfolders=false;
				}
			} else if (oFolder.oxfolder.data.type == 2) {
				$("allAppointmentListViewDiv").style.visibility = "hidden";
				$("allAppointmentsWeekViewDiv").style.visibility = "hidden";
				$("allAppointmentsWorkWeekViewDiv").style.visibility = "hidden";
				$("allAppointmentsMonthViewDiv").style.visibility = "hidden";
				calendar_activeallfolders=false;
			}
		});
	}
}

function showAllAppointments(){
	if(calendar_getAllFoldersAttribute()){
		$("allAppointmentListView").checked = "";
		configSetKey("gui.calendar.allfolders",false);
		if(currentpath2[1] == "list") {
			initLiveGridCalendar(currentpath2[2]);
		}
	} else {
		$("allAppointmentListView").checked = "checked";
		configSetKey("gui.calendar.allfolders",true);
		if(currentpath2[1] == "list") {
			initLiveGridCalendar(currentpath2[2]);
		} 		
	}
}

function openMailToApp(oDate, oMailObjects, sMod) {
	if (sMod == "contacts") {
		var aIds=new Array();
		for(var indx=0; indx<oMailObjects.length; indx++) {
			aIds.push({id:oMailObjects[indx].id,folder:oMailObjects[indx].folder || oMailObjects[indx].folder_id});
		}
		json.put(AjaxRoot + '/contacts?action=list&session='+session+"&columns="+"1,20,500,555,524,556,557",aIds,null,function (arg){
			calendar_createNewAppointment(oDate,configGetKey("folder.calendar"),false,{module:sMod,data:arg.data});		
		});
	} else if (sMod == "mail") {
		if (oMailObjects.length > 5) {
			oMailObjects = oMailObjects.slice(0,4);
		}
		var collection={};
        collection.objects = oMailObjects;
        collection.columns = OXMailMapping.GETPLAIN;
        OXCache.newRequest(null, "mail", collection, null,
        function (daten) {
            if (daten.objects.length) {
                calendar_createNewAppointment(oDate,configGetKey("folder.calendar"),false,{module:sMod,data:daten.objects});
            }
        });
	} else {
		calendar_createNewAppointment(oDate,configGetKey("folder.calendar"),false,{module:sMod,data:oMailObjects});
	}
}

function openAppointmentPopup(e) {
	triggerEvent("OX_Clear_All_Edit_Lines");	
	var selappoint;

	function edit_whole_series() {
		var app_id = selappoint.recurrence_id || selappoint.id;
		appointmentPopup=newWindow("newAppointment.html#modul=edit&view="+currentCalendarView+"&id="+app_id+"&recurrence_position=0&folder_id="+selappoint.folder, getWindowOptions(), selappoint);
	}
	function edit_single_series() {
		appointmentPopup=newWindow("newAppointment.html#modul=edit&view="+currentCalendarView+"&id="+selappoint.id+"&recurrence_position="+selappoint.recurrence_position+"&folder_id="+selappoint.folder+"&singleappointment=yes", getWindowOptions(), selappoint);
	}
	function edit_single_appointment() {
		appointmentPopup=newWindow("newAppointment.html#modul=edit&view="+currentCalendarView+"&id="+selappoint.id+"&recurrence_position=0&folder_id="+selappoint.folder, getWindowOptions(), selappoint);
	}
	
	if(selectedAppointment.length){
		selappoint=selectedAppointment[0];
	} else {
		selappoint=selectedAppointment;
	}
    if (!menucheckRight("WRITE")) {
        triggerEvent("OX_New_Error", 4,
            _("You do not have write permission for this object."));
    } else if(   selappoint.recurrence_position > 0 && selappoint.recurrence_id
              && selappoint.recurrence_id == selappoint.id)
    {
		newConfirm(_("Edit Appointment"),_("Do you want to edit the whole series or a single series appointment? Please note when changing a serial appointment, exceptions to the series are reset."),AlertPopup.SERIESBUTTON,null,null,null,null,edit_whole_series,edit_single_series);
	} else {
		edit_single_appointment();
	}
	if (e) { stopEvent(e) };
}

function deleteAppointment(){
    
    // the team view has its own handling of deleting appointments
    if (activemodule == "calendar" && currentpath2[1] == "team") {
        calendarGrid_deleteSelectedAppointments();
        return;
    }

    // get latest timestamp
    // start with "now"
	var timestamplocal = new Date().getTime()
	if (activemodule == "calendar") {
		if (currentpath2[1] == "list" && calendarGrid.storage.timestamp) {
		    // get storage timestamp
			timestamplocal = calendarGrid.storage.timestamp;
		} else if (lastUpdateOfCalendarTimestamp) {
		    // get global timestamp
			timestamplocal = lastUpdateOfCalendarTimestamp;
		}
	}
	
	todelete=[];
	if(selectedAppointment.length){
		todelete=selectedAppointment;
	} else { 
		todelete[0]=selectedAppointment;
	}
	for(var i = 0; i<todelete.length;i++){
		if(todelete[i].id.toString().indexOf("_")  != -1)
			todelete[i].id = todelete[i].id.substring(0,todelete[i].id.indexOf("_"));
	}
	var isseries=false;
	for(var x=0; x<todelete.length; x++) {
		if (todelete[x].recurrence_position > 0 && todelete[x].recurrence_id != null) {
			isseries=true;
			break;
		}
	}
	
	var fn_afterDelete_Appoitment = function(cb) {
		triggerEvent("OX_Refresh_Mini_Calendar");
		triggerEvent("OX_After_Delete_Appointment");
		for(var i=0; i<cb.length;i++){
			if(cb[i] && cb[i].error){
				newServerError(cb[i],4);	
			}
		}
		window.setTimeout(function() {
			if(currentpath[0] == "calendar") {
				if(currentpath2[1] == "calendar") {
					window.setTimeout("storageCache.update()", 300);
				} else if(currentpath2[1] == "list") {
					if(currentpath2[2] != "search") {
						storageCache.update();
					} else {
						calendarSearch(lastCalendarSearch.module, lastCalendarSearch.searchstring, null, null, lastCalendarSearch.folder);
					}
				} else if(currentpath2[1] == "detail") {
					triggerEvent("OX_Switch_View",configGetKey("gui.calendar.view"));
				}
			} 
			selectedAppointment={};	
			triggerEvent("Selected",new Array());
		},0);
	}
	
	function delete_single_series() {
		var multipleArray = [];
		for(var i = 0; i < todelete.length; i++){
			multipleArray[i] = {action : "delete", module : "calendar", timestamp : timestamplocal, data : todelete[i]};
		}
		json.put(AjaxRoot + "/multiple?session="+session +"&continue=true", multipleArray, null, fn_afterDelete_Appoitment);	
	}
	function delete_whole_series() {
		var multipleArray = [];
		var deleteobject=new Object();
		var newtodelete=new Array();
		for(var i = 0; i < todelete.length; i++){	
			if(!deleteobject[todelete[i].folder+":"+todelete[i].id]) {
				deleteobject[todelete[i].folder+":"+todelete[i].id]=true;
				newobject=new Object();
				newobject.id=todelete[i].recurrence_id || todelete[i].id;
				newobject.folder=todelete[i].folder;
				newtodelete.push(newobject);
			}
		}
		for (var i=0; i<newtodelete.length; i++) {	
			multipleArray[i] = { action : "delete", module : "calendar", timestamp : timestamplocal, data : newtodelete[i]};
		}
		json.put(AjaxRoot + "/multiple?session="+session +"&continue=true", multipleArray, null, fn_afterDelete_Appoitment);
	}
	if (todelete.length==1) {
		if (isseries) {
			newConfirm(_("Delete Appointment"),_("Do you want to delete the whole series or a single series appointment ?"),AlertPopup.SERIESBUTTON,null,null,null,null,delete_whole_series,delete_single_series);
		} else {
			newConfirm(_("Delete Appointment"),_("Are you sure you want to delete the selected item?"),AlertPopup.YESNO,null,null,delete_whole_series,null,null);
		}
	} else {
		if (isseries) {
			newConfirm(_("Delete Appointment"),_("Do you want to delete the whole series or a single series appointments?"),AlertPopup.SERIESBUTTON,null,null,null,null,delete_whole_series,delete_single_series);
		} else {
			newConfirm(_("Delete Appointment"),_("Are you sure you want to delete the selected items?"),AlertPopup.YESNO,null,null,delete_whole_series,null,null);
		}
		
	}
}
function cancelConflict(){
	$("appointmentConflictWindow").style.display = "none";
	storageCache.update();
}
function ignoreConflict(){
	if(dragObjDayView){
		dragAppointmentDayView.ignore_conflicts = true;
		json.put(AjaxRoot + "/calendar?action=update&session=" + session + "&id=" + dragObjDayView.id + "&timestamp=" + lastUpdateOfCalendarTimestamp + "&folder=" + dragObjDayView.folder, dragAppointmentDayView, null, function(cb){
				if(!cb.error && !cb.data.conflicts){
					storageCache.update();
				}
			});
	}
	dragObjDayView = null;
	$("appointmentConflictWindow").style.display = "none";
}

var cal_numOfObj = null; // last shown number of objects in head line
/**
 * change the number of objects in the folder shown in the headline of each view
 * @param num {Integer} the number to shown in the head line
 */
function calendar_HeaderUpdate(num) {
	if (cal_numOfObj == num) return; // nothing changed so nothing to render
	cal_numOfObj = num;
	calendarFolderPath.additionalNode=document.createTextNode(" (" + cal_numOfObj + ")");
	var view = "calendarofworkweekview";
	switch (currentpath2.join("/")) {
		case "calendar/calendar/month": 
			view = "calendarofmonthview"; 
			break;
		case "calendar/calendar/week":
			view = "calendarofweekview";
			break;
		case "calendar/list/day":
		case "calendar/list/workweek":
		case "calendar/list/month":
		case "calendar/list/week":
			view = "calendaroflistview";
			break;
	}
	calendarFolderPath.drawDOMNode(view, activefolder);
}

fileloaded();
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Andreas Mayer <andreas.mayer@open-xchange.com>
 * 
 */
 
/**
 * define day
 */
function cDay(nDateParam,nCurrMonthParam,bWeekendParam)
{
	this.nDate = nDateParam;
	this.nCurrMonth = nCurrMonthParam; 		 //flags: 0 last month, 1 current month, next month
	this.bWeekend = bWeekendParam;			 //true, false
}

/**
 * define week
 */
function cWeek()
{
	this.nKW;	
	this.aDays= new Array();	
	this.addDay = function (nDay,oDay)
	{
		this.aDays[nDay] = oDay;
	}
	this.setKW = function (nKWParam)	
	{
		this.nKW = nKWParam;
	}
}

/*
 * defines calender:
 */
function cCalendar(nMonthParam,nYearParam)
{
	this.nMonth = nMonthParam;
	this.nYear = nYearParam;
	this.aWeeks= new Array();
	/*
	 * compute Calendar
	 */
	this.addWeek = function (nNumWeek,oWeekParam)
	{
		this.aWeeks[nNumWeek] = oWeekParam;			
	}	
	 
	this.init = function ()
	{
		var oCalendarTool = new calendarTool(this.nMonth, this.nYear);
		oCalendarTool.init();		
		var nNumWeeks = oCalendarTool.getNumWeeks();
		var	startByDate = oCalendarTool.endDate - oCalendarTool.firstDay+1;			
		var nDayCounter = 0; var nDayNextMonth = 1;var nDayCurrMonth = 1;
		for(nWeek=0;nWeek < nNumWeeks;nWeek++)
		{			
			var oWeek = new cWeek();
			var oThDate = new Date();
			oThDate.setFullYear(oCalendarTool.nYear);
			oThDate.setDate(1);	
			oThDate.setMonth(oCalendarTool.nMonth);				
			
			for(indexDay=0;indexDay<7;indexDay++)
			{				
				if(indexDay > 4)
				 	bWeekend = true;
				else				 	
				 	bWeekend = false;				 	
				if(nDayCounter<oCalendarTool.firstDay)
				{
					var oDay = new cDay(startByDate+nDayCounter,0,bWeekend);
					if(indexDay==3) //th
					{
						oThDate.setDate(startByDate+nDayCounter);
						oThDate.setMonth(oThDate.getMonth()-1);						
					}
				}				
				else if(nDayCounter>=(oCalendarTool.nDays+oCalendarTool.firstDay))
				{
					var oDay = new cDay(nDayNextMonth,2,bWeekend);
					if(indexDay==3) //Th
					{
						oThDate.setDate(nDayNextMonth);
						oThDate.setMonth(oThDate.getMonth()+1);		
					}
					nDayNextMonth++;
				}				
				else
				{
					var oDay = new cDay(nDayCurrMonth,1,bWeekend);
					if(indexDay==3)
					{					
						oThDate.setDate(nDayCurrMonth);
						oThDate.setMonth(oThDate.getMonth());
					}
					nDayCurrMonth++;
				}	
				oWeek.addDay(indexDay,oDay);
				oWeek.setKW(oCalendarTool.getKW(oThDate.getFullYear(),oThDate.getMonth(),oThDate.getDate()));				
				nDayCounter++;
			}
			this.addWeek(nWeek,oWeek);
		}		
	}
	
	this.init();	

}

/*
 * tools for calendar computing
 */
function calendarTool(nMonthParam,nYearParam) {
	this.firstDay;
	this.startWeek;
	this.endDate;
	this.nDays;	
	
	this.nMonth = nMonthParam;	
	this.nYear = nYearParam;	
	this.curr_date;	
	this.init = function () {
		var currDate = new Date();
		currDate.setFullYear(this.nYear);
		currDate.setDate(1);
		currDate.setMonth(this.nMonth);
		currDate.setUTCDate(currDate.getUTCDate()-1);

		this.endDate = currDate.getDate();	
		currDate.setUTCDate(currDate.getUTCDate()+1);	
		this.startWeek =  this.getKW(currDate.getFullYear(),currDate.getMonth(),1);
		this.firstDay = currDate.getDay();
		if(this.firstDay>0) this.firstDay -= 1;	
		else if(this.firstDay==0) this.firstDay=6;		
		//compute num days	
		currDate.setMonth(currDate.getMonth()+1);
		currDate.setUTCDate(currDate.getUTCDate()-1);
		this.nDays = currDate.getDate();		
		this.curr_date = currDate;
	}
	
	this.getNumWeeks = function() {	
		var fullWeeks = parseInt(this.nDays/7);
		var overflow = this.nDays - fullWeeks*7;
		if(this.firstDay > 0)
			fullWeeks++;
		if ((7 - this.firstDay) < overflow || (overflow>0 && this.firstDay==0)) {
			fullWeeks++;
		}
		return fullWeeks;
	}
	
	//private functions
	/*
	 * compute calendar week
	 */
	this.getKW = function (j,m,t) {
		var DoDat = this.donnerstag(new Date(j,m,t));
		var DoKW1 = this.donnerstag(new Date(this.getKWYear(j,m,t),0,4));
		var kw = Math.floor(1.5+(DoDat.getTime()-DoKW1.getTime())/86400000/7);
		return kw;
	}
	
	this.getKWYear = function(j,m,t) {
		return this.donnerstag(new Date(j,m,t)).getFullYear();
	}
	
	/*
	 * compute first thursday, getKW need it
	 */
	this.donnerstag = function (datum) {
	  var Do=new Date();
	  Do.setTime(datum.getTime() + (3-((datum.getDay()+6) % 7)) * 86400000);
	  return Do;
	}
	
	this.sonntag = function (datum) { 
	  var Do=new Date();
	  Do.setTime(datum.getTime() + (3-((datum.getDay()+6) % 7)) * 86400000);
	  Do.setUTCDate(Do.getUTCDate()-4);
	  return Do;
	}	
	
	this.init();
}
fileloaded();
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Markus Wagner <markus.wagner@open-xchange.com>
 * 
 */

var calendarGrid = null;
var lastUpdateOfCalendarTimestamp = "";
var idsCalendarForSelected = new Array();
var lastCalendarSearch = {};
function calendarDeleteSearch()
{	
	calendarGrid.sort = function(id, order) {
		initLiveGridCalendar(currentpath2[2]);
	};	
	triggerEvent("OX_Switch_View", menulastviews["calendar"] || configGetKey("gui.calendar.view"));
}
function initCalenderLiveGridOnLoad() {
	
	if (calendarGrid) {
		removeChildNodes($("listeCalendar"));
		calendarGrid = null;
	}
	
    temporary.lists.calendar = temporary.MakeList({
        updated: function() { calendarGrid.recreate(); },
        columns: [
            {   
                text: "&#160;",
                sortable: false,
                width: "16px",
                style: {padding: 0},
                set: function(div, text) {
                    if (div.firstChild)
                        div.firstChild.src = text > 0 ? getFullImgSrc("img/calendar/calendar_series.gif") : getFullImgSrc("img/calendar/calendar16x16.gif");
                    else
                        div.appendChild(newnode("img", 0,
                            {src: text > 0 ? getFullImgSrc("img/calendar/calendar_series.gif") : getFullImgSrc("img/calendar/calendar16x16.gif"), alt: "Serie"}));
                },
                clear: function(div) {
                    if (div.firstChild)
                        div.firstChild.src = getFullImgSrc("img/dummy.gif");
                    else
                        div.appendChild(newnode("img", 0,
                            {src: getFullImgSrc("img/dummy.gif"), alt: "Serie", width: "16", height: "16"}));
                },
                index:5
            }, {    
                text: "&#160;",
                sortable: false,
                width: "16px",
                style: {padding: 0},
                set: function(div, text) {
                    if(text[12] && text[12].length > 1) {
                        var imgSrc = getFullImgSrc("img/calendar/group.gif");
                    } else {
                        var imgSrc = getFullImgSrc("img/dummy.gif");
                    }
                    if (div.firstChild) {
                        div.firstChild.src = !text[6] ? imgSrc : getFullImgSrc("img/private_flag.gif");
                        div.firstChild.alt = !text[6] ? "G" : "P";
                    } else {
                        div.appendChild(newnode("img", 0,
                            {
                                src: !text[6] ? imgSrc : getFullImgSrc("img/private_flag.gif"),
                                alt: !text[6] ? "G" : "P",
                                width: "16", 
                                height: "16"                            
                            }));
                    }
                },
                clear: function(div) {
                    if (div.firstChild)
                        div.firstChild.src = getFullImgSrc("img/dummy.gif");
                    else
                        div.appendChild(newnode("img", 0,
                            {src: getFullImgSrc("img/dummy.gif"), alt: "G", width: "16", height: "16"}));
                }
            }, {
                i18n: "description", /*i18n context*/
                text: "Title", /*i18n*/
                sortable: true,
                id: 200,
                set: LiveGrid.defaultSet,
                clear: LiveGrid.makeClear(""),
                index:7
            }, {
                text: "Start", /*i18n*/
                sortable: true,
                i18n: true,
                id: 201,
                width: "14%",
                set: function(div, data) {
                    if (!div.firstChild) {
                        div.appendChild(document.createTextNode(" "));
                    }
                    if (data) {
                        var dateToShow = data[8];
                        if (data[13] == true) {
                            div.firstChild.data=formatDate(dateToShow, "date");
                        } else {
                            div.firstChild.data=formatDate(dateToShow, "datetime");
                        }           
                    } else {
                        div.firstChild.data=""; 
                    }               
                },
                clear: LiveGrid.makeClear("")
            }, {
                text: "End", /*i18n*/
                sortable: true,
                i18n: true,
                id: 202,
                width: "14%",
                set: function(div, data) {
                    if (!div.firstChild) {
                        div.appendChild(document.createTextNode(" "));
                    }
                    if (data) {
                        var dateToShow = data[9];
                        /*
                         * if it's a full day appointment it's minus one day
                         */
                        if (data[13] == true) {
                            dateToShow -= 864e5; /* ms/day */
                            div.firstChild.data=formatDate(dateToShow, "date");
                        } else {
                            div.firstChild.data=formatDate(dateToShow, "datetime");
                        }           
                    } else {
                        div.firstChild.data=""; 
                    }               
                },
                clear: LiveGrid.makeClear("")
            }, {
                text: "Location", /*i18n*/
                sortable: true,
                i18n: true,
                id: 400,
                width: "10%",
                set: LiveGrid.defaultSet,
                clear: LiveGrid.makeClear(""),
                index:10
            }, {
                text: "Folder", /*i18n*/
                sortable: false,
                i18n: true,
                id: 20,
                width: "14%",
                set: function(div,text) {
                        if(text > 0){
                            oMainFolderTree.get_folder_path(text, function(path){
                                var tmp = path.split("/");
                                if(tmp.length == 1){
                                    if(div.firstChild)
                                        div.firstChild.data = tmp[tmp.length-1];
                                    else
                                        div.appendChild(document.createTextNode(tmp[tmp.length-1]));
                                }
                                else {
                                    if(div.firstChild)
                                        div.firstChild.data = tmp[tmp.length-2];
                                    else
                                        div.appendChild(document.createTextNode(tmp[tmp.length-2]));
                                }
                            });
                        } else {
                            if(div.firstChild)
                                div.firstChild.data = " ";
                            else
                                div.appendChild(document.createTextNode(" "));
                        }
                },
                clear: LiveGrid.makeClear(""),
                index:1
            }, {
                text: "Flag", /*i18n*/
                sortable: true,
                i18n: true,
                id: 102,
                width: "4em",
                set: function(div,text) {
                    if (div.firstChild)
                        div.firstChild.src = getFullImgSrc("img/menu/tag_" + (text||0) + ".gif");
                    else
                        div.appendChild(newnode("img", 0, {src: getFullImgSrc("img/menu/tag_" + (text||0) + ".gif"), alt: "tag"}));
                },
                clear: function(div) {
                    if (div.firstChild)
                        div.firstChild.src = getFullImgSrc("img/dummy.gif");
                    else
                        div.appendChild(newnode("img", 0, {src: getFullImgSrc("img/dummy.gif"), alt: "tag", width: "16", height: "16"}));
                },
                index:11
            }
        ]
    });
	
	calendarGrid = new LiveGrid(temporary.lists.calendar.columns,
        calendarSelection, "calendar");
	calendarGrid.contextmenu = globalContextMenus.calendar;
	
    calendarGrid.sort = function(id, order) {
		initLiveGridCalendar(currentpath2[2]);
	};
	
	calendarGrid.sort_id = 201;
	calendarGrid.sort_order = "asc";

	$("listeCalendarHeader").appendChild(calendarGrid.getHeader());
	calendarGrid.getTable($("listeCalendar"));
	
	// livegrid double click 
	calendarGrid.events.register("Activated", function(ids){
		if(ids && ids.length > 0) {
			if(menuhasRight(ids,"WRITE")) {
				selectedAppointment = ids;
				openAppointmentPopup();
			} else {
				triggerEvent("OX_New_Error", 4, _("You do not have write permission for this object."));
			}
		} else {
			triggerEvent("OX_Before_Create_Calendar");
		}
	});
	
	// what to do when select a appointment
	calendarGrid.events.register("Selected", function(ids){
		selectedAppointment = calendarGrid.selection.getSelected();
		triggerEvent("Selected", selectedAppointment);
	});
	
	calendarGrid.events.register("Delete", function(ids){
		if(menucheckRight("DELETE"))
			deleteAppointment();
		else
			triggerEvent("OX_New_Error", 4, _("You do not have write permission for this object."));
	});
	var hover = calendarGrid.addHover(OXAppointmentHover.getContent().node, function(id) {
		OXAppointmentHover.actualHover=hover;
		OXAppointmentHover.refillContent(id.id,id.folder,id.recurrence_position);					
	});
	hover.setSize(OXAppointmentHover.contentobject.node);
		
	calendarGrid.events.register("Changed", function(data) { calendar_HeaderUpdate(data.ids.length); });
}
var cwwcurrentmonday
function initLiveGridCalendar(range) {
	showallappointments=calendar_getAllFoldersAttribute();
	$("allAppointmentListView").checked=showallappointments;
	
	var start = null;
	var end = null;
	switch(range){
		case "day":	
				start = Date.UTC(activeYear, activeMonth, activeDay);
				end = Date.UTC(activeYear, activeMonth, (activeDay+1));
				break;
		case "month":
				start = Date.UTC(activeYear, activeMonth, 1);
				end = Date.UTC(activeYear, activeMonth + 1, 1);
				break;
		case "workweek":
		        start = getDayInSameWeek(new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0)), startofweek) * 864e5;		        
				end = start +(cwwdaysinweek*1000*60*60*24);
				break;
		case "week":
				start = getDayInSameWeek(new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0)), weekStart) * 864e5;
				end = start + 604800000;
				break;
		case "search":
				calendarGrid.sort = function (sort, order) {
					calendarSearch(lastCalendarSearch.module, 
					       lastCalendarSearch.searchstring,
					       { sort: sort, order: order},
					       null, lastCalendarSearch.folder
                        )};
				calendarGrid.disablehover=!configGetKey("gui.effects.hover.calendar");
				calendarGrid.enable(tmpStorage);
				break;
	}
	if(end != null && start != null){
		if(!calendar_getAllFoldersAttribute()){
				var options = {action: "all", folder: activefolder, start : start, end : end, columns : "1, 20, 207, 206, 2"};
				if (calendarGrid.sort_id) {
					options.sort = calendarGrid.sort_id;
					options.order = calendarGrid.sort_order;
				}
				storageCache.get(
					AjaxRoot + "/calendar",
					options,
					function(storage) {
						calendarGrid.disablehover=!configGetKey("gui.effects.hover.calendar");
						calendarGrid.enable(storage);
						storageCache.update();
					},
					"209,101,200,201,202,400,102,220,401",
					true,
					calendar_serializeid,
					calendar_makeID
				);
		} else {
			var options = {action: "all", start : start, end : end, columns : "1, 20, 207, 206, 2"};
			if (calendarGrid.sort_id) {
				options.sort = calendarGrid.sort_id;
				options.order = calendarGrid.sort_order;
			}
			storageCache.get(
				AjaxRoot + "/calendar",
				options,
				function(storage) {
					calendarGrid.disablehover=!configGetKey("gui.effects.hover.calendar");
					calendarGrid.enable(storage);
					storageCache.update();
				},
				"209,101,200,201,202,400,102,220,401",
				true,
				calendar_serializeid,
				calendar_makeID
			);
		}
	}
	updateHeaderListView();
}

function goToNextListView(direction){
	var tmp = new Date(Date.UTC(activeYear, activeMonth, activeDay));
	var start = null;
	var end = null;
	if(direction > 0){
		switch(currentpath2[2]){
			case "day":
				tmp.setTime(tmp.getTime() + 86400000*1);
				break;
			case "workweek":
				tmp.setTime(tmp.getTime() + 86400000*7);
				break;
			case "week":
				tmp.setTime(tmp.getTime() + 86400000*7);
				break;
			case "month":
				tmp.setUTCDate(1);
				tmp.setUTCMonth(activeMonth+1);
				tmp.setUTCDate(activeDay);
				break;
		}
	} else {
		switch(currentpath2[2]){
			case "day":
				tmp.setTime(tmp.getTime() - 86400000*1);
				break;
			case "workweek":
				tmp.setTime(tmp.getTime() - 86400000*7);
				break;
			case "week":
				tmp.setTime(tmp.getTime() - 86400000*7);
				break;
			case "month":
				tmp.setUTCDate(1);
				tmp.setUTCMonth(activeMonth-1);
				tmp.setUTCDate(activeDay);
				break;
		}
	}
	
	activeYear = tmp.getUTCFullYear();
	activeMonth = tmp.getUTCMonth();
	activeDay = tmp.getUTCDate();
	
	if (oMiniCalendar) oMiniCalendar.setSelectedByDate(activeYear, activeMonth, activeDay);
		
	switch(currentpath2[2]){
		case "day":	
				start = Date.UTC(activeYear, activeMonth, activeDay);
				end = Date.UTC(activeYear, activeMonth, (activeDay+1));
				break;
		case "month":
				start = Date.UTC(activeYear, activeMonth, 1);
				end = Date.UTC(activeYear, activeMonth+1, 1);
				break;
		case "workweek":
				start = getDayInSameWeek(new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0)), startofweek) * 864e5;
				end = start + 435600000;
				break;
		case "week":
				start = getDayInSameWeek(new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0)), weekStart) * 864e5;
				end = start + 604800000;
				break;
		case "search":
				calendarGrid.disablehover=!configGetKey("gui.effects.hover.calendar");
				calendarGrid.enable(tmpStorage);
				break;
	}
	if(end != null && start != null){
		if(!$("allAppointmentListView").checked){
				var options = {action: "all", folder: activefolder, start : start, end : end, columns : "1, 20, 207, 206, 2"};
				if (calendarGrid.sort_id) {
					options.sort = calendarGrid.sort_id;
					options.order = calendarGrid.sort_order;
				}
				storageCache.get(
					AjaxRoot + "/calendar",
					options,
					function(storage) {
						calendarGrid.disablehover=!configGetKey("gui.effects.hover.calendar");
						calendarGrid.enable(storage);
						storageCache.update();
					},
					"209,101,200,201,202,400,102,220,401",
					true,
					calendar_serializeid,
					calendar_makeID
				);
		} else {
			var options = {action: "all", start : start, end : end, columns : "1, 20, 207, 206, 2"};
			if (calendarGrid.sort_id) {
				options.sort = calendarGrid.sort_id;
				options.order = calendarGrid.sort_order;
			}
			storageCache.get(
				AjaxRoot + "/calendar",
				options,
				function(storage) {
					calendarGrid.disablehover=!configGetKey("gui.effects.hover.calendar");
					calendarGrid.enable(storage);
					storageCache.update();
				},
				"209,101,200,201,202,400,102,220,401",
				true,
				calendar_serializeid,
				calendar_makeID
			);
		}
	}
	updateHeaderListView();
}

function updateHeaderListView() {
	switch(currentpath2[2]){
		case "day":
			$("calendarlistheaderdiv").style.visibility = "visible";
			$("calendarlistheader").firstChild.data=formatDate(new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0)), "dateday");
			break;
		case "week": //No break
		case "workweek":
			$("calendarlistheaderdiv").style.visibility = "visible";		
			var oCalendarTool = new calendarTool(activeMonth,activeYear);
			$("calendarlistheader").firstChild.data = _("CW") + " " + 
					formatDateTime("w yyyy", new Date(getKeyDayOfWeek(new Date(Date.UTC(activeYear,activeMonth,activeDay))) * 864e5));
			break;
		case "month":
			$("calendarlistheaderdiv").style.visibility = "visible";		
			$("calendarlistheader").firstChild.data = formatDateTime("MM/yyyy", new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0)));
			break;
		case "search":
			$("calendarlistheaderdiv").style.visibility = "hidden";
			$("calendarlistheader").firstChild.data = ' ';
			break;
	}
}

function disableCalendarGrid(){
	if(calendarGrid)
		calendarGrid.disable();
	unregister("OX_Add_Flag", calendarAddTag);
}
var currentCalendarTagProgress = false;
function calendarAddTag(tag, editSingleSeriesOnly) {
	if(currentCalendarTagProgress) return;
	currentCalendarTagProgress = true;
	var tmpselected;
	if(currentpath2[0] == "calendar" && currentpath2[1] == "list" && storageCache.current){
		lastUpdateOfCalendarTimestamp = storageCache.current.timestamp;
	}
	function edit_whole_series() {
	    // process appointments in order to remove the recurrence position
		for(var i=0;i<tmpselected.length;i++) {
			tmpselected[i] = { id: tmpselected[i].id, folder: tmpselected[i].folder };
		}
		setTag(tag, tmpselected, lastUpdateOfCalendarTimestamp, finalSetTag);
	}
	function edit_single_series() {
		setTag(tag, tmpselected, lastUpdateOfCalendarTimestamp, finalSetTag);			
	}
	if(selectedAppointment.length){
		tmpselected=selectedAppointment;
	} else { 
		tmpselected=[selectedAppointment];
	}
	var editSingleSeriesOnly = !!editSingleSeriesOnly;
	var isseries=false;
	for(var x=0;x<tmpselected.length;x++) {
		if(tmpselected[x].recurrence_position>0 && tmpselected[x].recurrence_id &&
		   tmpselected[x].recurrence_id == tmpselected[x].id) {
			isseries=true;
			break;
		}
	}
	if(isseries && !editSingleSeriesOnly) {
		if(tmpselected.length>1) {
			newConfirm(_("Edit Appointment"),_("Do you want to edit the whole series or single series appointments? Please note when changing a serial appointment, exceptions to the series are reset."),AlertPopup.SERIESBUTTON,null,null,null,null,edit_whole_series,edit_single_series);			
		} else {
			newConfirm(_("Edit Appointment"),_("Do you want to edit the whole series or a single series appointment? Please note when changing a serial appointment, exceptions to the series are reset."),AlertPopup.SERIESBUTTON,null,null,null,null,edit_whole_series,edit_single_series);						
		}		
	} else {
		setTag(tag, tmpselected, lastUpdateOfCalendarTimestamp, finalSetTag);
	}
	
	function finalSetTag(timestamp) {	
		lastUpdateOfCalendarTimestamp = timestamp;
		if(currentpath2[0] == "calendar" && currentpath2[1] == "detail")
			reloadAppDetailView();
		else
			updateCalendarViewCbSetTag(timestamp);
		// notify
		triggerEvent("OX_After_Update_Appointment");
	}
}
var tmpStorage;
function endProgressCalendarTagging() {
	window.setTimeout(function(){currentCalendarTagProgress = false;},0);	
}

function updateCalendarViewCbSetTag(currentTagTimestamp) {
	
	if(currentpath2[0] == "calendar" && currentpath2[1] != "detail" && currentpath2[2] != "search"){
	window.setTimeout('endProgressCalendarTagging()',0);
		if(storageCache)
			storageCache.update(endProgressCalendarTagging);
	} else if(currentpath2[2] == "search"){
		if(currentTagTimestamp) {
			if(storageCache && storageCache.current)
				storageCache.current.timestamp = currentTagTimestamp;
		}
		calendarSearch(lastCalendarSearch.module, lastCalendarSearch.searchstring, null, endProgressCalendarTagging, lastCalendarSearch.folder);
	}
}

//var sPatternCalendarsearch;
function calendarSearch(module, searchstring, order, callback, folder) {
	//var sPatternCalendarsearch = searchstring;
	tmpStorage = new Storage(lastUpdateOfCalendarTimestamp, [], "", 0, 0, calendar_serializeid, calendar_makeID);
	lastCalendarSearch.module = module;
	lastCalendarSearch.searchstring = searchstring;
	lastCalendarSearch.folder = folder || (calendar_getAllFoldersAttribute() ? null : activefolder);	
	var sOrderString="";
	if (order) {
		sOrderString = "&sort=" + order.sort + "&order=" + order.order;
	}
	var sObject = { };
	sObject["pattern"] = searchstring;
	if (lastCalendarSearch.folder) {
		sObject["folder"] = lastCalendarSearch.folder;
	}
	json.put(AjaxRoot + "/calendar?action=search&session=" + session 
	       + "&columns=1,20,207,206,2,209,101,200,201,202,400,102,220,401" 
	       + sOrderString, sObject, null, 
				function(cb){
					if(callback) callback();
					tmpArray = new Array();
					for(var i=0; i < cb.data.length; i++){
						tmpArray[i] = [	cb.data[i][0],
										cb.data[i][1],
										cb.data[i][2],
										cb.data[i][3],
										cb.data[i][4],
										cb.data[i][5],
										cb.data[i][6],
										cb.data[i][7],
										cb.data[i][8],
										cb.data[i][9],
										cb.data[i][10],
										cb.data[i][11],
										cb.data[i][12],
										cb.data[i][13]
									]; 
					}
					tmpStorage.append(tmpArray);
					lastUpdateOfCalendarTimestamp = cb.timestamp;
					storageCache.current.timestamp = cb.timestamp;
					if(!calendarGrid){
						triggerEvent("OX_Switch_View", "calendar/list/search");
					} else if(currentpath2[2] != "search"){
						triggerEvent("OX_Switch_View", "calendar/list/search");
					} else {
						calendarGrid.disablehover=!configGetKey("gui.effects.hover.calendar");
						calendarGrid.enable(tmpStorage);
					}					
	});	
}

fileloaded();
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Andreas Mayer <andreas.mayer@open-xchange.com>
 * 
 */
// TEST DATA
var nNumCharAppointmentView = 10;
var calendarJson = new JSON();
//Global Vars
var monthViewCreated = false;
var calendarObjects;
var nMonthGlob; 
var nYearGlob;
var tmptimeOutDouble = false;
var tmptimeOutSingle = null;
var aAppointmentAssoc = new Object();
var sAssocMonthArray = new Object();
var aSelectedAppArray = new Array();
var oMonthViewObj;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//GUI Actions

function showAllAppointmentsMonth(){
	if(!$("allAppointmentMonthView").checked){
		configSetKey("gui.calendar.allfolders",false);
		oMonthViewObj.refreshMonthView(false);
	} else {
		configSetKey("gui.calendar.allfolders",true);
		oMonthViewObj.refreshMonthView(true);
	}
}

function refreshMonthView(changeFolder) {
	if (changeFolder!=undefined && changeFolder) {
		oMonthViewObj.refreshMonthView(calendar_getAllFoldersAttribute());
	} else {
		function fn() {
			if(!configContainsKey("gui.calendar.allfolders"))	
				oMonthViewObj.refreshMonthView(calendar_getAllFoldersAttribute());
		}
		//oMonthViewObj.getStorage();
	}
}

function resizeWeekSections() {	
	var calendar = $("calmain");
	var numWeeks=oMonthViewObj.oCalendarTool.getNumWeeks();
	var workWeekHeight = 0;
	var weekEndHeight = 0;
	var height = (calendar.offsetHeight - pxPerEm)/numWeeks;
	for(var indx=0;indx<calendar.childNodes.length;indx++)
	{
		var oCurrentNode =calendar.childNodes[indx];
		for(var subIndx=0;subIndx<oCurrentNode.childNodes.length;subIndx++)
		{
			var oCurrentSubNode = oCurrentNode.childNodes[subIndx];
			if(oCurrentSubNode && oCurrentSubNode.tagName == "DIV" && height>0)
			{
				if((height)>0)
				{
					oCurrentSubNode.style.height = (height) + "px";	
					workWeekHeight = (height);
				}

				if(oCurrentSubNode.className.indexOf("weekendInMonthView") != -1)
				{
					for(var subSubIndx=0;subSubIndx<oCurrentSubNode.childNodes.length;subSubIndx++)
					{
						var oCurrentSubSubNode = oCurrentSubNode.childNodes[subSubIndx];
						if(oCurrentSubSubNode && oCurrentSubSubNode.tagName == "DIV")
						{
							if((height/2)>0)
							{
								oCurrentSubSubNode.style.height = (height/2) + "px";	
								weekEndHeight = (height/2);	
							}
						}
					}
				}				
			}				
		}
		var aSplittId = oCurrentNode.id ? oCurrentNode.id.split("_") : [];
		if(aSplittId[0] == "weekmonth" || true) {
			if((height)>0 && oCurrentNode.style)
				oCurrentNode.style.height = (height) + "px";
		}
	}
	oMonthViewObj.redrawAppointments(workWeekHeight,weekEndHeight);
}

function loadMonthView() {
	if (!oMonthViewObj) {
		oMonthViewObj = new monthView();
		oMonthViewObj.loadMonthView(calendar_getAllFoldersAttribute());
		//resizeEvents.register("Resized",resizeWeekSections);	
	} else {
		if (oMonthViewObj.oCalendarTool.nMonth == activeMonth &&
		    oMonthViewObj.oCalendarTool.nYear == activeYear) {
			function fn() {
				oMonthViewObj.refreshMonthView(calendar_getAllFoldersAttribute());
			}
			//storageCache.update(fn);
			oMonthViewObj.getStorage(fn);
		} else {
			goToActiveMonth();
		}
	}
}

function goToActiveMonth() {
	if (monthViewCreated) {
		aSelectedAppArray = new Array();
		var oDateLoc = new Date();
		oDateLoc.setFullYear(activeYear);
		oDateLoc.setDate(1);	
		oDateLoc.setMonth(activeMonth); 		
		oMonthViewObj.oCalendarTool = new calendarTool(activeMonth, activeYear);	
		oMonthViewObj.updateMonthView();
		refreshMonthView();
	}
}

function goToNextMonth(nDirection) {
	if (monthViewCreated) {
		aSelectedAppArray = new Array();
		var oDateLoc = new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0)); 
		oDateLoc.setUTCDate(1);   
        oDateLoc.setUTCMonth(oDateLoc.getUTCMonth()+(nDirection));
        activeMonth = oDateLoc.getUTCMonth();
        activeYear = oDateLoc.getUTCFullYear();     
        activeDay = oDateLoc.getUTCDate();					
		if (oMiniCalendar) oMiniCalendar.setSelectedByDate(activeYear, activeMonth, activeDay);
		oMonthViewObj.oCalendarTool = new calendarTool(activeMonth, activeYear);	
		oMonthViewObj.updateMonthView();
		refreshMonthView();
	}
}

function monthAction(evt) {	
	var sId = '';
	var DOMTarget = (evt.target)?evt.target:evt.srcElement;	
	if (DOMTarget.id.substr(0,4) == 'but_') {
		if(evt.type == "click") {
			sId = DOMTarget.id.substr(4) + '_num';
			if (tmptimeOutSingle != null) {
				clearTimeout(tmptimeOutSingle);
				tmptimeOutSingle = null;
				stopEvent(evt);	
				return;
			}
			tmptimeOutSingle = setTimeout(function (){
				        var oDate = new Date();
						oDate.setDate(1);		
						oDate.setFullYear(sAssocMonthArray[sId].year);				
						oDate.setMonth(sAssocMonthArray[sId].month);				
						oDate.setDate(sAssocMonthArray[sId].day.nDate);		
						if (sAssocMonthArray[sId].day.nCurrMonth == 0)
							oDate.setMonth(oDate.getMonth()-1);
						else if (sAssocMonthArray[sId].day.nCurrMonth == 2)			
							oDate.setMonth(oDate.getMonth()+1);
						activeDay = oDate.getDate();
						activeMonth = oDate.getMonth();
						activeYear = oDate.getFullYear();				
						triggerEvent("OX_Switch_View","calendar/calendar/day");
						tmptimeOutSingle = null;				
					}, 300);
			stopEvent(evt);		
			return;
		}
	} else {
		sId = DOMTarget.id;
	}

	if (sId.substr(0,1) == 'd' && sId.split('_')[3] == 'num') {		
		if (evt.type == "dblclick") {
			if (tmptimeOutSingle != null) {
				clearTimeout(tmptimeOutSingle);			
			}
			if (sId.substr(0,1) == 'd' && sId.split('_')[3] == 'num') {
				var oDate = new Date();
				oDate.setUTCDate(1);		
				oDate.setUTCFullYear(sAssocMonthArray[sId].year);				
				oDate.setUTCMonth(sAssocMonthArray[sId].month);					
				if (sAssocMonthArray[sId].day.nCurrMonth == 0) {
					oDate.setUTCMonth(oDate.getUTCMonth()-1);
				} else if(sAssocMonthArray[sId].day.nCurrMonth == 2) {											
					oDate.setUTCMonth(oDate.getUTCMonth()+1);
				}
				oDate.setUTCDate(sAssocMonthArray[sId].day.nDate);						
				activeDay = oDate.getUTCDate();
				activeMonth = oDate.getUTCMonth();
				activeYear = oDate.getUTCFullYear();		
				calendar_activedate=new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0));
				calendar_wholeday=true;		
				triggerEvent('OX_Before_Create_Calendar');
				stopEvent(evt);		
			}				
		}
		
	} else if (sId.substr(0,1) == 'd' && sId.split('_')[4] != 'app' && 
	           sId.split('_')[3] != 'title' && sId.split('_')[3] != 'headcontainer') {
		sId = DOMTarget.id+'_num';
		var oDate = new Date();
		oDate.setUTCDate(1);		
		oDate.setUTCFullYear(sAssocMonthArray[sId].year);				
		oDate.setUTCMonth(sAssocMonthArray[sId].month);				
		if (sAssocMonthArray[sId].day.nCurrMonth == 0)
			oDate.setUTCMonth(oDate.getUTCMonth()-1);
		else if (sAssocMonthArray[sId].day.nCurrMonth == 2)			
			oDate.setUTCMonth(oDate.getUTCMonth()+1);
		oDate.setUTCDate(sAssocMonthArray[sId].day.nDate);					
		activeDay = oDate.getUTCDate();
		activeMonth = oDate.getUTCMonth();
		activeYear = oDate.getUTCFullYear();
		var startTime = configGetKey("gui.calendar.starttime") || 8;
		calendar_activedate=new Date(Date.UTC(activeYear,activeMonth,activeDay,startTime,0,0));
		calendar_wholeday=false;		
		triggerEvent('OX_Before_Create_Calendar');
		stopEvent(evt);
		
	} else if (sId.substr(0,1) == 'd' && sId.split('_')[4] != 'app' && 
	           sId.split('_')[3] == 'title' && sId.split('_')[3] != 'headcontainer') {
		var sFakeId = sId.split('_')[0]+'_'+sId.split('_')[1] + '_' + sId.split('_')[2] + '_num';
		var oDate = new Date();
		oDate.setDate(1);		
		oDate.setFullYear(sAssocMonthArray[sFakeId].year);				
		oDate.setMonth(sAssocMonthArray[sFakeId].month);				
		oDate.setDate(sAssocMonthArray[sFakeId].day.nDate);		
		if (sAssocMonthArray[sFakeId].day.nCurrMonth == 0)
			oDate.setMonth(oDate.getMonth()-1);
		else if (sAssocMonthArray[sFakeId].day.nCurrMonth == 2)			
			oDate.setMonth(oDate.getMonth()+1);
		activeDay = oDate.getDate();
		activeMonth = oDate.getMonth();
		activeYear = oDate.getFullYear();		
	    var startTime = configGetKey("gui.calendar.starttime") || 8;
		calendar_activedate = new Date(Date.UTC(activeYear,activeMonth,activeDay,startTime,0,0));
		calendar_wholeday = false;		
		triggerEvent('OX_Before_Create_Calendar');
		stopEvent(evt);
	}
	
	stopEvent(evt);		
}

function selectDayInMonthView(evt) {
	var DOMTarget = (evt.target) ? evt.target : evt.srcElement;    
	var sDOMId = DOMTarget.id || DOMTarget.parentNode.id || null;
	if (sDOMId == null) return;
	var sId = sDOMId.match(/^[a-z]_[0-9]_[0-9]/)[0] + "_num";
	for (var i in sAssocMonthArray) {
	    if (sAssocMonthArray[i].active && sAssocMonthArray[i].active == true) {
	        sAssocMonthArray[i].active=false;
	        $2(i.substr(0,5)+"_headcontainer").className="headerContStyle";
	    } 
	}
	sAssocMonthArray[sId].active=true;
	$2((sDOMId.match(/^[a-z]_[0-9]_[0-9]/)[0]) + "_headcontainer").className = 
	       "headerContStyle background-color-PMG-selection-elements";

	var oDate = new Date();
	oDate.setUTCDate(1);        
	oDate.setUTCFullYear(sAssocMonthArray[sId].year);               
	oDate.setUTCMonth(sAssocMonthArray[sId].month);             
	if (sAssocMonthArray[sId].day.nCurrMonth == 0)
	    oDate.setUTCMonth(oDate.getUTCMonth()-1);
	else if (sAssocMonthArray[sId].day.nCurrMonth == 2)          
	    oDate.setUTCMonth(oDate.getUTCMonth()+1);
	oDate.setUTCDate(sAssocMonthArray[sId].day.nDate);                  
	activeDay = oDate.getUTCDate();
	activeMonth = oDate.getUTCMonth();
	activeYear = oDate.getUTCFullYear();
	var startTime = configGetKey("gui.calendar.starttime") || 8;
	calendar_activedate=new Date(Date.UTC(activeYear,activeMonth,activeDay,startTime,0,0));
	calendar_wholeday=false;
	
	if (!sDOMId.match(/_app$/gi)) {
        if (calendarSelection.count != 0) calendarSelection.reset();
        deselectAppointments();
        triggerEvent("Selected",aSelectedAppArray);
	}
	//stopEvent(evt);
}

function monthView() {
	this.dateIdAssoc = new Object();
	this.aAppointmentAssoc = new Object();
	this.calendarwfields = new Array(200, 201, 202, 203, 209, 221, 401, 402, 102, 400);
	this.oCalendarTool = new calendarTool(activeMonth, activeYear);
	this.oCalendar;
	this.storage = null;	
	this.timestamp = null;
	this.show_all = false;
	this.today = new Date();
	this.calendar_start = null;
	this.calendar_end = null;	
	this.redraw = function (){};
	this.show_we = 1;
	this.show_ww = 2;
}

function moveAppToNewStart(objc, startTimeParam, endTimeParam, fn_opt) {
	if (objc["full_time"])
		editAppointment(objc,{start_date:startTimeParam.getTime(),end_date:endTimeParam.getTime(),full_time:true},fn_opt);
	else
		editAppointment(objc,{start_date:startTimeParam.getTime(),end_date:endTimeParam.getTime()},fn_opt);
}

monthView.prototype = {
	updateHeader: function() {
		$("monthviewheader").firstChild.data = 
		       formatDateTime("MM/yyyy", new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0)));
	},
	
	loadMonthView: function(bAll) {
		this.show_all = bAll;
		if(calendar_getAllFoldersAttribute()){
			$("allAppointmentMonthView").checked = true;
		}
		else
			$("allAppointmentMonthView").checked = false;
		this.createMonthView(activeMonth, activeYear);
	},
	
	updateMonthView:function() {
	    //Old, use refresh function
		calendarObjects = new Array();
		this.dateIdAssoc = new Object();
		this.aAppointmentAssoc = new Object();
		sAssocMonthArray = new Object();
		var calendar = $("calmain");
		while (calendar.childNodes.length>0) {
			calendar.removeChild(calendar.firstChild);
		}
		monthViewCreated = false;
		this.createMonthView(activeMonth, activeYear);
	},
	
	createMonthView: function(nMonth, nYear) {
		var Self = this;
		if (!monthViewCreated) {
			for(var i=0;$2("kwnumber_"+i);i++) {
				removeTMPId("kwnumber_"+i);
			}	
			for(var i=0;$2("weekmonth_"+i);i++) {
				removeTMPId("weekmonth_"+i);
			}	
			for(var i=0;$2("d_"+i+"_0");i++) {
				for(var i2=0;$2("d_"+i+"_"+i2);i2++) {
					removeTMPId("d_"+i+"_"+i2);
				}
			}	
			this.updateHeader();
			var numWeeks=this.oCalendarTool.getNumWeeks(); 
			var weektmpl = $("weekmonth_tmpl");
			var calendar = $("calmain");
			var sIdPrefix = weektmpl.id.split('_');
	//		var height = (98 / numWeeks) + "%";
			addDOMEvent(calendar, "selectstart", function(evt) { cancelBubbling(evt); });
			addDOMEvent(calendar, "drag" , function(evt) { cancelBubbling(evt); });
			var height = -1;
			if(calendar.offsetHeight != undefined && calendar.offsetHeight>0)
				height = (((calendar.offsetHeight - pxPerEm) / numWeeks));		
				
			function drawpreviewDragandDrop(evt,type,object) {	
				var subObject = object;
				if(object && object.currentObjects && object.currentObjects.length>0) {
						subObject = object.currentObjects[0]; 
				}
				if(!$2("dnd_div_")) {
					var iconNode = Self.getAppointmentDiv(subObject);			
					iconNode.id = "dnd_div_";
					iconNode.style.left = (evt.clientX + iconOffset) + "px";	
					iconNode.style.top = (evt.clientY + iconOffset)+ "px";	
					body.appendChild(iconNode);
					iconNode.style.width = "120px";
					iconNode.style.position = "absolute";							
					addTMPId(iconNode);
				} else {
					var iconNode = $2("dnd_div_");
					iconNode.style.left = (evt.clientX + iconOffset) + "px";
					iconNode.style.top = (evt.clientY + iconOffset)+ "px";	
				}
				var DOMTarget = (evt.target)?evt.target:evt.srcElement;
				var sId = DOMTarget.id;
				if(!sId) return false;
				if(sId.split('_')[3] != 'num' && sId.split('_')[3] != 'title' 
						&& sId.split('_')[4] != 'app'
								 && sId.split('_')[3] != 'headcontainer')			
					DOMTarget.className += " dndSelectionCalendarBG";
				DOMTarget.style.cursor = "move";
				stopEvent(evt);
				return false;							
			}
			function removeDDPreviewDivs(evt,type,object) {
				if($2("dnd_div_")) {
					body.removeChild($2("dnd_div_"));
					removeTMPId($2("dnd_div_"));
				}
				var DOMTarget = (evt.currentTarget)?evt.currentTarget:evt.srcElement;
				var DOMTarget2 = (evt.target)?evt.target:evt.srcElement;
				DOMTarget2.style.cursor = "default";
				var sId = DOMTarget.id;
				DOMTarget.className = removeClass(DOMTarget.className,"dndSelectionCalendarBG");
			}					
	
			function dropCallBack(evt,dragType,object,mousepos,targetNode) {
				var subObject = object;
				if(object && object.currentObjects && object.currentObjects.length>0) {
						subObject = object.currentObjects[0]; 
				}
				var DOMTarget = targetNode;//(evt.currentTarget)?evt.currentTarget:evt.srcElement;			
				var DOMTarget2 = (evt.target)?evt.target:evt.srcElement;
				DOMTarget2.style.cursor = "default";			
				var sId = DOMTarget.id;	
				if(!sId || !sAssocMonthArray[sId+'_num'])
					return;					
				var oDate = new Date();
				
				//var oDate=new Date(Date.UTC(sAssocMonthArray[sId+'_num'].year,sAssocMonthArray[sId+'_num'].month,sAssocMonthArray[sId+'_num'].day.nDate,0,0,0));
				oDate.setUTCHours(0);		
				oDate.setUTCMinutes(0);					
				oDate.setUTCSeconds(0);								
				oDate.setUTCDate(1);		
				oDate.setUTCFullYear(sAssocMonthArray[sId+'_num'].year);				
				oDate.setUTCMonth(sAssocMonthArray[sId+'_num'].month);				
				if(sAssocMonthArray[sId+'_num'].day.nCurrMonth == 0)
					oDate.setUTCMonth(oDate.getUTCMonth()-1);
				else if(sAssocMonthArray[sId+'_num'].day.nCurrMonth == 2)			
					oDate.setUTCMonth(oDate.getUTCMonth()+1);			
				oDate.setUTCDate(sAssocMonthArray[sId+'_num'].day.nDate);					
				var start_date = new Date(subObject.start_date);
				var end_date = new Date(subObject.end_date);	
				oDate.setUTCHours(start_date.getUTCHours());
				oDate.setUTCMinutes(start_date.getUTCMinutes());
				var changes=oDate.getTime()-start_date.getTime();
				var oDateEnd=new Date(end_date.getTime()+changes);	
				moveAppToNewStart(subObject,oDate,oDateEnd);
				DOMTarget.className = removeClass(DOMTarget.className,"dndSelectionCalendarBG");			
				if($2("dnd_div_") != undefined)
					body.removeChild($2("dnd_div_"));
					removeTMPId($2("dnd_div_"));
			}
			for (ind=0; ind<numWeeks; ind++) {
				//clone
				var nextWeekNumber = ind;
				obj = weektmpl.cloneNode(true);
				obj.style.display = "block";
				if (height > 0) obj.style.height = height + "px";
				obj.id = sIdPrefix[0]+'_'+nextWeekNumber;
				addTMPId(obj);	
				//change Ids here		
				for (numNode=0; numNode<obj.childNodes.length; numNode++) {					
					if (obj.childNodes[numNode].id == "kwnumber_0") {
						newId = 'kwnumber_'+ nextWeekNumber;
						obj.childNodes[numNode].id = newId;
						addTMPId(obj.childNodes[numNode]);	
					} else if (obj.childNodes[numNode].id && obj.childNodes[numNode].id.substr(0,1) == "d") {
						var id = 'd_'+nextWeekNumber+'_'+obj.childNodes[numNode].id.substr(4,1);					
						//set new id d_(currentWeek)_(currentDay)
						obj.childNodes[numNode].id = id;
						addTMPId(obj.childNodes[numNode]);
																	
						addDOMEvent(obj.childNodes[numNode], "click", selectDayInMonthView);
						addDOMEvent(obj.childNodes[numNode], "dblclick", monthAction);
						
						registerTarget(obj.childNodes[numNode],{calendar:dropCallBack},null,null,
							drawpreviewDragandDrop,	//drawpreviewDragandDrop
							drawpreviewDragandDrop,	//drawreviewDragandDrop
							removeDDPreviewDivs );//removeDDPreviewDivs

					} else {						
						for (numSubNode=0; numSubNode<obj.childNodes[numNode].childNodes.length; numSubNode++)	{
							if (obj.childNodes[numNode].childNodes[numSubNode].id && 
							    obj.childNodes[numNode].childNodes[numSubNode].id.substr(0,1)=="d") {
								//set new id d_(currentWeek)_(currentDay)
								addDOMEvent(obj.childNodes[numNode].childNodes[numSubNode], "click", selectDayInMonthView);
								addDOMEvent(obj.childNodes[numNode].childNodes[numSubNode],'dblclick',monthAction);
								obj.childNodes[numNode].childNodes[numSubNode].id 
										= 'd_'+nextWeekNumber+'_'+obj.childNodes[numNode].childNodes[numSubNode].id.substr(4,1);
								addTMPId(obj.childNodes[numNode].childNodes[numSubNode]);																	
								registerTarget(obj.childNodes[numNode].childNodes[numSubNode],
									{ calendar:dropCallBack },null,null,
										drawpreviewDragandDrop,	//drawpreviewDragandDrop
										drawpreviewDragandDrop,	//drawreviewDragandDrop
										removeDDPreviewDivs 
								);
							}
							else if(obj.childNodes[numNode].childNodes[numSubNode].id 
										&& obj.childNodes[numNode].childNodes[numSubNode].id.substr(0,2)=="wd")							
							{
								for(weindex=0;weindex<obj.childNodes[numNode].childNodes[numSubNode].childNodes.length;weindex++)
								{
									if(obj.childNodes[numNode].childNodes[numSubNode].childNodes[weindex].id 
										&& obj.childNodes[numNode].childNodes[numSubNode].childNodes[weindex].id.substr(0,1)=="d")				
										{			
												//set new id d_(currentWeek)_(currentDay)
												addDOMEvent(obj.childNodes[numNode].childNodes[numSubNode],'dblclick',monthAction);
												obj.childNodes[numNode].childNodes[numSubNode].childNodes[weindex].id 
																	= 'd_'+nextWeekNumber+'_'+obj.childNodes[numNode].childNodes[numSubNode].childNodes[weindex].id.substr(4,1);
												addTMPId(obj.childNodes[numNode].childNodes[numSubNode]);																	
												 																
										}
								}
							}
							else if(obj.childNodes[numNode].childNodes[numSubNode].id 
										&& obj.childNodes[numNode].childNodes[numSubNode].id=="kwnumber_0")								
							{
								newId = 'kwnumber_'+ nextWeekNumber;
								obj.childNodes[numNode].id = newId;
								addTMPId(obj.childNodes[numNode]);																	
							}
						}
					}				
				}			
				calendar.appendChild(obj);			
			}				
			this.initMonthView();		
		}	
	},
	
	attachAppointments:function (index,	calendarObjects) {
		/////////////////////////////////////////////////////////////////////////////////////////////////
		//	aAppointments = this.getAppointments(currentDay.nDate,oCalendarTool.nMonth,
		//															currentDay.nCurrMonth,aAllAppointments);
	
		var id = calendarObjects.id;
		var folder_id = calendarObjects.folder;
		var selectedAppointmentWeek={id : id, folder : folder_id};
		var divAppId = 'appInWeekView' + folder_id + "_" + id;

		var title = calendarObjects.title;
		var start_date = calendarObjects.start_date;
		var end_date = calendarObjects.end_date;	
		var full_time = calendarObjects.full_time;
		
		// Setze Startzeit des Termins um , falls er vor der Anzeige beginnt
		if (start_date < this.calendar_start.getTime()) {
			start_date=this.calendar_start.getTime();
		}
		
		// Setze Endzeit des Termins um , falls er nach der Anzeige endet
		if (end_date > this.calendar_end.getTime()) {
			end_date = this.calendar_end.getTime();		
		}
	
		var startDate = new Date(start_date);	
		var endDate = new Date(end_date);	
		
		var MULTstartDate = new Date(start_date);	
		MULTstartDate.setUTCHours(0);
		
		var MULTendDate = new Date(end_date);		
		MULTendDate.setUTCHours(0)
	
		//Create multiday appointments here
		var nDaysMultiDay = Math.round((((MULTendDate.getTime() - MULTstartDate.getTime()))  / 1000 / 60 / 60 / 24));
		if (nDaysMultiDay > 0) {
			if(endDate.getUTCHours() == 0 &&
					endDate.getUTCMinutes() == 0 &&
								endDate.getUTCMilliseconds() == 0) {
				nDaysMultiDay--;
			}	
			var oCurrDayIter = new Date(startDate);				
			for (var nNumDay=0; nNumDay<=nDaysMultiDay; nNumDay++) {
				//do nothing at start day - that's default case
				if (nNumDay != 0) {		
					this.doDrawAppointment(oCurrDayIter, calendarObjects, index, nNumDay<nDaysMultiDay);				
				}
				oCurrDayIter.setUTCDate(oCurrDayIter.getUTCDate()+1);
                oCurrDayIter.setTime(Math.floor(oCurrDayIter.getTime() / 864e5) * 864e5); // ms / day
			}
		}
		
		this.doDrawAppointment(startDate, calendarObjects, index, false);
	},
	
	redrawAppointments: function(workWeekHeight,weekEndHeight) {
		var constHeader = pxPerEm*1.1;
		var constBottom = 7;
		var constBottomBorder = 1;
		var constAppointmentPlusBorderPlusMargin = pxPerEm*1.1 + 4 + 1;	
	    var calc_ww = (workWeekHeight - constHeader - constBottom - constBottomBorder) / constAppointmentPlusBorderPlusMargin
	    var calc_we = (weekEndHeight - constHeader - constBottom - constBottomBorder) / constAppointmentPlusBorderPlusMargin
		this.show_ww = IE6 ? Math.floor(calc_ww) : Math.ceil(calc_ww);
		this.show_we = IE6 ? Math.floor(calc_we) : Math.ceil(calc_we);
		this.redraw();
	},
	
	doDrawAppointment: function(startDate, calendarObjects, index, isMultiday) {
		var isMultiday = isMultiday;
		var id = calendarObjects.id;
		var folder_id = calendarObjects.folder;
		var created_by = calendarObjects.created_by;
		var selectedAppointmentWeek={id : id, folder : folder_id};
		var divAppId = 'appInWeekView' + folder_id + "_" + id;
		var shown_as = calendarObjects.shown_as;
		var title = calendarObjects.title;
		var start_date = calendarObjects.start_date;
        if (startDate.getTime() > start_date) start_date = startDate.getTime();
		var end_date = calendarObjects.end_date;	
		var full_time = calendarObjects.full_time;	
		var nRecPosition = calendarObjects.recurrence_position;	
		var oViewObj = this.dateIdAssoc[startDate.getUTCDate() + '_' + startDate.getUTCMonth() + '_' + startDate.getUTCFullYear()];
	
		if (!oViewObj) return;
		var sCurrId = oViewObj.id;
		
		if (oViewObj.app < this.show_ww && !oViewObj.day.bWeekend || 
		    oViewObj.app < this.show_we && oViewObj.day.bWeekend) {
			
	        var sAppId = sCurrId + "_" + oViewObj.app + (full_time ? "f_app" : "_app");
			var dAppDiv = this.getAppointmentDiv(				
			     new CalendarWorkWeekObject(
			             calendarObjects.id, calendarObjects.folder, calendarObjects.recurrence_position, 
			             calendarObjects.recurrence_id, calendarObjects.created_by, calendarObjects.title,
			             start_date, calendarObjects.end_date, calendarObjects.note,
			             calendarObjects.recurrence_type, calendarObjects.participants, 
			             calendarObjects.full_time, calendarObjects.shown_as, calendarObjects.color_label, 
			             calendarObjects.location, null, null),
	             sAppId, full_time, isMultiday);
			
			$2(sCurrId).appendChild(dAppDiv);
			addTMPId(dAppDiv);
			calendarObjects["index"] = index;
			this.aAppointmentAssoc[sAppId] = calendarObjects;
		}
		oViewObj.app++;
					
		if (oViewObj.app == (this.show_ww + 1) && !oViewObj.day.bWeekend || 
		    oViewObj.app == (this.show_we + 1) && oViewObj.day.bWeekend ) {
			
			var oButtonDiv = getContentAsDiv('','expandButton','but_'+sCurrId);
			addDOMEvent(oButtonDiv,'click',monthAction);
			$2(sCurrId).appendChild(oButtonDiv);
			addTMPId(oButtonDiv)		
			oViewObj.app++;					
		}	
						
		if (oViewObj.day.bWeekend && oViewObj.app > 0 && $2(sCurrId+'_0_app')) {
			$2(sCurrId+'_0_app').style.height = "1.0em";
			$2(sCurrId+'_0_app').style.lineHeight = "1.0em";
		}					
	
		var aSelected = calendarSelection.getSelected();
		for (var nindx=0; nindx < aSelected.length; nindx++) {
			if(id == aSelected[nindx].id && folder_id == aSelected[nindx].folder && $2(sAppId)) {			
				$2(sAppId).className = removeClass($2(sAppId).className,
				'border-color-image');			
				$2(sAppId).className += ' selectedAppMonth border-color-selection-elements';
				aSelectedAppArray.push(sAppId);
				break;
			}
		}
	},
	
	getDateFromDay: function(nCurrDay) {
		oCompMonth = new Date();
		oCompMonth.setTime(0);					
		oCompMonth.setUTCDate(1);				
		oCompMonth.setUTCFullYear(this.oCalendarTool.nYear);	
		oCompMonth.setUTCMonth(this.oCalendarTool.nMonth);			
		
		
		if(nCurrDay.nCurrMonth==0)
			oCompMonth.setUTCMonth(oCompMonth.getUTCMonth()-1);
		else if(nCurrDay.nCurrMonth==2)
		{
			var getMonthReturns = oCompMonth.getUTCMonth();
			if(getMonthReturns!=0)
			{
				oCompMonth.setUTCMonth(oCompMonth.getUTCMonth()+1);
			}
			else
				oCompMonth.setUTCMonth(1);		
		}
		oCompMonth.setUTCDate(nCurrDay.nDate);	
		currMonth = oCompMonth.getUTCMonth();	
		currYear = oCompMonth.getUTCFullYear();
		return oCompMonth;	
	},
	
	//Initialize month view with calendar data, add Appointments
	initMonthView: function() {	
		    var Self = this;	
			var aAppointments = new Array();
			this.oCalendar = new cCalendar(this.oCalendarTool.nMonth,this.oCalendarTool.nYear);
			this.aMonthBtAppointments = new Array();
			var aAllAppointments = 	calendarObjects;	
			for (var indexWeek=0; indexWeek<this.oCalendar.aWeeks.length; indexWeek++) {
				var sTableId = 'table_'+indexWeek;	//Nicht mehr benutzt ?				
				var currentWeek = this.oCalendar.aWeeks[indexWeek];
				for(indexDay=0;indexDay<7;indexDay++)
				{
					var nDisplayedAppToday = 0;
					var nAppCount=0;
					var bDisplayFirst = false;
					var sCurrId = 'd_'+indexWeek+'_'+indexDay;
					var currentDay = currentWeek.aDays[indexDay];				
					
					sAssocMonthArray[sCurrId+'_num'] = {day:currentDay,month:this.oCalendarTool.nMonth,
																			year:this.oCalendarTool.nYear};
				
					var currDateObj = this.getDateFromDay(currentDay);
					this.dateIdAssoc[currDateObj.getUTCDate() + '_' + currDateObj.getUTCMonth() + '_' + currDateObj.getUTCFullYear()] 
							= {id:sCurrId, app:0,fdapp:0,day:currentDay};
					//disable			
					if(currentDay.nCurrMonth==0 || currentDay.nCurrMonth==2)
					{
						$2(sCurrId).className += " wholeDayBackground";
					}
					var ShowFullTimeDiv =  newnode('div',0,{id:sCurrId+'_title',
																className:"fullTimeAppointmentMonthTitle"});														
					
					
					if(isToday(currDateObj))
						var bgCurrentDay = " background-color-PMG-selection-elements";
					else 
						var bgCurrentDay = "";
					if(currentDay.bWeekend)
						dDiv =  getContentAsDiv(currentDay.nDate,"dateCalendarFontSize dateAlignMonthView"+bgCurrentDay,sCurrId+'_num');
					else					
						dDiv =  getContentAsDiv(currentDay.nDate,"dateCalendarFontSize dateAlignMonthView"+bgCurrentDay,sCurrId+'_num');
					if(!currentDay.bWeekend)
						var oHeadDiv = newnode('div',0,{className:"headerContStyle",id:sCurrId+"_headcontainer"});
					else					
						var oHeadDiv = newnode('div',0,{className:"headerContStyle",id:sCurrId+"_headcontainer"});					
		
					oHeadDiv.appendChild(ShowFullTimeDiv);
					addTMPId(ShowFullTimeDiv);
					oHeadDiv.appendChild(dDiv);				
					addTMPId(dDiv);
					addDOMEvent(dDiv,"selectstart",function (e) {
						stopEvent(e);
						return false;
					});					
					addDOMEvent(dDiv,"click",monthAction);						
					addDOMEvent(dDiv,"dblclick",monthAction);														
					if(this.today.getUTCFullYear() == currDateObj.getUTCFullYear() &&
									this.today.getUTCMonth() == currDateObj.getUTCMonth() &&
												this.today.getUTCDate() == currDateObj.getUTCDate())
					{
						$2(sCurrId).className += " background-color-content";
					}
					$2(sCurrId).appendChild(oHeadDiv);
					addTMPId(oHeadDiv);
					
					if(indexWeek == this.oCalendar.aWeeks.length-1 &&
																indexDay != 6 && !currentDay.bWeekend)
					{
						try{
							$2(sCurrId).className += " grayBorderCalendarMonth border-color-image";
						}catch(e){}
					}
	
					if(indexWeek == 0 && indexDay == 0)
					{
						//set start date of month				
						this.calendar_start = new Date(currDateObj);
					}				
					if(indexWeek == (this.oCalendar.aWeeks.length-1) && indexDay == 6)
					{	
						//set date date of month
						this.calendar_end = new Date(currDateObj);
						this.calendar_end.setDate(this.calendar_end.getDate()+1)
						try{
							$2(sCurrId).parentNode.style.borderBottom = "1px solid";
							$2(sCurrId).parentNode.className += " border-color-image";
						}catch(e){}					
					}								
					/////////////////////////////////////////////////////////////////////////////////////////////////7
				}				
				myTablePH = getContentAsDiv("","placeHolderKWCenter",'kw_place_'+indexWeek);
				myTable = getContentAsDiv(currentWeek.nKW,"",'kw_month_'+indexWeek);
				$2('kwnumber_'+indexWeek).appendChild(myTablePH);							
				addTMPId(myTablePH);	
				$2('kwnumber_'+indexWeek).appendChild(myTable);	
				
				/* add click and mousover on kw numbers to allow direct jump into week view */
				addDOMEvent($2('kwnumber_'+indexWeek), "click", (function(oCalendar, indexWeek) {
					return function(e) {		
						if (oCalendar == undefined || indexWeek == undefined) {
							return false;
						}
						var mDate = Self.getDateFromDay(oCalendar.aWeeks[indexWeek].aDays[0]);
						activeDay = mDate.getUTCDate();
                        activeMonth = mDate.getUTCMonth();
                        activeYear = mDate.getUTCFullYear();               
						objWeekCalendarTool.init(activeYear, activeMonth, activeDay);
						objWeekCalendarTool.setKW();
                        if (oMiniCalendar) { 
                            oMiniCalendar.setSelectedByDate(activeYear, activeMonth, activeDay);
                        }
                        $2('kwnumber_'+indexWeek).className = removeClass($2('kwnumber_'+indexWeek).className, "background-color-PMG-selection-elements");
						triggerEvent('OX_Switch_View','calendar/calendar/week');
					}
                })(this.oCalendar, indexWeek));
                addDOMEvent($2('kwnumber_'+indexWeek), "mouseover", (function(indexWeek) {
                	return function(e) {
                        $2('kwnumber_'+indexWeek).className += " background-color-PMG-selection-elements";
                	}
                })(indexWeek));
                addDOMEvent($2('kwnumber_'+indexWeek), "mouseout", (function(indexWeek) {
                	return function(e) {
                        $2('kwnumber_'+indexWeek).className = removeClass($2('kwnumber_'+indexWeek).className, "background-color-PMG-selection-elements");
                	}
                })(indexWeek));
                
											
				addTMPId(myTable);
			}		
			this.getStorage();
	},
	
	buildAppointments: function(index, object) {
		    var newObject = {
		    	id: object[0], // 1
		    	folder: object[1], // 20
		    	recurrence_position: object[2], // 207
		    	recurrence_id: object[3], // 206
		    	created_by: object[4], // 2
		    	title: object[5], // 200
		    	start_date: object[6], // 201
		    	end_date: object[7], // 202
		    	note: object[8], // 203
		        recurrence_type: object[9], // 209
		        participants: object[10], // 221
		        full_time: object[11], // 401
		        shown_as: object[12], // 402
		        color_label: object[13], // 102
		        location: object[14]    			    	
		    }
		    newObject["indxxx"] = index;
		    object["indxxx"] = index;
			this.aMonthBtAppointments[index] = newObject;
	},
	
	sortAppointments: function() {	
		this.aMonthBtAppointments.sort(
			function(a, b){
				return (a.full_time || a.start_date < b.start_date ? -1 : 1); 
			}
		);
		for(var i=0; i<this.aMonthBtAppointments.length;i++) {
			this.attachAppointments(this.aMonthBtAppointments[i]["indxxx"],this.aMonthBtAppointments[i]);
		}
	},
	
	getStorage: function(fnParamComplete) {
		var Self = this;
		function cb_fkt(storage) {
			monthViewCreated = true;		
			if (storage) {			
				Self.storage = storage;
			}		
			monthViewCreated = true;		
			Self.redraw = function() {
				Self.clearMonthView();					
				Self.storage.newIterate(Self.storage.ids, emptyFunction, function(x,y) {
					Self.buildAppointments(x,y);
				},
				function() {
					Self.sortAppointments();
				});
				monthViewCreated = true;				
				Self.timestamp = storage.timestamp;
				lastUpdateOfCalendarTimestamp  = Self.storage.timestamp;
				calendar_HeaderUpdate(Self.storage.ids.length);
			};
				
			Self.storage.events.register("Changed", Self.redraw);			

			storage.newIterate(storage.ids, emptyFunction, function(x,y) {
				Self.buildAppointments(x,y);	
			}, function () {
				//duplicated entries
				Self.sortAppointments();
			 	storageCache.update();		
			});	
			if (fnParamComplete != undefined) {		
				fnParamComplete();	
			}
		}			
		sFolderId = activefolder;		
		var oChDate = this.getDateFromDay(this.oCalendar.aWeeks[0].aDays[0]);
		var starttime = oChDate.getTime();
		oChDate = this.getDateFromDay(this.oCalendar.aWeeks[this.oCalendar.aWeeks.length-1].aDays[6]);
		oChDate.setUTCDate(oChDate.getUTCDate()+1);
		var endtime = oChDate.getTime();
		if (Self.show_all) {
			var oParameters = { action:"all", start:starttime, end:endtime, columns:"1,20,207,206,2" };
		} else {		
			var oParameters = { action:"all", start:starttime, end:endtime, folder:sFolderId, columns:"1,20,207,206,2" };
		}	
	
		storageCache.get(AjaxRoot + "/calendar", oParameters, 
		                function(storage) {
						 	cb_fkt(storage);
							calendarSelection.setStorage(storage);
							calendarSelection.reset();
							Self.timestamp = storage.timestamp;
							lastUpdateOfCalendarTimestamp  = storage.timestamp; 
						},
						 this.calendarwfields.toString(),
						 true,
						 calendar_serializeid,
				         calendar_makeID);	
	},

	clearMonthView: function() {	
		//loop day divs
		for(var nItem in sAssocMonthArray) {
			var aTMP = nItem.split('_');
			var sDayDivId = '';
			//remove postfix of current id
			for (var nIndx=0; nIndx<aTMP.length-1; nIndx++) {
				sDayDivId += aTMP[nIndx];
				if (nIndx < aTMP.length-2) sDayDivId += '_';
			}
			var currentDayDiv = $2(sDayDivId);
	
			//find and remove appointments and expand buttons
			var aNodesToRemove = new Array();
			for (var nNodeX=0; nNodeX < currentDayDiv.childNodes.length; nNodeX++) {
				var currentChild;
				if (currentChild = currentDayDiv.childNodes[nNodeX]) {
					var sPostfix = '';
					var sPrefix = '';
					if (currentChild.id) {
						var aTMP2 = currentChild.id.split('_');
						sPostfix = aTMP2[aTMP2.length-1];	
						sPrefix = aTMP2[0];			
					}
					if (sPostfix == 'app' || sPrefix == 'but') {					
						aNodesToRemove.push(currentChild);
					}
					if (sPostfix == 'headcontainer') {	
						for(var nDivIndx=0; nDivIndx<currentChild.childNodes.length; nDivIndx++) {
							if (currentChild.childNodes[nDivIndx].id.split('_')[3] == 'title') {
								removeChildNodes(currentChild.childNodes[nDivIndx]);					
								currentChild.childNodes[nDivIndx].className = 'fullTimeAppointmentMonthTitle';
							}
						}
					}
				}
			}
			for(var nNodeTR=0; nNodeTR<aNodesToRemove.length; nNodeTR++) {
				currentDayDiv.removeChild(aNodesToRemove[nNodeTR]);
			}
		}
		aAppointmentAssoc = new Object();
		this.aMonthBtAppointments  = new Array();
		aSelectedAppArray = new Array();
		monthViewCreated = false;	
		for(var i in this.dateIdAssoc) {
			this.dateIdAssoc[i].app = 0;
			this.dateIdAssoc[i].fdapp = 0;
		}
	},
	
	refreshMonthView: function(bAll) {
		this.show_all = bAll;
		if(calendar_getAllFoldersAttribute()){
			$("allAppointmentMonthView").checked = true;
		}
		else
			$("allAppointmentMonthView").checked = false;	
		oMonthViewObj.clearMonthView();
		oMonthViewObj.getStorage();
	},
	
	getAppointments: function(currDate, currMonth, currCurrMonth, aAppointmentsArray) {
		var aAppointmentsToday = new Array();
		var nIndex=0;
		var nLength = parseInt(aAppointmentsArray.length);
		oCompMonth = new Date();
		oCompMonth.setMonth(currMonth);		
		if(currCurrMonth==0)
			oCompMonth.setMonth(oCompMonth.getMonth()-1);
		else if(currCurrMonth==2)
			oCompMonth.setMonth(oCompMonth.getMonth()+1);
		currMonth = oCompMonth.getMonth();	
		for(var nNumAppX=0;nNumAppX < nLength;nNumAppX++)	
		{		
			var aTMP = aAppointmentsArray[nNumAppX].day.split('.');
			var nAppDay = parseInt(aTMP[0]);
			var nAppMonth = parseInt(aTMP[1]);
			if(nAppDay == parseInt(currDate) && nAppMonth == parseInt(currMonth))
			{
				aAppointmentsToday[nIndex] = aAppointmentsArray[nNumAppX];
				nIndex++;
			}
		}
		return aAppointmentsToday;
	},
	
	getAppointmentDiv: function(calendarObjects,sId,fullday,isMultiday) {
		var appointmentDate = new Date(calendarObjects.start_date);
		if (isMultiday) appointmentDate.setUTCMinutes(0);
		var formattedTime = formatDate(appointmentDate, "time");	
		var shownDiv = getShownDiv(calendarObjects.shown_as);
	
		var datDIV = newnode("div", {border: "1px solid"}, 
					{ className: (fullday ? 
						"showFullTimeAppointmentMonth wholeDayBackground border-color-image"	: 
						"appInMonthView background-color-content border-color-image")} );
		var textDiv = newnode("div", { overflow: "hidden" }, 
					{ className: "showAppointmentTextMonth" + (fullday ? "Full" : "") + " dateCalendarFontSize" });
	
		var timeDisplayString = (!fullday && !isMultiday ? formattedTime : "") + " " + (calendarObjects.title || "");		
		
		if (calendarObjects.location) {
			timeDisplayString += " - " + calendarObjects.location;
		}
		
		// setting confirmation styles
		if (calendarObjects.participants.length) {
			for (var i in calendarObjects.participants) {
				if (calendarObjects.participants[i].id == config.identifier) {
					switch(calendarObjects.participants[i].confirmation) {
						case 0:
							datDIV.style.opacity = ".6";
							textDiv.style.fontStyle = "italic";
							timeDisplayString += " (" + _("waiting") + ")";
							break;
						case 2:
							textDiv.style.textDecoration = "line-through";
							break;
						case 3:											
							textDiv.style.fontStyle = "italic";
							timeDisplayString += " (" + _("tentative") + ")";
							break;
					}
					break;
				}
			}
		}

		textDiv.appendChild(document.createTextNode(timeDisplayString));
		
		datDIV.appendChild(shownDiv);		
		datDIV.appendChild(textDiv);	
		datDIV.id = sId;	
	
	 	var oInplEdit = new inplaceEdit(textDiv, calendarObjects.title, 
	          function(sStr, oObj){
			      editAppointment(calendarObjects, {title:sStr});
			  }
	    );
	
		registerSource(datDIV, "calendar", function () { 
			objMoveCopy.begin("calendar", [calendarObjects]);
			return objMoveCopy;
		}, null, null, calendardefaultdisabled, defaultdisabledremove);
		oInplEdit.input_height = "1em";
		oInplEdit.input_fontsize = "8pt";
        addDOMEvent(datDIV, "contextmenu", function(e) {
            stopEvent(e);
            globalContextMenus.calendar.display(e.clientX, e.clientY,
                                                calendarSelection);
        });
	
	    addDOMEvent(datDIV, "click", function(e) {
	        var SelObj = calendarSelection.getID();
	        if (SelObj && SelObj.id == calendarObjects.id &&
	            SelObj.folder == calendarObjects.folder_id &&
	            SelObj.recurrence_position == calendarObjects.recurrence_position) {
	            if (window.event) {
	                e = window.event;
	            }
	            oInplEdit.on();
	            cancelBubbling(e);      
	        } else {
	            getSelectMonthView(e);
	        }
	    });     
	    addDOMEvent(datDIV, "mousedown", function(e) { 
	        getSelectMonthView(e);
	    });
	    addDOMEvent(datDIV, "dblclick", openAppointmentPopup);
	
		datDIV.oxcalobj = "" + calendarObjects.folder_id + "-" + calendarObjects.id 
		                  + "-" + calendarObjects.recurrence_position;
		return datDIV;
	}
}

function getShownDiv(shown_as) {
	var shownDiv = newnode('div');
	switch (shown_as) {
		case 1:
			shownDiv.className = "appointmentRESERVED";
			break;
		case 2:
			shownDiv.className = "appointmentTEMPORARY";
			break;
		case 3:
			shownDiv.className = "appointmentABSENT";
			break;
		case 4:
			shownDiv.className = "appointmentFREE";
			break;
		default:
			shownDiv.className = "appointmentNONE";
			break;
	}
	shownDiv.className += " showAppointmentShownMonth";
	return shownDiv;
}

function getContentAsDiv(sContent,CSSClass,sId) {
	sContent = sContent.toString();
	var datDIV = document.createElement("div");
	datDIV.className = CSSClass;
	if(sContent.length > 0) {		
		datDIV.appendChild(document.createTextNode(sContent));
	}
	datDIV.id=sId;		
	addTMPId(datDIV)
	return datDIV;
}

function getSelectMonthView(evt){
	var DOMTarget = (evt.target) ? evt.target : evt.srcElement;	
	var sDOMId = DOMTarget.parentNode.id;
	if(!sDOMId || sDOMId.split('_')[4] != 'app')
		return;	
	var oObj = oMonthViewObj.aAppointmentAssoc[sDOMId];		
	deselectAppointments(oObj.full_time);	
	$2(sDOMId).className = removeClass($2(sDOMId).className, "border-color-image");
	$2(sDOMId).className += " selectedAppMonth border-color-selection-elements";
	$2(sDOMId).style.borderWidth = "2px";
	setTimeout((function(sDOMId) {
        return function() {
            aSelectedAppArray.push(sDOMId);
        }
    })(sDOMId), 20);	
	if (calendarSelection.count != 0) {
		calendarSelection.reset();
	}
	selectedAppointment = { 
        id:oObj.id, folder:oObj.folder, recurrence_position:oObj.recurrence_position, 
        recurrence_id:oObj.recurrence_id, created_by:oObj.created_by
    };
	calendarSelection.click(oMonthViewObj.aAppointmentAssoc[sDOMId].index);
	triggerEvent("Selected", [ selectedAppointment ]);
}

function deselectAppointments(fullday) {
	for (var nSelApp=0; nSelApp < aSelectedAppArray.length; nSelApp++) {
		if($2(aSelectedAppArray[nSelApp])) {
			$2(aSelectedAppArray[nSelApp]).className = 
				removeClass($2(aSelectedAppArray[nSelApp]).className,'border-color-selection-elements');
			$2(aSelectedAppArray[nSelApp]).className = 
				removeClass($2(aSelectedAppArray[nSelApp]).className,'selectedAppMonth');			
			$2(aSelectedAppArray[nSelApp]).className += " border-color-image";
			$2(aSelectedAppArray[nSelApp]).style.borderWidth = "1px";
		}
	}
	aSelectedAppArray = new Array();
}

fileloaded();
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * ,
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Jan Finsel <jan.finsel@open-xchange.com>
 * 
 */
/* IE FIX */
register("Loaded", function () {
	if((document.all) && (window.offscreenBuffering)) {
		changeClassAttributes(".wwline","height","19px");		
	}
	addDOMEvent($("calendarWorkweek"), "click" ,function(e) {
		if (calendarSelection.count != 0) {
			cwwchangeSelected();
			cwwchangeSelectedFull();
			calendarSelection.reset();
			selectedAppointment = null;
			triggerEvent("Selected",[]);
		}
	});
});

register("OX_Configuration_Changed", function () {
	if (calendarinittemplates) {
        calendarinittemplates=false;
        createTemplatesWW(getWWInterval(), getWWStartTime(), getWWEndTime());
    }
});

var maxappointmentsperday=3;
var cwwdaysinweek = 5;
var startofweek=0;
var cww_oldvalues=new Object();

function setDaysInWeek(days,start) {
	cwwdaysinweek = days;
	startofweek=start;
	calendarinittemplates=false;
	showCalendarWorkweek();
}

var calendar_activedate;
var calendar_wholeday;
register("_IE_resize_complete",cwwwresizeHeader);

/* Enable/Disable Calendar Workweek Functions */
function cwwenable() {
	resizeEvents.register("Resized",cwwwresizeHeader);
}
function cwwdisable() {
	resizeEvents.unregister("Resized",cwwwresizeHeader);
}

var allcalendarobjects=new Object();
var wwhovers=new Object();
function default_calendar_workweek_create_object(module,folder) {
	
	if(module=="calendar")
	{
		if(!calendar_activedate) {
//			var myinterval=configGetKey("gui.calendar.interval");
			var myinterval=15;
			var d=new Date();			
			d.setUTCDate(1);
			d.setUTCMonth(activeMonth);
			d.setUTCFullYear(activeYear);
			d.setUTCDate(activeDay);
			d.setUTCHours(d.getHours());
			var mymin=d.getUTCMinutes();
			if(mymin % myinterval) {
				mymin=mymin - (mymin % myinterval)+ myinterval;
			}
			d.setUTCMinutes(mymin);
			calendar_createNewAppointment(d,folder,false);	
		} else {
			calendar_createNewAppointment(calendar_activedate,folder,calendar_wholeday);
		}
	}
	calendar_activedate=null;
	calendar_wholeday=null;
}
function CalendarWorkWeekObject(id,folderid,recurrence_position,recurrence_id,created_by,title,start_date,end_date,note,recurrence_type,participants,full_time,shown_as,color_label,location,private_flag)
{
	this.index;
	this.selected=false;
	this.id=id;
	this.folder_id=folderid;
	this.title=title;
	this.start_date=new Date(start_date);
	this.end_date=new Date(end_date);
	this.note=note;
	this.recurrence_type=recurrence_type;
	this.recurrence_position=recurrence_position;
	this.recurrence_id=recurrence_id;
	this.created_by=created_by;
	this.location=location;
	this.participants=participants;
	this.full_time=full_time;				
	this.shown_as=shown_as;
	this.color_label=color_label;
	this.private_flag=private_flag;
	this.divs=new Object();
}
CalendarWorkWeekObject.prototype = {
	addDiv : function (id,node) {
		this.divs[id]=node;
	}
}
function MatrixObject(folderid,id,recurrence_id,div_id,startpoint,endpoint) {
	this.folderid=folderid;
	this.id=id;
	this.element_id=div_id;
	this.recurrenceid=recurrence_id;
	this.startpoint = startpoint || 0;
	this.endpoint = endpoint || 0;
}

var globalinterval = 0;
var globalintervalcounts=0;
var calendarwwfields = new Array(200, 201, 202, 203, 209, 221, 401, 402, 102, 400, 101, 220);
var calendarinittemplates=false;
var firstenter=false;
var cwwstorage;
var wwline=20;

var cwwdays;
var cwwdaysfull;
var cwwactualday;
var showallappointments=true;
function cww_onShowAllAppointmentsChanged(mybool) {
	showallappointments=mybool;
	configSetKey("gui.calendar.allfolders",mybool); 
	var oDateLoc = new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0));	
	updateWWView(oDateLoc);
}

function showCalendarWorkweek() {	
	if(!firstenter) {
		var callbacks=new Object();
		callbacks["CALENDAR_BOTTOM"]=function(event,dragType,data,mouseposition,targetNode) { cww_saveAppointment(data,event,"CALENDAR_BOTTOM");  }
		callbacks["calendar"]=function(event,dragType,data,mouseposition,targetNode) { cww_saveAppointment(data.currentObjects[0],event,"calendar",false);  }
		callbacks["CALENDAR_TOP"]=function(event,dragType,data,mouseposition,targetNode) { cww_saveAppointment(data,event,"CALENDAR_TOP");  }
		callbacks["CALENDAR_NEW"]=function(event,dragType,data,mouseposition,targetNode) { cww_createAppointment(data,event,"CALENDAR_NEW",mouseposition,createPreviewNewAppointmentWW) } 
		registerTarget($("cwwtimecontentall"),callbacks,null,null,
		function(event,type,object,mouseposition) { 
			if(type=="CALENDAR_NEW") {
				drawpreviewnewAppointment(event,type,object,mouseposition)
			} else if (type == "calendar") {
				if(object && object.currentObjects && object.currentObjects.length>0) {
					drawpreviewDragandDrop(event,type,object.currentObjects[0]); 
				}
			}
			else {
				drawpreviewDragandDrop(event,type,object); 
			}
		},
		function(event,type,object,mouseposition) { 
			if(type=="CALENDAR_NEW") {
				drawpreviewnewAppointment(event,type,object,mouseposition)
			} else if (type == "calendar") {
				if(object && object.currentObjects && object.currentObjects.length>0) {
					drawpreviewDragandDrop(event,type,object.currentObjects[0]); 
				}
			} else {
				drawpreviewDragandDrop(event,type,object); 
			}
		},
		function(event,type,object) { removeDDPreviewDivs(); } );
				
		function disabled(e) {
			var iconNode = $2("dragdrop_default_node");
			if(iconNode == undefined) {
				iconNode = newnode("div", {left: (e.clientX + iconOffset) + "px",top: (e.clientY + iconOffset) + "px"},
 			    { className : "dragdropdefaultnode background-color-default border-color-design", id: "dragdrop_default_node"},[
 			    	newnode("table",{ width:"100%" , height: "100%" },{ cellPadding:"0" , cellSpacing :"0" , border : "0" },[
 			    		newnode("tbody",null,null,[
 			    			newnode("tr",null,null,[
 			    				newnode("td",null,null,[
 			    					newnode("img",{},{ src : getFullImgSrc("img/dnd_disabled.gif") },[])
 			    				]),
 			    				newnode("td",{ fontSize : "8pt", whiteSpace : "nowrap" },null,[
 			    					newnode("div",{ overflow: "hidden"},null,[addTranslated("No subject")])
 			    				])
 			    			])
 			    		])
 			    	])
 			    ]);
				body.appendChild(iconNode);
				addTMPId(iconNode);
			}
			iconNode.style.left = (e.clientX + iconOffset) + "px";
			iconNode.style.top = (e.clientY + iconOffset)+ "px";
			stopEvent(e);
			return false;
		}
		function disabledremove(e) {
			if($2("dragdrop_default_node")) {
				$2("dragdrop_default_node").parentNode.removeChild($2("dragdrop_default_node"));
				removeTMPId($2("dragdrop_default_node"));	
			}
			
		}
		registerSource($("cwwcontentresize"),"CALENDAR_NEW",function () { return null; },null,null,disabled,disabledremove);
		firstenter=true;
	}
	function checkChanges() {
        if (cwwdaysinweek != cww_oldvalues.cwwdaysinweek) { 
            return true; 
        }
        return false;
    }

    var tmpinterval=getWWInterval();
    var tmpstarttime=getWWStartTime();
    var tmpendtime=getWWEndTime();        
    
    if (!calendarinittemplates || checkChanges()) {
        createTemplatesWW(tmpinterval,tmpstarttime,tmpendtime);
        cww_oldvalues.cwwdaysinweek=cwwdaysinweek;
        calendarinittemplates=true;
    }
    
    var scrollpoint=tmpstarttime/tmpinterval;
    setTimeout(function() {
        $("cwwtimecontentall").scrollTop=(scrollpoint*20); 
    },0);    
	
	showallappointments=calendar_getAllFoldersAttribute();
	$("cwwshowAllAppointments").checked=showallappointments;
	var oDateLoc = new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0));	
	updateWWView(oDateLoc);
}

function getWWInterval() {
	var tmpinterval=15*60000;
	if (configContainsKey("gui.calendar.interval")) {
        tmpinterval=configGetKey("gui.calendar.interval")*60000;
    }
    return tmpinterval;
}

function getWWStartTime() {
    var tmpstarttime=8*60*60000;
    if (configContainsKey("gui.calendar.starttime")) {
        tmpstarttime=configGetKey("gui.calendar.starttime")*60*60000;
    }
    return tmpstarttime;
}

function getWWEndTime() {
    var tmpendtime=17*60*60000;
    if (configContainsKey("gui.calendar.endtime")) {
        tmpendtime=configGetKey("gui.calendar.endtime")*60*60000;
    }
    return tmpendtime;
}

function clearAppointments() {
	if (cwwdaysfull) {
		for (var zaehler1=0; zaehler1 < cwwdaysfull.length; zaehler1++){ 
			removeChildNodes(cwwdaysfull[zaehler1].node); 
		}
	}
	if(cwwdays) {
		for(var zaehler1=0;zaehler1<cwwdays.length;zaehler1++){
			removeChildNodes(cwwdays[zaehler1].node); 
			for(zaehler2=0;zaehler2<cwwdays[zaehler1].matrix.length;zaehler2++) {
				cwwdays[zaehler1].matrix[zaehler2]=new Array();
			}
		}
	}
	for(var i in allcalendarobjects) {
		for(var i2 in allcalendarobjects[i]["divs"]) {
			removeTMPId(i2);
		}
	}
	allcalendarobjects=new Object();
	if(calendarSelection.count != 0) {
		calendarSelection.reset();
	}
	for (var i=0;$2("cwwmoreappointments"+i);i++) {
		if ($2("cwwmoreappointments"+i).style.display!="none") { 
			$2("cwwmoreappointments"+i).style.display="none" 
		}
	}
}
function cwwchangeView(days,view){
	cwwactualday.setUTCDate(cwwactualday.getUTCDate()+days);
	activeMonth = cwwactualday.getUTCMonth();
	activeYear = cwwactualday.getUTCFullYear();		
	activeDay = cwwactualday.getUTCDate();	
	if(view == "list") {
		menuswitchCalendarView(view);
	}
	if(view == "day") {
		menuswitchCalendarTimerange(view);
	}
}
function goToNextWeek(nDirection)
{
	var step=1;
	if(currentpath2.join("/") == "calendar/calendar/workweek"  || currentpath2.join("/") == "calendar/list/workweek")  { step = 7 }
	if(currentpath2.join("/") == "calendar/calendar/custom"  || currentpath2.join("/") == "calendar/list/custom")  { step = configGetKey("gui.calendar.custom.countdays") }
	var oDateLoc = new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0));	
	oDateLoc.setUTCDate(oDateLoc.getUTCDate()+(nDirection*step));	
	activeMonth = oDateLoc.getUTCMonth();
	activeYear = oDateLoc.getUTCFullYear();		
	activeDay = oDateLoc.getUTCDate();	
	if(currentpath2.join("/") == "calendar/calendar/workweek"  || currentpath2.join("/") == "calendar/list/workweek") {
		
	} else {
		startofweek=oDateLoc.getUTCDay(); 
	}
	oMiniCalendar.setSelectedByDate(activeYear, activeMonth, activeDay);
	updateWWView(oDateLoc);
}
function updateWWView(oDateLoc) {		
	clearAppointments();
    cwwactualday = new Date(getDayInSameWeek(oDateLoc, startofweek) * 864e5);      
	updateHeaderWW(oDateLoc);
	getCalendarObjectsWW(cwwactualday);
}
function updateHeaderWW(oDateLoc) {
	var oHeader = $("wwViewHeader");
	if(currentpath2.join("/") == "calendar/calendar/workweek") {
		$("wwViewHeader").parentNode.parentNode.parentNode.parentNode.parentNode.style.width="200px";
		oHeader.style.width="160px";
		oHeader.firstChild.nodeValue = _("CW") + " " + 
				formatDateTime("w yyyy", new Date(getKeyDayOfWeek(new Date(Date.UTC(activeYear,activeMonth,activeDay))) * 864e5));		
	} else if (currentpath2.join("/") == "calendar/calendar/day"){
		$("wwViewHeader").parentNode.parentNode.parentNode.parentNode.parentNode.style.width="200px";
		oHeader.style.width="160px";
		oHeader.firstChild.nodeValue = formatDate(oDateLoc,"dateday");
	} else {
		if(cwwdaysinweek!=1) {
			var myenddate=new Date(oDateLoc.getTime());
			myenddate.setUTCDate(oDateLoc.getUTCDate() + (cwwdaysinweek-1) );
			$("wwViewHeader").parentNode.parentNode.parentNode.parentNode.parentNode.style.width="300px";
			oHeader.style.width="260px";
			oHeader.firstChild.nodeValue = formatDate(oDateLoc,"date") +" - "+ formatDate(myenddate,"date");
		} else {
			$("wwViewHeader").parentNode.parentNode.parentNode.parentNode.parentNode.style.width="200px";
			oHeader.style.width="160px";
			oHeader.firstChild.nodeValue = formatDate(oDateLoc,"dateday");
		}
	}
	var oMonday= new Date(cwwactualday);
	for (indDay=0;indDay<cwwdaysinweek;indDay++) {
		if (currentpath2.join("/") == "calendar/calendar/workweek" || currentpath2.join("/") == "calendar/calendar/custom") {
			var sId = 'singleDay' + (indDay) + 'stWw';			
			var nCurrHeader = $2(sId);
			//if (!nCurrHeader) continue;
			if (isToday(oMonday))
				nCurrHeader.className = "font-color-default background-color-PMG-selection-elements";
			else 
				nCurrHeader.className = "font-color-default";
				
			nCurrHeader.innerHTML = formatDate(oMonday,"dateday"); 
			if( nCurrHeader.style.display!="") { nCurrHeader.style.display=""; }
			oMonday.setDate(oMonday.getDate()+1);

		} else {
			var sId = 'singleDay' + (indDay) + 'stWw';
			var nCurrHeader  = $2(sId);
			//if (!nCurrHeader) continue;
			removeChildNodes(nCurrHeader);
			var document1=document.createTextNode(".");
			nCurrHeader.className += " font-color-selected";
			nCurrHeader.appendChild(document1); 
		}
	}
}
function getCalendarObjectsWW(actualmonday) {
	var start = actualmonday;
	var end = new Date(start.getTime());
	end.setDate(start.getDate()+cwwdaysinweek);
	var starttime=start.getTime();
	var endtime=end.getTime();
	var params=new Object();
	params.action="all";
	params.start=starttime;
	params.end=endtime;
	params.columns="1,20,207,206,2";
	if(!showallappointments) {
		params.folder= activefolder;
	}
	storageCache.get(AjaxRoot + "/calendar", params, initCalendarObjectsWW,
        calendarwwfields.toString(), true,
		function (x) { 
			return x.folder + "." + x.id + "." + x.recurrence_position 
                + "." + x.recurrence_id + "." + x.created_by;
		},
		function (x) { 
		 	var mObj = { id: x[0], folder: x[1], recurrence_position: x[2],
		 	    recurrence_id: x[3], created_by: x[4] }; 
		    return mObj;
		}
	);
	storageCache.update();	
}
function cwwwupdateView()
{
	if(!lastUpdateOfCalendarTimestamp || lastUpdateOfCalendarTimestamp<cwwstorage.timestamp) {
		lastUpdateOfCalendarTimestamp=cwwstorage.timestamp;
	}
	clearAppointments();
	cwwstorage.newIterate(cwwstorage.ids, emptyFunction, showCalendarObject,
		function(){ 
			redrawAppointment();
		 });
	calendar_HeaderUpdate(cwwstorage.ids.length);	
}
var cww_myboolean=true;

function initCalendarObjectsWW(storage) {
	cww_activehovers=new Object();
	if(cww_myboolean) {
		cww_myboolean=false;
	}
	cwwstorage=storage;
	calendarSelection.setStorage(cwwstorage);
	if (!lastUpdateOfCalendarTimestamp || lastUpdateOfCalendarTimestamp<cwwstorage.timestamp) {
		lastUpdateOfCalendarTimestamp=cwwstorage.timestamp;
	}
	cwwstorage.newIterate(cwwstorage.ids, emptyFunction, showCalendarObject,
		function() {
			redrawAppointment();
		}
	);
	storage.events.register("Changed", cwwwupdateView);	
	calendar_HeaderUpdate(cwwstorage.ids.length);
}

function showCalendarObject(index,object) {
	// consolidate users and participants (field 220 and 221)
	for (var i in object[16]) {
		var a1 = object[16][i];
		for (var ia in object[10]) {
			if (object[10][ia].id == a1.id) {
				a1.confirmation = object[10][ia].confirmation;
				if (object[10][ia].confirmmessage) a1.confirmmessage = object[10][ia].confirmmessage;
				break;
			}
		}
	}
	var calendarobject = new CalendarWorkWeekObject(object[0],object[1],object[2],object[3],object[4],object[5],object[6],object[7],object[8],object[9],object[10],object[11],object[12],object[13],object[14],object[15]);
	calendarobject.index=index;
	allcalendarobjects[""+calendarobject.folder_id+"-"+calendarobject.id+"-"+calendarobject.recurrence_position]=calendarobject;		
	cwwdrawAppointment(calendarobject);
}
function cwwchangeSelected(elementid) {
	if(calendarSelection.count != 0) {
		calendarSelection.reset();		
	}
	for(var myzaehler in allcalendarobjects) {
		for(var myzaehler2 in allcalendarobjects[myzaehler].divs) {
			var tds = allcalendarobjects[myzaehler].divs[myzaehler2].getElementsByTagName("td");
			var divs = allcalendarobjects[myzaehler].divs[myzaehler2].getElementsByTagName("div");			
			var wholeDayEl = !(tds[1] && tds[2]);
			if(wholeDayEl)
			{ 
				divs[2].className = removeClass(divs[2].className,"background-color-selection-elements");
				divs[2].className = removeClass(divs[2].className,"font-color-selected");		
				divs[2].className = removeClass(divs[2].className,"font-weight-high");									
			}
			if(tds[2])
			{
				tds[2].className = removeClass(tds[2].className,"appointmentTitleBG");
				tds[2].className = removeClass(tds[2].className,"background-color-selection-elements");
				tds[2].className = removeClass(tds[2].className,"font-color-selected");
				tds[2].className = removeClass(tds[2].className,"font-weight-high");
			}
			if(tds[1])			
			{
				tds[1].className = removeClass(tds[1].className,"border-color-content-default");				
				tds[1].className = removeClass(tds[1].className,"border-color-selection-elements");
			}
			allcalendarobjects[myzaehler].divs[myzaehler2].className = 
				removeClass(allcalendarobjects[myzaehler].divs[myzaehler2].className,"border-color-selection-elements");
			allcalendarobjects[myzaehler].divs[myzaehler2].className = 
				removeClass(allcalendarobjects[myzaehler].divs[myzaehler2].className,"background-color-selection-elements");								
			allcalendarobjects[myzaehler].divs[myzaehler2].className = 			
				removeClass(allcalendarobjects[myzaehler].divs[myzaehler2].className,"selectedAppointmentWW");				
			allcalendarobjects[myzaehler].divs[myzaehler2].className = 
				removeClass(allcalendarobjects[myzaehler].divs[myzaehler2].className,"deselectedAppointmentWW");							
			allcalendarobjects[myzaehler].divs[myzaehler2].className = 
				removeClass(allcalendarobjects[myzaehler].divs[myzaehler2].className,"border-color-image");											
			allcalendarobjects[myzaehler].divs[myzaehler2].className = 
				removeClass(allcalendarobjects[myzaehler].divs[myzaehler2].className,"appointmentBGWW");					
			if(elementid == myzaehler)
			{
				allcalendarobjects[myzaehler].divs[myzaehler2].className += " selectedAppointmentWW background-color-selection-elements border-color-selection-elements";
				if(tds[1])
					tds[1].className += " border-color-selection-elements";	
				if(tds[2])
					tds[2].className += " background-color-selection-elements font-color-selected font-weight-high";
				if(wholeDayEl)
				{
					divs[2].className += " background-color-selection-elements font-color-selected font-weight-high";
					//divs[2].className = removeClass(divs[2].className,"background-color-selection-elements");					
				}
			}
			else
			{
				allcalendarobjects[myzaehler].divs[myzaehler2].className += "  border-color-image deselectedAppointmentWW appointmentBGWW";
				if(tds[1])
					tds[1].className += " border-color-content-default";									
				if(tds[2])				
					tds[2].className += " appointmentTitleBG";
			}
		}
	}
	
	if (elementid) {
		calendarSelection.toggle(allcalendarobjects[elementid].index);
		selectedAppointment = { 
			id: allcalendarobjects[elementid].id, folder: allcalendarobjects[elementid].folder_id, 
			recurrence_position: allcalendarobjects[elementid].recurrence_position, 
			recurrence_id: allcalendarobjects[elementid].recurrence_id, 
			created_by: allcalendarobjects[elementid].created_by 
		};
		triggerEvent("Selected", calendarSelection.getSelected());
	}	
}
function cwwchangeSelectedFull(elementid) {
	if(calendarSelection.count != 0)
		calendarSelection.reset();
	for(var myzaehler in allcalendarobjects) {
		for(var myzaehler2 in allcalendarobjects[myzaehler].divs) {
			/*
			allcalendarobjects[myzaehler].divs[myzaehler2].className = 
				removeClass(allcalendarobjects[myzaehler].divs[myzaehler2].className,"border-color-selection-elements");
			allcalendarobjects[myzaehler].divs[myzaehler2].className = 
				removeClass(allcalendarobjects[myzaehler].divs[myzaehler2].className,"background-color-selection-elements");				
			allcalendarobjects[myzaehler].divs[myzaehler2].className = 
				removeClass(allcalendarobjects[myzaehler].divs[myzaehler2].className,"wholeDayBackground");
			allcalendarobjects[myzaehler].divs[myzaehler2].className = 
				removeClass(allcalendarobjects[myzaehler].divs[myzaehler2].className,"selectedAppointmentWW");				
			allcalendarobjects[myzaehler].divs[myzaehler2].className = 
				removeClass(allcalendarobjects[myzaehler].divs[myzaehler2].className,"deselectedAppointmentWW");											
			allcalendarobjects[myzaehler].divs[myzaehler2].className = 
				removeClass(allcalendarobjects[myzaehler].divs[myzaehler2].className,"appointmentTitleBG");
			*/
			if(elementid == myzaehler)
				allcalendarobjects[myzaehler].divs[myzaehler2].className += " selectedAppointmentWW background-color-selection-elements border-color-selection-elements";
//			else
//				allcalendarobjects[myzaehler].divs[myzaehler2].className += " deselectedAppointmentWW wholeDayBackground";
				
		}
	}
	
	if (elementid) {
		if(calendarSelection.count != 0) {
			calendarSelection.toggle(allcalendarobjects[elementid].index);		
		}
		selectedAppointment = { 
			id: allcalendarobjects[elementid].id, folder: allcalendarobjects[elementid].folder_id,
			recurrence_position: allcalendarobjects[elementid].recurrence_position, 
	        recurrence_id: allcalendarobjects[elementid].recurrence_id, 
	        created_by: allcalendarobjects[elementid].created_by
		};
		triggerEvent("Selected",calendarSelection.getSelected());
	}
}

function cwwdrawAppointment(calendarobject) {
	function cww_doDblClick (event) {
		if(!event) {
			event=window.event;
		}
		triggerEvent('OX_Calendar_Edit');
		stopEvent(event);
	}
	//Check Full Time Object
	//Check Time
	var oneday=1000*60*60*24;
	var showappointment=true;
	var zaehler3=0;

	// Startdate Correction Complete Appointment
	// Setze Startzeit des Termins um , falls er vor meiner Anzeige beginnt
	var sdt=calendarobject["start_date"].getTime();
	if (sdt<cwwactualday.getTime()) {
		sdt=new Date(cwwactualday.getTime()).getTime();
	}
	// Setze Endzeit des Termins um , falls er nach meiner Anzeige beginnt
	var edt=calendarobject["end_date"].getTime();
	if (edt>cwwactualday.getTime()+(cwwdaysinweek*(oneday))) {
		edt=new Date(cwwactualday.getTime()+(cwwdaysinweek*oneday)).getTime();
	}
	var sdttmp=sdt;
	// IST es ein Ganztagstermin
	if (calendarobject.full_time){		
		// Kann Termin angezeigt werden???
		showappointment=true;
		while (sdttmp<edt) {
			if (!document.getElementById(calendarobject.folder_id+"-"+calendarobject.id+"-"+calendarobject.recurrence_position+"-"+zaehler3))	{
				// Hol mir den Tag der Woche
				var sdd=(new Date(sdttmp).getUTCDay()-new Date(cwwactualday.getUTCDay()) +7) % 7;
				
				/*
				// Hat dieser Tag schon einen Ganztagstermin, wenn ja Listenbild einblenden
				if (cwwdaysfull[sdd].node.hasChildNodes()) {
					showappointment=false;
					if ($2("cwwmoreappointments"+sdd).style.display != "") {
						$2("cwwmoreappointments"+sdd).style.display="";
					}
				} else {
				*/
					//Zeichne Ganztagstermin					
					showappointment=true;
					var appointmentcontainer = cwwdaysfull[sdd].node;
					var myappointment=$("cww_appointment_template_full_time").cloneNode(true);
					var	paramObj = { 
						id: calendarobject.id, folder_id: calendarobject.folder_id,
					    recurrence_position: calendarobject.recurrence_position,
					    recurrence_id: calendarobject.recurrence_id, created_by: calendarobject.created_by
					};	
					var divs = myappointment.getElementsByTagName("div");			
					
					var oInplEdit  = new inplaceEdit(divs[2], calendarobject.title, function (sStr, oObj) {
						editAppointment(paramObj,{title:sStr});
					});
					oInplEdit.input_fontcolor = "#FFFFFF";
					oInplEdit.input_fontsize = "8pt";
					oInplEdit.input_height = "1.2em";
						//fixed bug with auto selection													
					calendarSelection.reset();	
					var myfunc1 = function(oInplEdit) {
						return function(e) {
							var SelObj = calendarSelection.getID();
							if(SelObj && SelObj.id == paramObj.id &&
								SelObj.folder == paramObj.folder_id &&
								SelObj.recurrence_position == paramObj.recurrence_position)
								{
									if(window.event) {
										e = window.event;
									}
									oInplEdit.on();
									stopEvent(e);
							}
						} 
					};
					addDOMEvent(myappointment,"mousedown",function (e) {
						cwwchangeSelected(calendarobject.folder_id+"-"+calendarobject.id+"-"+calendarobject.recurrence_position); 
					});
                    var tds = myappointment.getElementsByTagName("td");
                    var title = tds[1].getElementsByTagName("span")[0];
					addDOMEvent(tds[1], "click", myfunc1(oInplEdit));		
					addDOMEvent(myappointment, "click", function(e) {
							var SelObj = calendarSelection.getID();
							if (SelObj && SelObj.id == paramObj.id &&
								SelObj.folder == paramObj.folder_id &&
								SelObj.recurrence_position == paramObj.recurrence_position) {
							} else {
								cwwchangeSelected(calendarobject.folder_id+"-"+calendarobject.id+"-"+calendarobject.recurrence_position); 
							}
							cancelBubbling(e);
						});								
			
					/* Drag & Drop Source */
					registerSource(myappointment,"calendar",function () { 
						var mymove=new MoveCopy(); 
						mymove.begin("calendar",[calendarobject]); 
						return mymove; 
					},null,null,calendardefaultdisabled,defaultdisabledremove);
					myappointment.id=calendarobject.folder_id+"-"+calendarobject.id+"-"+calendarobject.recurrence_position+"-"+zaehler3;
					myappointment.oxcalobj=calendarobject.folder_id+"-"+calendarobject.id+"-"+calendarobject.recurrence_position+"-"+zaehler3;
					addTMPId(myappointment,"calendar/calendar/workweek");
					zaehler3++
					addDOMEvent(myappointment,"dblclick",cww_doDblClick);
					 					
                    addDOMEvent(myappointment, "contextmenu", function(e) {
                        stopEvent(e);
                        globalContextMenus.calendar.display(e.clientX, e.clientY,
                                                            calendarSelection);
                    });
					allcalendarobjects[calendarobject.folder_id+"-"+calendarobject.id+"-"+calendarobject.recurrence_position].addDiv(myappointment.id,myappointment);
					/** Shown As **/
					switch (calendarobject.shown_as) {
						case 1:
							tds[0].className = "appointmentRESERVED";
							break;
						case 2:
							tds[0].className = "appointmentTEMPORARY";
							break;
						case 3:
							tds[0].className = "appointmentABSENT";
							break;
						case 4:
							tds[0].className = "appointmentFREE";
							break;
					}
					
					// title & location
					var appTitle = [];
					if (calendarobject.title) appTitle.push(calendarobject.title);
					if (calendarobject.location) appTitle.push(calendarobject.location);
					title.firstChild.data = appTitle.join(" - ") || "";
					
					// setting confirmation styles
					if (calendarobject.participants.length) {
						for (var i in calendarobject.participants) {
							if (calendarobject.participants[i].id == config.identifier) {
								switch(calendarobject.participants[i].confirmation) {
									case 0:
										myappointment.style.opacity = ".6";
										title.style.fontStyle = "italic";
										calendarobject.title += " (" + _("waiting") + ")";
										break;
									case 2:
										title.style.textDecoration = "line-through";
										break;
									case 3:											
										title.style.fontStyle = "italic";
										calendarobject.title += " (" + _("tentative") + ")";
										break;
								}
								break;
							}
						}
					}
					
					appointmentcontainer.appendChild(myappointment);
				}
			//}
			//Zaehle Temp Startzaehler um 24 Stunden hoch
			sdttmp = sdttmp+oneday;
		}
	} else {
		if (sdttmp==edt && edt != new Date(cwwactualday.getTime()+(cwwdaysinweek*oneday)).getTime() ) {
			edt++;
		}
		//Hier alle nicht Ganztagstermine	
		while (sdttmp<edt) {
			if (!document.getElementById(calendarobject.folder_id+"-"+calendarobject.id+"-"+calendarobject.recurrence_position+"-"+zaehler3)) {
				showappointment=true;
				var sdd        = (new Date(sdttmp).getUTCDay()-new Date(cwwactualday.getUTCDay()) +7) % 7;
				var longstart  = (sdttmp % oneday);
                if (longstart < 0) longstart += oneday;
				var startpoint = Math.floor((longstart/globalinterval)+0.0000001);
				var edttmp     = edt;
				var longend    = (edt % oneday);
                if (longend < 0) longend += oneday;
				var endpoint   =   Math.ceil((longend/globalinterval)-0.0000001);
				// An diesem Tag nicht zu Ende
				if(Math.floor(sdttmp / oneday + 1) * oneday <= edt) {
					endpoint   = globalintervalcounts;							
				}
				if(startpoint  ==  endpoint) {
					endpoint++;
				}
				/** Check Place Allocation*/
				
				var tiefe=0;  			//Place
				var allelements=0;		//AllElements
			
				for (var zaehler=startpoint;zaehler<endpoint;zaehler++) {
					var maxelements = cwwdays[sdd].matrix[zaehler].length;
					if(allelements<maxelements) {
						allelements=maxelements;
					} 
					if(cwwdays[sdd].matrix[zaehler][tiefe]) {
						tiefe++;
						zaehler=startpoint-1; //Zaehler is added after loop
						if(tiefe==maxappointmentsperday) {
							showappointment=false;	
							break;
						}
					}
				}
				if (!showappointment){
					$2("cwwmoreappointments"+sdd).style.display = "";
				} else {
					var myappointment=$("cww_appointment_template").cloneNode(true);
					myappointment.id=+calendarobject.folder_id+"-"+calendarobject.id+"-"+calendarobject.recurrence_position+"-"+zaehler3;
					myappointment.oxcalobj=calendarobject.folder_id+"-"+calendarobject.id+"-"+calendarobject.recurrence_position+"-"+zaehler3;
					addTMPId(myappointment,"calendar/calendar/workweek");
					zaehler3++
					addDOMEvent(myappointment,"mousedown",function (e) {
						cwwchangeSelected(calendarobject.folder_id+"-"+calendarobject.id+"-"+calendarobject.recurrence_position); 
					});				
					var	paramObj = { 
						id: calendarobject.id, folder_id: calendarobject.folder_id,
						recurrence_position: calendarobject.recurrence_position, recurrence_id: calendarobject.recurrence_id,
						created_by: calendarobject.created_by
					};	
					var tds = myappointment.getElementsByTagName("td");			
						var oInplEdit  = new inplaceEdit(tds[2].getElementsByTagName("div")[0], calendarobject.title, function (sStr,oObj){
						editAppointment(paramObj,{title:sStr});
						});
	
					oInplEdit.input_fontcolor = "#FFFFFF";
					oInplEdit.input_fontsize = "8pt";
					oInplEdit.input_height = "1.2em";
											//fixed bug with auto selection													
					calendarSelection.reset();
					var myfunc1 = function(oInplEdit) {
						return function(e) {
							var SelObj = calendarSelection.getID();
							if(SelObj && SelObj.id == paramObj.id &&
							SelObj.folder == paramObj.folder_id &&
							SelObj.recurrence_position == paramObj.recurrence_position)
							{
								oInplEdit.on();
								stopEvent(e);
							}
						} 
					};
                    var tds = myappointment.getElementsByTagName("td");
                    var title = tds[2].getElementsByTagName("span")[0];
					addDOMEvent(tds[2], "click", myfunc1(oInplEdit));		
					addDOMEvent(myappointment, "click", function(e) {
							var SelObj = calendarSelection.getID();
							if(SelObj && SelObj.id == paramObj.id &&
								SelObj.folder == paramObj.folder_id &&
								SelObj.recurrence_position == paramObj.recurrence_position) {							
							} else {
								cwwchangeSelected(calendarobject.folder_id+"-"+calendarobject.id+"-"+calendarobject.recurrence_position); 
							}
							cancelBubbling(e);
						});								
	
					//addDOMEvent(myappointment,"click",function (e){cancelBubbling(e);});							
					addDOMEvent(myappointment,"dblclick",cww_doDblClick);
					var domdivs=myappointment.getElementsByTagName("div");
					/* Cursor fuer Drag&Drop */
					addDOMEvent(domdivs[1], "mouseover",
						(function(obj){
							return function(){ 
								obj.style.cursor = "move"; };
						})(domdivs[1]));
					addDOMEvent(domdivs[0], "mouseover",
						(function(obj){
							return function(){ obj.style.cursor = "n-resize"; };
						})(domdivs[0]));
					addDOMEvent(domdivs[4], "mouseover",
						(function(obj){ return function(){ obj.style.cursor = "s-resize"; }; })(domdivs[4]));
					
                    addDOMEvent(myappointment, "contextmenu", function(e) {
                        stopEvent(e);
                        globalContextMenus.calendar.display(e.clientX, e.clientY,
                                                            calendarSelection);
                    });
					function disabled(e) {
						var iconNode=$2("cww_node_dd");
						if(iconNode == undefined) {
							iconNode = newnode("img", {position:"absolute", left:(e.clientX + iconOffset) + "px", top:(e.clientY + iconOffset) + "px"},
			 			    {src: getFullImgSrc("img/dnd_disabled.gif"), alt: ""});
							iconNode.id="cww_node_dd";
							body.appendChild(iconNode);
							addTMPId(iconNode);
						}
						iconNode.style.left = (e.clientX + iconOffset) + "px";
						iconNode.style.top = (e.clientY + iconOffset)+ "px";
						stopEvent(e);
						return false;
					}
					function disabledremove(e) {
						
						if($2("cww_node_dd")) {
							$2("cww_node_dd").parentNode.removeChild($2("cww_node_dd"));
							removeTMPId($2("cww_node_dd"));
						}
					}				
					registerSource(domdivs[0],"CALENDAR_TOP",function () { return calendarobject; }, null,null,disabled,disabledremove);
					registerSource(domdivs[4],"CALENDAR_BOTTOM",function () { return calendarobject; },null,null,disabled,disabledremove);
					registerSource(domdivs[1],"calendar",function () { 
						var mymove=new MoveCopy(); 
						mymove.begin("calendar",[calendarobject]); 
						return mymove; 
					},null,null,calendardefaultdisabled,defaultdisabledremove);
					allcalendarobjects[calendarobject.folder_id+"-"+calendarobject.id+"-"+calendarobject.recurrence_position].addDiv(myappointment.id,myappointment);			
					
						
				
					var appointmentcontainer=cwwdays[sdd].node;
					//FILL APPOINTMENTS
						/** Shown As **/
						switch (calendarobject.shown_as) {
							case 1:
								tds[0].className="appointmentRESERVED";
								break;
							case 2:
								tds[0].className="appointmentTEMPORARY";
								break;
							case 3:
								tds[0].className="appointmentABSENT";
								break;
							case 4:
								tds[0].className="appointmentFREE";
								break;
						}
						
						// Title & location
						var appTitle = [];
						if (calendarobject.title) appTitle.push(calendarobject.title);
						if (calendarobject.location) appTitle.push(calendarobject.location);
						title.firstChild.data = appTitle.join(" - ") || "";
						
						// setting confirmation styles
						if (calendarobject.participants.length) {
							for (var i in calendarobject.participants) {
								if (calendarobject.participants[i].id == config.identifier) {
									switch(calendarobject.participants[i].confirmation) {
										case 0:
											myappointment.style.opacity = ".6";
											title.style.fontStyle = "italic";
											calendarobject.title += " (" + _("waiting") + ")";
											break;
										case 2:
											title.style.textDecoration = "line-through";
											break;
										case 3:											
											title.style.fontStyle = "italic";
											calendarobject.title += " (" + _("tentative") + ")";
											break;
									}
									break;
								}
							}
						}
												
						/** Recurrence Type **/
						if(calendarobject.recurrence_type && calendarobject.recurrence_type!=0)
							tds[3].getElementsByTagName("img")[0].style.display = "inline";
						else 
							tds[3].getElementsByTagName("img")[0].style.display = "none";
						
						/** Private or Group **/
						if(calendarobject.participants && calendarobject.participants.length>1) {
							tds[3].getElementsByTagName("img")[1].style.display = "inline";
							tds[3].getElementsByTagName("img")[1].src=getFullImgSrc("img/calendar/group.gif");							
						}
						else {
							tds[3].getElementsByTagName("img")[1].style.display = "none";
						}
		
						/** Tagging **/
						if(calendarobject.color_label && calendarobject.color_label!=0) {
							tds[3].getElementsByTagName("img")[2].style.display = "inline";
							tds[3].getElementsByTagName("img")[2].src=getFullImgSrc("img/menu/tag_"+calendarobject["color_label"]+".gif");
						}				
						else 
							tds[3].getElementsByTagName("img")[2].style.display = "none";
							
						if(calendarobject.private_flag) {
							tds[3].getElementsByTagName("img")[3].style.display = "inline";
							tds[3].getElementsByTagName("img")[3].src=getFullImgSrc("img/private_flag.gif");
						}
						else {
							tds[3].getElementsByTagName("img")[3].style.display = "none";
						}
						
						//New Element is Latest
					if(allelements<tiefe+1) {
						allelements=tiefe+1;
					}
					myappointment.style.height=((endpoint-startpoint) * wwline) + "px";
					myappointment.getElementsByTagName("div")[1].style.height = (((endpoint-startpoint) * wwline)-4) + "px";
					myappointment.style.top=startpoint*wwline+"px";							
					appointmentcontainer.appendChild(myappointment);
					
					for(var zaehler=startpoint;zaehler<endpoint;zaehler++)
					{
						cwwdays[sdd].matrix[zaehler][tiefe]=new MatrixObject(calendarobject.folder_id,calendarobject.id,calendarobject.recurrence_position,myappointment.id,startpoint,endpoint);
					}
					allredrawelements=new Object();
				}
			}
			if(sdt==sdttmp) {
				sdttmp = Math.floor(sdttmp / oneday) * oneday;
			}
			sdttmp = sdttmp+oneday;
		}
	}
}
var allredrawelements=new Object();
function redrawAppointment(elementlength,position,elementid) {
	var allobjects=new Object();
	for(var tagezaehler=0;tagezaehler<cwwdaysinweek;tagezaehler++) {  //<7
		var currentday=cwwdays[tagezaehler].matrix;
		for(var intervalzaehler=0;intervalzaehler<currentday.length;intervalzaehler++) { // 24,48,96,144,288
			var currentinterval=currentday[intervalzaehler];
			var elementsperline=currentinterval.length;
			var tmpobjects=new Array();
			for(var i=0;i<currentinterval.length;i++) { //<max appointments
				var element=currentinterval[i] 
				if(element) {
					tmpobjects.push(element.element_id);
					if(!allobjects[element.element_id]) { 
						allobjects[element.element_id]=new Object(); 
						allobjects[element.element_id]["name"]=element.element_id; 
						allobjects[element.element_id]["nodes"]=new Object();
						allobjects[element.element_id]["plength"]=currentinterval.length;
						allobjects[element.element_id]["pposition"]=i;
					} else if(allobjects[element.element_id]["plength"]<currentinterval.length) {
						allobjects[element.element_id]["plength"]=currentinterval.length;
						allobjects[element.element_id]["pposition"]=i;
					}
				}				
			}
			//Fill childnodes
			for(var i=0;i<tmpobjects.length;i++) {
				var element=allobjects[tmpobjects[i]];
				for(var i2=0;i2<tmpobjects.length;i2++) {
					if(!element["nodes"][tmpobjects[i2]] && (tmpobjects[i2]!=tmpobjects[i])) {
						element["nodes"][tmpobjects[i2]]=true;
					}
				}
			}
		}
	}
	for(var myelements in allobjects) {
		if(!allobjects.highest) {
			var highest=0;
			var checked=new Object();
			var tocheck=new Array();
			var zaehler=0;
			function checkNext() {
				for(zaehler=0;zaehler<tocheck.length;zaehler++) {
					var curobj=tocheck[zaehler];
					if(!checked[curobj.name]) {
						if(highest<curobj.plength) { highest=curobj.plength }
						checked[curobj.name]=true;
						for(var i in curobj.nodes) {
							if(!checked[i]) {
								tocheck.push(allobjects[i]);
							}
						}					
					}
				}
			}
			var myobject=allobjects[myelements];		
			tocheck.push(myobject)
			checkNext();
			
			for(var i1 in checked) {
				allobjects[i1].highest=highest;
			}
		}
	}
	for(var last in allobjects) {
		var d=$2(allobjects[last].name);
		d.style.width=(Math.round(100/allobjects[last].highest)-2)+"%";
		d.style.left=(Math.round(100/allobjects[last].highest)*allobjects[last].pposition)+"%";
	}
	cwwresizeTopHeader();	
}						

/**** Templates Creation *******************************************************************************************/
/**
* Create Matrix
* Create Template Timeline
* Create Template Background Calendar Main Content
*/
function createTemplatesWW(interval,startworktime,endworktime) {
	this.globalinterval=interval;
	var completetime=1000*60*60*24;
	this.globalintervalcounts=Math.round(completetime/interval);
	
	createNewTimeLine(interval);
	createCalendarContentFullTime();
	createCalendarContentWW(interval,startworktime,endworktime);
	cwwdays=new Array();	
	for(var zaehler=0;zaehler<cwwdaysinweek;zaehler++) {
		cwwdays[zaehler]=new Object();
		cwwdays[zaehler].node=$2("cww-line-"+zaehler);
		cwwdays[zaehler].matrix=new Array(globalintervalcounts);
		for(i=0;i<cwwdays[zaehler].matrix.length;i++)
		{
			cwwdays[zaehler].matrix[i]=new Array();
		}
	}
	cwwdaysfull=new Array();
	for(var zaehler=0; zaehler<cwwdaysinweek; zaehler++) {
		cwwdaysfull[zaehler]=new Object();
		cwwdaysfull[zaehler].node = $2("cwwheaderfull:"+zaehler);
	}
}
function createCalendarContentFullTime() {
	function getfunction(list,i) {
		return function() {
			cwwchangeView(i,list);
		}
	}
	
	for (var i=0; $2("cwwmoreappointments"+i); i++) {
		removeTMPId("singleDay"+(i)+"stWw");
		removeTMPId("cwwmoreappointments"+(i));
		removeTMPId("cwwwholeday_dnd:"+i);
		removeTMPId("cwwheaderfull:"+i);
		removeTMPId("cww_wholeday:"+i);	
	}
	
	removeChildNodes($("cwwheaderdates"));
	var tbody=newnode("tbody", null,null,[]);
	//var firsttr=newnode("tr", null, null,[]);
	var secondtr=newnode("tr", { height: "100%" }, { height: "100%" }, []);
	for(var i=0; i<cwwdaysinweek; i++) {
		var onetd=
			newnode("td",{ width: Math.floor(100/cwwdaysinweek)+"%", borderRight: "1px solid", paddingLeft: "5px", overflow:"hidden" },{className: "border-color-design appointmentViewHeadBG"},[
				newnode("table",{ width: "100%"},{ cellPadding: 0 , cellSpacing : 0 , border : 0 },[
					newnode("tbody",null,null,[
						newnode("tr",null,null,[
							newnode("td",{ width: "100%"},null,[
								newnode("div",{ overflow: "hidden" },null,[
									newnode("div",{ whiteSpace :"nowrap" ,cursor: "pointer", zIndex:"-1" },{id: "singleDay"+(i)+"stWw"},[
										newnode("img",{},{src: getFullImgSrc("img/dummy.gif")},[])
									])
								])
							]),
							newnode("td",null,{ id : "cwwmoreappointments"+(i), title : _("Click here for further appointments") },[
								newnode("div", { paddingRight : "7px", height: "16px" , cursor : "pointer" }, null,[
									newnode("img",null,{ src : getFullImgSrc("img/menu/weiterzuListe.gif") },[])
								])
							])
						])
					])
				])
			]);
		addTMPId(onetd.getElementsByTagName("div")[1]);
		addTMPId(onetd.getElementsByTagName("td")[1]);
		addDOMEvent(onetd.getElementsByTagName("td")[0],"click",getfunction("day",i));
		addDOMEvent(onetd.getElementsByTagName("td")[1],"click",getfunction("list",i));
		//firsttr.appendChild(onetd.);
		$("cwwheaderdates").appendChild(onetd);
		
		var secondtd = newnode("td",{ width: Math.floor(100/cwwdaysinweek) + "%" , borderRight: "1px solid" }, 
				{ className: "border-color-design", vAlign: "top", id: "cwwwholeday_dnd:" + i },[
			newnode("div",{ width:"100%", position:"relative"},{ id: "cwwheaderfull:"+i },[ document.createTextNode(" ") ]) ]);
		function getfunction2(node) {
			return function() {
				cww_createNewWholeAppointment(node);
			}
		}
		
		addTMPId(secondtd);
		addTMPId(secondtd.getElementsByTagName("div")[0]);
		//addTMPId(secondtd.getElementsByTagName("table")[0]);
		addDOMEvent(secondtd, "dblclick", getfunction2(secondtd.getElementsByTagName("div")[0]));	
		//addDOMEvent(secondtd.getElementsByTagName("table")[0],"dblclick",getfunction2(secondtd.getElementsByTagName("table")[0]));	
		secondtr.appendChild(secondtd);
	}
	if (IE < 8) $("cwwheaderscroll").style.display="none";
	else $("cwwheaderscroll").style.display="";
	
	delete calendarhovers["daywwcustom"];
	calendarhovers["daywwcustom"]=new Hover(secondtr,OXAppointmentHover.getContent().node);
	calendarhovers["daywwcustom"].setSize(OXAppointmentHover.contentobject.node);
	calendarhovers["daywwcustom"].getTarget = function (node) {
        try {
            while (node) {
                if (node.oxcalobj) return node.parentNode ? node : null;
                node = node.parentNode;
            }
        } catch (e) { }
    };
	calendarhovers["daywwcustom"].onShow = function (node) {
		OXAppointmentHover.actualHover=this;
		var splitid=node.oxcalobj.split("-");
		OXAppointmentHover.refillContent(splitid[1],splitid[0],splitid[2]);
		
	}
	tbody.appendChild(secondtr);
	var d=$("cwwheader");
	removeChildNodes(d);
	d.appendChild(tbody);
	var i=0;
	while($2("cwwwholeday_dnd:"+i)) {
		var mynode=$2("cwwwholeday_dnd:"+i);
		var mycallback=new Object();
		mycallback["calendar"]=returni(i);
		registerTarget(mynode,mycallback,null,null,
			function(event,type,object,mouseposition) {
				drawpreviewDragandDropWholeDay(event,type,object.currentObjects[0]); 
			},
			function(event,type,object,mouseposition) { 
				drawpreviewDragandDropWholeDay(event,type,object.currentObjects[0]); 
			},
			function(event,type,object) { cww_lastdateddwholeday=null; removeDDPreviewDivs(); } 
		);
		i++;
	}
	function returni(i) {
		return function(event,dragType,data,mouseposition,targetNode) { cww_saveAppointment(data.currentObjects[0],event,"calendar",true,i);  }
	}
}

function cww_createNewAppointment(clickedobject) {
	var day=clickedobject.id.split(":")[1];
	var time=clickedobject.parentNode.id.split("-")[2];

	var mydate=new Date(Date.UTC(cwwactualday.getUTCFullYear(),cwwactualday.getUTCMonth(),cwwactualday.getUTCDate()+parseInt(day),0,0,0));
	mydate.setTime(mydate.getTime()+parseInt(time));
	calendar_activedate=mydate;
	calendar_wholeday=false;
	triggerEvent("OX_Before_Create_Calendar");
}
function cww_createNewWholeAppointment(clickedobject) {
	var day=clickedobject.id.split(":")[1];
	var mydate=new Date(Date.UTC(cwwactualday.getUTCFullYear(),cwwactualday.getUTCMonth(),cwwactualday.getUTCDate()+parseInt(day),0,0,0));
	mydate.setTime(mydate.getTime());
	calendar_activedate=mydate;
	calendar_wholeday=true;
	triggerEvent("OX_Before_Create_Calendar");
}
function createCalendarContentWW(interval,startworktime,endworktime){
	var mytable=$("cwwtimecontent");
	for(var zaehler=0;$2("cww-line-"+zaehler);zaehler++) {
		removeTMPId("cww-line-"+zaehler);
	}
	var completetime =1000*60*60*24; 
	var timehour =1000*60*60;
	var templateofftimenohour=$("cwww_calendar_offtime_nohour");
	templateofftimenohour.style.width=Math.floor(100/cwwdaysinweek)+"%";
	var templateofftimehour=$("cwww_calendar_offtime_hour");
	templateofftimehour.style.width=Math.floor(100/cwwdaysinweek)+"%";
	var templateworktimenohour=$("cwww_calendar_worktime_nohour");
	templateworktimenohour.style.width=Math.floor(100/cwwdaysinweek)+"%";
	var templateworktimehour=$("cwww_calendar_worktime_hour");
	templateworktimehour.style.width=Math.floor(100/cwwdaysinweek)+"%";	
	var comtemplateofftimenohour =document.createElement("tr");
	var comtemplateofftimehour =document.createElement("tr");
	var comtemplateworktimenohour =document.createElement("tr");
	var comtemplateworktimehour =document.createElement("tr");
	for (i=0;i<cwwdaysinweek;i++) {
		var tmp1=templateofftimenohour.cloneNode(true);
		tmp1.id="App-ONH:"+i;
		comtemplateofftimenohour.appendChild(tmp1);
		var tmp2=templateofftimehour.cloneNode(true);
		tmp2.id="App-OH:"+i;
		comtemplateofftimehour.appendChild(tmp2);
		var tmp3=templateworktimenohour.cloneNode(true);
		tmp3.id="App-WNH:"+i;
		comtemplateworktimenohour.appendChild(tmp3);
		var tmp4=templateworktimehour.cloneNode(true);
		tmp4.id="App-WH:"+i;
		comtemplateworktimehour.appendChild(tmp4);
	}
	var mytable2=mytable.cloneNode(false);
	var mytable2tbody=document.createElement("tbody");
    var numOfLines = 0;
	for (currenttime=0;currenttime<completetime;numOfLines++) {
		var temp1;
		if((currenttime % timehour) == 0) {
			if(currenttime >= startworktime && currenttime < endworktime) {
				temp1 = comtemplateworktimehour.cloneNode(true);
				temp1.id="cww-tr-"+currenttime;
			}
			else {
				temp1 = comtemplateofftimehour.cloneNode(true);
				temp1.id="cww-tr-"+currenttime;
			}
		}
		else {
			if(currenttime >= startworktime && currenttime < endworktime) {
				temp1 = comtemplateworktimenohour.cloneNode(true);
				temp1.id="cww-tr-"+currenttime;
			}
			else {
				temp1 = comtemplateofftimenohour.cloneNode(true);
				temp1.id="cww-tr-"+currenttime;
			}
		}
		if(currenttime==0)
		{
			for (var zaehler=0;zaehler<temp1.childNodes.length;zaehler++)
			{
				var shorttmp=temp1.childNodes[zaehler];
				var table1=shorttmp.getElementsByTagName("table");
				//var appointmentinsert=table1[0].getElementsByTagName("td")[0];
				var appointmentinsert = document.createElement("div");
				appointmentinsert.id="cww-line-"+zaehler;
				addTMPId(appointmentinsert);
				appointmentinsert.style.width="100%";
				appointmentinsert.style.height="0px";
				appointmentinsert.style.lineHeight="0px";
				appointmentinsert.style.fontSize="0pt";
				appointmentinsert.style.position="relative";
				shorttmp.insertBefore(appointmentinsert,table1[0]);
			}
		}
		mytable2tbody.appendChild(temp1);
		currenttime=currenttime+interval;
	}
	mytable2.appendChild(mytable2tbody);
	replace$(mytable2);	
	
	mytable.parentNode.replaceChild(mytable2,mytable);
	mytable=$("cwwtimecontent");
	calendarhovers["daywwcustom2"]=new Hover(mytable,OXAppointmentHover.getContent().node);
	calendarhovers["daywwcustom2"].setSize(OXAppointmentHover.contentobject.node);
	calendarhovers["daywwcustom2"].getTarget = function (node) {
		try {
            while (node) {
                if (node.oxcalobj) return node.parentNode ? node : 0;
                node = node.parentNode;
            }
        } catch (e) { /*see default implementation*/ }
	}
	calendarhovers["daywwcustom2"].onShow = function (node) {
		OXAppointmentHover.actualHover=this;
		var splitid=node.oxcalobj.split("-");
		OXAppointmentHover.refillContent(splitid[1],splitid[0],splitid[2]);	
	}
	
	cwwwresizeHeader();
    wwline = (temp1.offsetTop + temp1.offsetHeight) / numOfLines;
}

function cwwwresizeHeader() {
	$("cwwheaderresize").style.width = cww_maxShownFD <= 5 ? $("cwwcontentresize").offsetWidth + "px" : $("cwwtimecontentall").offsetWidth + "px";
	$("cwwheaderresize").style.overflowY = cww_maxShownFD <= 5 ? "hidden" : "scroll";
}

var cww_maxShownFD = 0;
function cwwresizeTopHeader() {
	var tmpMax = 0;
	// iterate through curr whole day apps and check if something changed
	// from previous view to current view
	for (var i=0; i<cwwdaysfull.length; i++) {
		if (cwwdaysfull[i].node.childNodes.length > tmpMax) {
			tmpMax = cwwdaysfull[i].node.childNodes.length;			
		}
	}
	// has something changed? if yes, resize the header
	if (tmpMax != cww_maxShownFD) {
		var size = tmpMax <= 1 ? 62 : ((tmpMax > 5 ? 5 : tmpMax) * 20) + 40;		
		resizeSplit("cwwheader_top", size + "px");
		cww_maxShownFD = tmpMax;
	}	
}

/* Create TimeLine */
function createNewTimeLine(interval) {
	$("cwwcontentresize").className = "interval" + (interval/60000);
	var template5min=$("cww_timeline_minutes5_template");
	var time5min = 1000*60*5;
	var template15min=$("cww_timeline_minutes15_template");
	var time15min = time5min*3;
	var template60min=$("cww_timeline_onehour_template");
	var time60min = time15min*4;
	var templateonehourcomplete;
	var node=$("cww_timeline");
	var node2=node.cloneNode(false);
	removeChildNodes(node2);
	var nodetbody=document.createElement("tbody");
	var templateonehourcomplete = new Array();
	var currenttime=0;
	var hourzaehler=0;
	var completetime=1000*60*60; //Zeit einer Stunde
	var rowsperhour=Math.round(time60min/interval);
	for(;currenttime<completetime;) {
		var tmp;
		var modulo=currenttime % time15min;
		if((currenttime % time15min == 0)) 
		{
			if((currenttime % time60min == 0)) 
			{
				tmp=template60min.cloneNode("true");
				if(rowsperhour>1) 
				{
					var bigtext=tmp.getElementsByTagName("td")[0];
					//bigtext.setAttribute("rowspan",rowsperhour);
					bigtext.rowSpan=rowsperhour;
				}
			}
			else  
			{
				tmp=template15min.cloneNode("true");
				tmp.id="T"+currenttime;
				var smalltext=tmp.getElementsByTagName("td")[0];
				if(interval<time15min) 
				{
					var minutes=currenttime / (1000*60);
					smalltext.firstChild.nodeValue=""+minutes; 
				}
			}
		}
		else 
		{
			tmp=template5min.cloneNode("true");
			tmp.id="T"+currenttime;
		}
		templateonehourcomplete.push(tmp);
		currenttime=currenttime+interval;
	}

	/*** Create TimeLine ***/

	for(i=0;i<24;i++)
	{
		for(var zaehler=0;zaehler<templateonehourcomplete.length;zaehler++)
		{
			var myelement=templateonehourcomplete[zaehler].cloneNode(true);
			if(zaehler==0)
			{
				var d1 = new Date(i * 36e5); // ms/h
				if(interval>=time60min)
				{
					myelement.getElementsByTagName("td")[0].style.fontSize="0pt"
					myelement.getElementsByTagName("td")[1].firstChild.nodeValue =formatDate(d1, "hour");
				}			
				else
				{
					myelement.getElementsByTagName("td")[0].firstChild.nodeValue = formatDate(d1, "prefix");
					myelement.getElementsByTagName("td")[1].firstChild.nodeValue = formatDate(d1, "suffix");
				}
			}
			myelement.id="cwwtimeline_"+((i*time60min)+((zaehler*interval) % time60min));
			if(i==23 && (zaehler+1) == templateonehourcomplete.length)	{
				//TODO: BORDER BOTTOM
			}
			nodetbody.appendChild(myelement);
		}
		
	}
	node2.appendChild(nodetbody);
	replace$(node2);
	node.parentNode.replaceChild(node2,node);
	
}
var scrollit=false;
var scrolltimeoutww;
function getDragandDropPointWholeDay(e) {
	var node = e.currentTarget || e.srcElement || false;
	var myday=node.id.split(":")[1];
	if(myday) {
		myday=parseInt(myday);
		return new Date(Date.UTC(cwwactualday.getUTCFullYear(),cwwactualday.getUTCMonth(),cwwactualday.getUTCDate()+myday,0,0,0));	
	}
}
function getDragandDropPoint(e) {
	var coreelement=$("cwwtimecontent");
	var scrollposition= $("cwwtimecontentall").scrollTop;
	var posx=getAbsolutePositionLeft(coreelement);
	var posy=getAbsolutePositionTop(coreelement);
	var poscontent=e.clientY-getAbsolutePositionTop(coreelement)-scrollposition;
	var posheight= $("cwwtimecontentall").offsetHeight;
	var fieldposx=e.clientX-posx;
	var fieldposy=e.clientY-posy;
	if(fieldposx<0 || fieldposx>coreelement.offsetWidth) {
		return null;
	}
	if(fieldposy<0 || fieldposy>coreelement.offsetHeight) {
		return null;
	}
	if(poscontent<40 && $("cwwtimecontentall").scrollTop>0) {
		if(!scrollit) {
			function scrollworkweekup() {
				var newscroll=scrollposition-80;
				if(newscroll>0) {
					$("cwwtimecontentall").scrollTop=newscroll;
				} else {
					$("cwwtimecontentall").scrollTop=0;
				}
				scrollit=false;
			}
			scrolltimeoutww=setTimeout(scrollworkweekup,100);
		}
		scrollit=true;
	} 
	else if((posheight-poscontent)<40 && $("cwwtimecontentall").scrollTop<coreelement.offsetHeight-posheight) {
		if(!scrollit) {
			function scrollworkweekdown() {
				var newscroll=scrollposition+80;
				if(newscroll<coreelement.offsetHeight-posheight) {
					$("cwwtimecontentall").scrollTop=newscroll;
				} else {
					$("cwwtimecontentall").scrollTop=coreelement.offsetHeight-posheight;
				}
				scrollit=false;
			}
			scrolltimeoutww=setTimeout(scrollworkweekdown,100);
		}
		scrollit=true;
	} 
	var daylength=coreelement.offsetWidth/cwwdaysinweek;
	var day=Math.floor(fieldposx/daylength);
	var intervalcount=Math.floor(fieldposy/wwline)*configGetKey("gui.calendar.interval")*60000;
	var mydate=new Date(intervalcount);
	return new Date(Date.UTC(cwwactualday.getUTCFullYear(),cwwactualday.getUTCMonth(),cwwactualday.getUTCDate()+day,mydate.getUTCHours(),mydate.getUTCMinutes(),0));
	 
}
var cww_lastdatedd;
var cww_lastdatestart;
var cww_lastdateend;
function drawpreviewnewAppointment(event,target,object,mouseposition) {
	var startdate;
	if(!mouseposition.startdate) {
		startdate = getDragandDropPoint(mouseposition);
		mouseposition.startdate=startdate;
	} else {
		startdate=mouseposition.startdate;
	}
	var enddate = getDragandDropPoint(event);
	if((startdate && enddate) && (!cww_lastdatestart || cww_lastdateend || (cww_lastdatestart.getTime() != startdate.getTime() && enddate.getTime() != enddate.getTime()))) {
		cww_lastdatestart = startdate;
		cww_lastdateend = enddate;
		if(startdate.getTime()<=enddate.getTime()) {
			createDDPreviewDivs(startdate,enddate);
		} else {
			createDDPreviewDivs(enddate,startdate);
		}
	}
}
var cww_lastdateddwholeday;
function drawpreviewDragandDropWholeDay(event,target,object) {
	var newdate=getDragandDropPointWholeDay(event);
	if((cww_lastdateddwholeday && !newdate) || (!cww_lastdateddwholeday && newdate) || (cww_lastdateddwholeday && newdate && (cww_lastdateddwholeday.getTime() != newdate.getTime()))) {
		cww_lastdateddwholeday = newdate;
		var starttime=object.start_date;
		var endtime=object.end_date;
		if(newdate) {
			var oneday=1000*60*60*24;
			var changes=newdate.getTime()-starttime.getTime();
			starttime=new Date(starttime.getTime()+changes);
			endtime=new Date(endtime.getTime()+changes);
			if(endtime.getTime % oneday) {
				endtime.setUTCDate(endtime.getUTCDate()+1);
			}
		}
		createDDPreviewDivsWholeDay(starttime,endtime);	
	}
}
function drawpreviewDragandDrop(event,target,object) {
	var newdate = getDragandDropPoint(event);
	if((cww_lastdatedd && !newdate) || (!cww_lastdatedd && newdate) || (cww_lastdatedd && newdate && (cww_lastdatedd.getTime() !=newdate.getTime()))) {
		cww_lastdatedd = newdate;
		var starttime=object.start_date;
		var endtime=object.end_date;
		var oneintervaltime=configGetKey("gui.calendar.interval")*60000;
		switch (target) {
			case "CALENDAR_BOTTOM":
				if(newdate) {
					if(newdate.getTime()<=starttime.getTime()) {
						endtime=new Date(starttime.getTime()+oneintervaltime);
					} else {
						endtime=newdate;
					}								
				}
				break;
			case "CALENDAR_TOP":
				if(newdate) {
					if(newdate.getTime()>=endtime.getTime()) {
						starttime=new Date(endtime.getTime()-oneintervaltime);
					} else {
						starttime=newdate;
					}								
				}
				break;
			case "calendar":
				if(newdate) {
					var changes=newdate.getTime()-starttime.getTime();
					starttime=new Date(starttime.getTime()+changes);
					endtime=new Date(endtime.getTime()+changes);
				}
				
				break;
			default:
				break;
		}
		createDDPreviewDivs(starttime,endtime);
	}
}
function removeDDPreviewDivs() {
	var zaehler3=0;
	while($2("cwwpreview"+zaehler3)) {
		if($2("cwwpreview"+zaehler3).parentNode) {
			$2("cwwpreview"+zaehler3).parentNode.removeChild($2("cwwpreview"+zaehler3));
		}
		removeTMPId($2("cwwpreview"+zaehler3));
		zaehler3++;
	}
	if($2("dnd_div_") != undefined) {
		body.removeChild($2("dnd_div_"));
		removeTMPId($2("dnd_div_"));
		
	}
	
}
function createDDPreviewDivsWholeDay(starttime,endtime) {
	removeDDPreviewDivs();
	var oneday=1000*60*60*24;
	var sdt=starttime.getTime();
	if(sdt<cwwactualday.getTime()) {
		sdt=new Date(cwwactualday.getTime()).getTime();
	}
	var edt=endtime.getTime();
	if(edt>cwwactualday.getTime()+cwwdaysinweek*(oneday)) {
		edt=new Date(cwwactualday.getTime()+(cwwdaysinweek*oneday)).getTime();
	}
	var sdttmp=sdt;
	var zaehler3=0;
	while(sdttmp<edt) {
		var sdd  = (new Date(sdttmp).getUTCDay()-new Date(cwwactualday.getUTCDay()) +7) % 7;
		myappointment=document.createElement("div");
		myappointment.className="dndSelectionCalendarBG dndAbsoluteSelection";
		myappointment.id="cwwpreview"+zaehler3;
		addTMPId(myappointment);
		myappointment.style.height="19px";
		myappointment.style.top="0px";
		myappointment.style.width="98%";
		var appointmentcontainer=$2("cwwheaderfull:"+sdd);
		appointmentcontainer.appendChild(myappointment);
		zaehler++;
		if(sdt==sdttmp) {
			sdttmp = Math.floor(sdttmp / oneday) * oneday;
		}
		sdttmp = sdttmp+oneday;
	}
}
function createDDPreviewDivs(starttime,endtime) {
	removeDDPreviewDivs();
	zaehler3=0;
	var oneday=1000*60*60*24;
	
	var sdt=starttime.getTime();
	if(sdt<cwwactualday.getTime()) {
		sdt=new Date(cwwactualday.getTime()).getTime();
	}
	var edt=endtime.getTime();
	if(edt>cwwactualday.getTime()+cwwdaysinweek*(oneday)) {
		edt=new Date(cwwactualday.getTime()+(cwwdaysinweek*oneday)).getTime();
	}
	var sdttmp=sdt;
	while(sdttmp<edt)
	{
		var sdd        = (new Date(sdttmp).getUTCDay()-new Date(cwwactualday.getUTCDay()) +7) % 7;
		var longstart  = (sdttmp % oneday);
        if (longstart < 0) longstart += oneday;
		var startpoint = Math.floor((longstart/globalinterval)+0.0000001);
		var edttmp     = edt;
		var longend    = (edt % oneday);
        if (longend < 0) longend += oneday;
		var endpoint   =   Math.ceil((longend/globalinterval)-0.0000001);
		// An diesem Tag nicht zu Ende
		if(Math.floor(sdttmp / oneday + 1) * oneday <= edt) {
			endpoint   = globalintervalcounts;							
		}
		if(startpoint  ==  endpoint) {
			endpoint++;
		}
		var myappointment=document.createElement("div");
		myappointment.className="dndSelectionCalendarBG dndAbsoluteSelection";
		myappointment.id="cwwpreview"+zaehler3;
		addTMPId(myappointment);
		myappointment.style.height=((endpoint-startpoint) * wwline) + "px";
		myappointment.style.top=startpoint*wwline+"px";
		myappointment.style.width="98%";
		zaehler3++
		var appointmentcontainer=cwwdays[sdd].node;
		appointmentcontainer.appendChild(myappointment);
		if(sdt==sdttmp) {
			sdttmp = Math.floor(sdttmp / oneday) * oneday;
		}
		sdttmp = sdttmp+oneday;
	}
}
function cww_saveAppointment(object,event,target,wholeday,i) {
	var myappointmentobj;
	if(!wholeday) {
		var newdate=getDragandDropPoint(event);
	} else {
		var newdate=new Date(Date.UTC(cwwactualday.getUTCFullYear(),cwwactualday.getUTCMonth(),cwwactualday.getUTCDate()+parseInt(i),0,0,0));
	}
	var starttime=object.start_date;
	var endtime=object.end_date;
	var oneintervaltime=configGetKey("gui.calendar.interval")*60000;
	switch (target) {
		case "CALENDAR_BOTTOM":
			if(newdate) {
				if(newdate.getTime()<=starttime.getTime()) {
					endtime=new Date(starttime.getTime()+oneintervaltime);
				} else {
					endtime=newdate;
				}								
			}
			break;
		case "CALENDAR_TOP":
			if(newdate) {
				if(newdate.getTime()>=endtime.getTime()) {
					starttime=new Date(endtime.getTime()-oneintervaltime);
				} else {
					starttime=newdate;
				}								
			}
			break;
		case "calendar":
			if(newdate) {
				var changes=newdate.getTime()-starttime.getTime();
				starttime=new Date(starttime.getTime()+changes);
				endtime=new Date(endtime.getTime()+changes);
				if(wholeday) {
					var oneday=1000*60*60*24;
					if(endtime.getTime() % oneday) {
						endtime.setUTCDate(endtime.getUTCDate()+1);
					}
				}
			}
			break;
	}
	if((starttime.getTime() == object.start_date.getTime()) && (endtime.getTime() == object.end_date.getTime())) {
		removeDDPreviewDivs();
		return;
	}
	if(!wholeday) {
		wholeday=false;
	}
	myappointmentobj = { start_date : starttime.getTime(), end_date : endtime.getTime(), full_time: wholeday };
	editAppointment(object,myappointmentobj);
	removeDDPreviewDivs();
}

function cww_createAppointment(object,event,type,mouseposition,cb) {
	if(!object) {
		object=new Object();
	} 
	if(!object.folder_id) {
		object.folder_id=activefolder;
	}
	var startdate;
	if(!mouseposition.startdate) {
		startdate = getDragandDropPoint(mouseposition);
		mouseposition.startdate=startdate;
	} else {
		startdate=mouseposition.startdate;
	}
	var enddate = getDragandDropPoint(event);
	if(startdate && enddate) {
		if(startdate.getTime() <= enddate.getTime()) {
			object.start_date=startdate.getTime();
			object.end_date=enddate.getTime();
		} else {
			object.end_date=startdate.getTime();
			object.start_date=enddate.getTime();
		}
		removeDDPreviewDivs();
		if(configGetKey("gui.calendar.openpopup")) {
			openNewAppointmentWindow(object);
		} else {
			if(object.end_date!=object.start_date) {
				setTimeout(function (){openImplaceEdit(object,cb);},0);
			}
		}
	}
}
function openImplaceEdit(object,cb) {
	removeDDPreviewDivs();
	if(cb) cb(object);
}
function createPreviewNewAppointmentWW(object) {
	function dounregister() {
		removeDDPreviewDivs();
		unregister("OX_Dummy",dounregister);
	}
	zaehler3=0;
	var oneday=1000*60*60*24;
	var sdt=object.start_date;
	if(sdt<cwwactualday.getTime()) {
		sdt=new Date(cwwactualday.getTime()).getTime();
	}
	var edt=object.end_date;
	if(edt>cwwactualday.getTime()+cwwdaysinweek*(oneday)) {
		edt=new Date(cwwactualday.getTime()+(cwwdaysinweek*oneday)).getTime();
	}
	var sdttmp=sdt;
	while(sdttmp<edt)
	{
		var sdd        = (new Date(sdttmp).getUTCDay()-new Date(cwwactualday.getUTCDay()) +7) % 7;
		var longstart  = (sdttmp % oneday);
        if (longstart < 0) longstart += oneday;
		var startpoint = Math.floor((longstart/globalinterval)+0.0000001);
		var edttmp     = edt;
		var longend    = (edt % oneday);
        if (longend < 0) longend += oneday;
		var endpoint   =   Math.ceil((longend/globalinterval)-0.0000001);
		// An diesem Tag nicht zu Ende
		if(Math.floor(sdttmp / oneday + 1) * oneday <= edt) {
			endpoint   = globalintervalcounts;							
		}
		if(startpoint  ==  endpoint) {
			endpoint++;
		}
		var myappointment=$("cww_appointment_template").cloneNode(true);
		myappointment.id="cwwpreview"+zaehler3;
		addTMPId(myappointment);
		var appointmentcontainer=cwwdays[sdd].node;
		var tds = myappointment.getElementsByTagName("td");					
		tds[0].className="appointmentShowAsReserved";
		tds[2].getElementsByTagName("div")[0].firstChild.data = "tre";
		tds[3].getElementsByTagName("img")[0].style.display = "none";
		tds[3].getElementsByTagName("img")[1].style.display = "none";
		tds[3].getElementsByTagName("img")[2].style.display = "none";
		tds[3].getElementsByTagName("img")[3].style.display = "none";
		myappointment.style.height=((endpoint-startpoint) * wwline) + "px";
		myappointment.getElementsByTagName("div")[1].style.height = (((endpoint-startpoint) * wwline)-4) + "px";
		myappointment.style.top=startpoint*wwline+"px";		
		myappointment.style.width="98%";
		appointmentcontainer.appendChild(myappointment);					
		
		if(sdt==sdttmp) {
			sdttmp = Math.floor(sdttmp / oneday) * oneday;
		}
		sdttmp = sdttmp+oneday;
		zaehler3++;
	}
	var implElem=$2("cwwpreview0");
	var oInplEdit  = new inplaceEdit(implElem.getElementsByTagName("div")[2], _("New appointment"), function (sStr,oObj){
			object.title=sStr;
			createAppointment(object);
			removeDDPreviewDivs();
			storageCache.update();
 	});		
 	oInplEdit.really_edit_check = false;
	oInplEdit.input_fontcolor = "#000000";
	oInplEdit.input_fontsize = "8pt";
	oInplEdit.input_height = "1.2em";
	oInplEdit.on();
	register("OX_Dummy",dounregister);
}
function editAppointment(object,changes, fn_opt) {
    changes.notification = true;
	function edit_whole_series(id) {
		var app_id = id || object.recurrence_id || object.id;
		json.put(AjaxRoot + "/calendar?action=update&session=" + session + "&id=" + app_id + "&timestamp=" + lastUpdateOfCalendarTimestamp + "&folder=" + object.folder_id, changes, null, 
		function(cb){
			if(cb && cb.data && cb.data.conflicts){
				function cbignore() {
					changes.ignore_conflicts=true;
					json.put(AjaxRoot + "/calendar?action=update&session=" + session + "&id=" + app_id + "&timestamp=" + lastUpdateOfCalendarTimestamp + "&folder=" + object.folder_id, changes, null,
					function(response) {
						if(response && response.data && response.data.id) {	
							storageCache.update(triggerEvent("OX_Refresh_Mini_Calendar"));	
							
							if(fn_opt)
								fn_opt();							
						} 		
					});
				}
				ConflictsPopup.open(object, cb.data.conflicts, null, null,
                                    cbignore);
				
			} else {
				storageCache.update(triggerEvent("OX_Refresh_Mini_Calendar"));
				if(fn_opt)
					fn_opt();
			}
		});		
	}
	function edit_single_series() {
		changes.recurrence_position = object.recurrence_position;
		edit_whole_series(object.id);
	}
	function cancel_edit() {
		storageCache.update();
	}
	if(menuhasRightandLoad([object],"WRITE",
		function () {
			if(object.recurrence_position>0 && object.recurrence_id && 
			   object.recurrence_id == object.id) {
				function isSameDate(date1,date2) {
					var diff= date1.getTime() - date2.getTime();
					if((diff < 1000*60*60*24) && (diff > -(1000*60*60*24))) {
						if(date1.getUTCDate() == date2.getUTCDate()) {
							return true;
						}
					}
					return false;
				}
				if(!object.start_date | !changes.start_date || isSameDate(object.start_date,new Date(changes.start_date))) {
					newConfirm(_("Edit Appointment"),_("Do you want to edit the whole series or a single series appointment? Please note when changing a serial appointment, exceptions to the series are reset."),AlertPopup.SERIESBUTTON,null,cancel_edit,null,null,edit_whole_series,edit_single_series,cancel_edit);					
				} else {
					newConfirm(_("Edit Appointment"),
                        _("By changing the date of this appointment you are creating an appointment exception to the series. Do you want to continue?"),
                        AlertPopup.CONTINUECANCEL, edit_single_series,
                        cancel_edit, null, null, edit_whole_series, null,
                        cancel_edit, null, edit_single_series);
				}
			} else {
				edit_whole_series(object.id);
			}	
		}
	)); 
} 	
function createAppointment(object) {
	if (!object.alarm && configContainsKey("gui.calendar.default_reminder")) {
		object.alarm = configGetKey("gui.calendar.default_reminder");
	}
	json.put(AjaxRoot + "/calendar?action=new&session=" + session, object, null, 
	function(response){
		function cbignore() {
			object.ignore_conflicts=true;
			json.put(AjaxRoot + "/calendar?action=new&session=" + session, object, null,
			function(responseinternal) {
				if(responseinternal && responseinternal.data && responseinternal.data.id) {	
					storageCache.update();	
				} 		
			});
		}
		if(response && response.data && response.data.id) {	
			storageCache.update();	
		} 
		else if(response && response.data && response.data.conflicts) {
			ConflictsPopup.open(object,response.data.conflicts,null,null,cbignore);
		}
	});
}
function openNewAppointmentWindow(object) {
	function createAppyes() {
		function myok() {
			object.title=$("create_window_text_field").value;
			createAppointment(object);
		}
		function myadvanced() {
			newWindow("newAppointment.html#modul=new&folder_id="+object.folder_id+"&start_date="+object.start_date+"&end_date="+object.end_date+"", getWindowOptions());
		}
		removeDDPreviewDivs();
		newAppointmentObject(object,myok,function() {},myadvanced,null);
	}
	rightshandler.checkRights("calendar",null , object, createAppyes, function() {}); 
}

fileloaded();
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Suphi Basdemir <suphi.basdemir@open-xchange.com>
 * 
 */

var weekStorage;
var allcalendarWeekObjects=new Object();

function openNewAppointmentInWeek(date) {
	calendar_activedate=date;
	calendar_wholeday=false;
	triggerEvent("OX_Before_Create_Calendar");
}

function WeekCalendarTool() {
	this.firstinit=false;
	this.oCalendarTool = new calendarTool(activeMonth,activeYear);
	this.switchedToNext = 0;
	this.oCalendarTool1970 = new calendarTool(1,1970);
	this.calendarwfields = new Array(200, 201, 202, 203, 207, 221, 401, 402, 102, 400, 101, 206, 2, 220);
	//title, startdate, enddate, note, seq(0-4), participants, fulltime(true|false), shownas, tag
	this.calendarWeekTimer = null;
	this.timer = 700;
	this.showallappointments = true;
	this.aAppointmentAssoc = new Object();
	this.divIdCurrentDay = "";
	this.aDateForNewWindow = new Object();
	this.aMonthForNewWindow = new Object();
	this.registeredEvents = new Array();
	this.registeredEventsAdd = new Array();
	this.displayedWeekLinks = new Array();
	this.appendetFirstChilds = new Array();	
}

WeekCalendarTool.prototype = {
	
	init: function(paramYear, paramMonth, paramDay) {
		//fixed text selection bug in IE
		if(!this.firstinit) {
			addDOMEvent($("weekcontainer"), "selectstart", function() { return false; });			
			addDOMEvent($("weekcontainer"), "drag" , function() { return false; });
			addDOMEvent($("weekcontainer"), "click", function (e) {
				if (calendarSelection.count != 0) {
					triggerEvent("Selected", []);
					calendarSelection.reset();
					selectedAppointment = null;
					selectAppointmentWeekView();					
				}
			});
			this.firstinit=true;
			var hover=new Hover($("weekcontainer"),OXAppointmentHover.getContent().node);
			calendarhovers["week"]=hover;
			
			hover.setSize(OXAppointmentHover.contentobject.node);
			hover.getTarget = function (node) {
                try {
                    while (node) {
                        if (node.oxcalobj) return node.parentNode ? node : null;
                        node = node.parentNode;
                    }
                } catch (e) { /*see default implementation*/ }
            };
			hover.onShow = function (node) {
				OXAppointmentHover.actualHover=this;
				var splitid=node.oxcalobj.split("-");
				OXAppointmentHover.refillContent(splitid[1],splitid[0],splitid[2]);	
			}
		}
		
		if (configGetKey("gui.effects.hover.calendar") && calendarhovers["week"]) {
			calendarhovers["week"].enable();
		}
		
		for (var t = 1; t< 8; t++) {
			$("imageList"+t).style.display = "none";
		}
		
		if (paramYear && (paramMonth != undefined) && paramDay) {			
			activeYear = paramYear;
			activeMonth = paramMonth;
			activeDay = paramDay;
			this.oDateLoc = new Date(Date.UTC(paramYear, paramMonth, paramDay, 0, 0, 0));
		} else {
			this.oDateLoc = new Date(Date.UTC(activeYear, activeMonth, activeDay, 0, 0, 0));
		}
		
		this.setKW();
		this.setDays();
		
		if(calendar_getAllFoldersAttribute()) {
			this.showallappointments=true;
			$("allAppointmentWeekView").checked = true;
		} else { 
			this.showallappointments=false;
			$("allAppointmentWeekView").checked = false;
		}
			
		this.getStorageCache();
		this.getData = 0;		
	},
	setKW: function() {
		this.currentKW = this.oCalendarTool.getKW(this.oDateLoc.getUTCFullYear(),
															this.oDateLoc.getUTCMonth(),
																	this.oDateLoc.getUTCDate());
		
		
		this.testToday();
	},
	testToday: function() {
		if(this.divIdCurrentDay != "") {
			/* reset background-color */
			$(this.divIdCurrentDay).className = "weekViewDivContent font-color-default workTimeBackground border-color-design";
		}
		this.oDateAct = new Date();
		this.nKW = this.oCalendarTool.getKW(this.oDateAct.getUTCFullYear(),this.oDateAct.getUTCMonth(),this.oDateAct.getDate());
		if(this.currentKW == this.nKW && activeYear == this.oDateAct.getUTCFullYear()) {
			this.divIdCurrentDay = 'weekDayContent'+this.oDateAct.getDay();
			$(this.divIdCurrentDay).className = "weekViewDivContent background-color-content todaySelectionBorder";
		}
		
	},
	openNewCalendar: function(evt,clicked) {
		var DOMTarget = (evt.target) ? evt.target : evt.srcElement;
		var oDate = new Date();
		activeDay = oDate.getUTCDate();
		activeMonth = oDate.getUTCMonth();
		activeYear = oDate.getUTCFullYear();
		if (clicked == "dblclick") {
			triggerEvent('OX_Before_Create_Calendar');
		} else if (clicked == "click") {
			window.setTimeout(function() {
				menuswitchCalendarTimerange("day")
			}, 20);
		}
	},
	setDays: function() {
		this.oMonday = new Date(getDayInSameWeek(new Date(Date.UTC(activeYear,activeMonth,activeDay,0,0,0)), weekStart) * 864e5);
		var Self = this;
		
		function drawpreviewDragandDrop(evt,type,object) {
			var subObject = object;
			if(object && object.currentObjects && object.currentObjects.length>0) {
					subObject = object.currentObjects[0]; 
			}			
			var DOMTarget = (evt.target)?evt.target:evt.srcElement;	
			DOMTarget.style.cursor = "move";
			if(!$2("dnd_div_")) {				
				var iconNode = Self.getAppointmentDiv(subObject);					
				iconNode.id = "dnd_div_";
				iconNode.style.left = (evt.clientX + iconOffset) + "px";
				iconNode.style.top = (evt.clientY + iconOffset)+ "px";	
				body.appendChild(iconNode);
				addTMPId(iconNode);
				iconNode.style.width = "320px";
				iconNode.style.position = "absolute";				
			} else {
				var iconNode = $2("dnd_div_");
				iconNode.style.left = (evt.clientX + iconOffset) + "px";
				iconNode.style.top = (evt.clientY + iconOffset)+ "px";	
			}
			var sId = DOMTarget.id;
			if(sId.slice(0,14) != "weekDayContent")
				return;
			DOMTarget.className = removeClass(DOMTarget.className,"dndSelectionCalendarBG");			
			DOMTarget.className += " dndSelectionCalendarBG";
		}
		
		function removeDDPreviewDivs(evt,type,object) {
			var DOMTarget = (evt.currentTarget)?evt.currentTarget:evt.srcElement;
			var DOMTarget2 = (evt.target)?evt.target:evt.srcElement;
			DOMTarget2.style.cursor = "default";
			DOMTarget.className = removeClass(DOMTarget.className,"dndSelectionCalendarBG");
			if($2("dnd_div_")) {
				body.removeChild($2("dnd_div_"));
				removeTMPId($2("dnd_div_"));
			}
		}					

		function dropCallBack(evt,dragType,object,mousepos,targetNode,dropNode){
			var subObject = object;
			if (object && object.currentObjects && object.currentObjects.length > 0) {
				subObject = object.currentObjects[0]; 
			}						
			var DOMTarget = targetNode;//(evt.target)?evt.target:evt.srcElement;
			var sId = DOMTarget.id;	

			var nNum = (parseInt(sId.slice(sId.length-1, sId.length)) == 0)
			     ? 7 : parseInt(sId.slice(sId.length-1, sId.length));

			var sKey = "weekviewDay" + nNum;
			if (Self.aDateForNewWindow[sKey] == undefined
			    || Self.aMonthForNewWindow[sKey] == undefined) return;

			DOMTarget.className = removeClass(DOMTarget.className,"dndSelectionCalendarBG");			

			var oDate = new Date();
			oDate.setUTCHours(0);		
			oDate.setUTCMinutes(0);					
			oDate.setUTCSeconds(0);								
			oDate.setUTCDate(1);		
			oDate.setUTCFullYear(activeYear);				
			oDate.setUTCMonth((Self.aMonthForNewWindow[sKey]-1));				
			oDate.setUTCDate(Self.aDateForNewWindow[sKey]);			
			var start_date = new Date(subObject.start_date);
			var end_date = new Date(subObject.end_date);			
			oDate.setUTCHours(start_date.getUTCHours());
			oDate.setUTCMinutes(start_date.getUTCMinutes());
			var changes=oDate.getTime()-start_date.getTime();
			var oDateEnd=new Date(end_date.getTime()+changes);
			
			// drop only allowed if destination date != current date	
			if (formatDate(start_date, "date") != formatDate(oDate, "date"))
			     moveAppToNewStart(subObject,oDate,oDateEnd);
			     
			if($2("dnd_div_")) {
				body.removeChild($2("dnd_div_"));
				removeTMPId($2("dnd_div_"));
			}
			var DOMTarget2 = (evt.target)?evt.target:evt.srcElement;
			DOMTarget2.style.cursor = "default";						
		}
		
		for (var inWeekDay=0; inWeekDay<7; inWeekDay++) {
			(function (inWeekDayIndex) {
				
				if (inWeekDayIndex == 6) {
					var dayContentId = 'weekDayContent0';
				} else {
					var dayContentId = 'weekDayContent'+(inWeekDayIndex +1);
				}
				
				var sId = 'weekviewDay'+(inWeekDayIndex+1);			
				var nCurrHeader  = $(sId);
				var nCurrContent  = $(dayContentId);				
				
				if (Self.registeredEvents["dbl_1_" + inWeekDayIndex]) {
					removeDOMEvent(nCurrContent,'dblclick', Self.registeredEvents["dbl_1_" + inWeekDayIndex]);
				}
				if (Self.registeredEvents["dbl_2_" + inWeekDayIndex]) {
					removeDOMEvent(nCurrHeader,'dblclick', Self.registeredEvents["dbl_2_" + inWeekDayIndex]);
				}				
				
				var nDisplayMonth = Self.oMonday.getUTCMonth()+1;
				Self.aMonthForNewWindow[sId] = nDisplayMonth;
				
				if (nDisplayMonth < 10)	nDisplayMonth = '0' + nDisplayMonth;
				var nDisplayDate = Self.oMonday.getUTCDate();
				Self.aDateForNewWindow[sId] = nDisplayDate;
				
				if(nDisplayDate < 10) nDisplayDate = '0' + nDisplayDate;		

				var date=new Date();
				date.setUTCDate(1);
				date.setUTCFullYear(Self.oMonday.getUTCFullYear());
				date.setUTCMonth(Self.oMonday.getUTCMonth());
				date.setUTCDate(Self.oMonday.getUTCDate());
				date.setUTCHours(configGetKey("gui.calendar.starttime") || 8);
				
				function openNewAppointmentInWeek(e) {
					calendar_activedate=date;
					calendar_activedate.setUTCMinutes(0);					
					calendar_wholeday=false;
					triggerEvent("OX_Before_Create_Calendar");					
				}
				
				function openNewWholeAppointmentInWeek(e) {
					var date2=date;
					date2.setUTCHours(0);
					date2.setUTCMinutes(0);
					calendar_activedate=date2;
					calendar_wholeday=true;
					triggerEvent("OX_Before_Create_Calendar");
				}
				Self.registeredEvents["dbl_1_" + inWeekDayIndex] = openNewAppointmentInWeek;
				Self.registeredEvents["dbl_2_" + inWeekDayIndex] = openNewWholeAppointmentInWeek;
				
				addDOMEvent(nCurrContent, 'dblclick', openNewAppointmentInWeek);	
				addDOMEvent(nCurrHeader, 'dblclick', openNewWholeAppointmentInWeek);
				
				Self.yearUTC=Self.oMonday.getUTCFullYear();	
				nCurrHeader.className = isToday(Self.oMonday) 
				        ? "background-color-PMG-selection-elements" : "";

				nCurrHeader.firstChild.nodeValue = formatDate(Self.oMonday,"dateday");
				Self.oMonday.setUTCDate(Self.oMonday.getUTCDate()+1);
				
				registerTarget(nCurrContent,{calendar:dropCallBack},null,null,
						drawpreviewDragandDrop,	//drawpreviewDragandDrop
						drawpreviewDragandDrop,	//drawreviewDragandDrop
						removeDDPreviewDivs );//removeDDPreviewDivs				
			})(inWeekDay)
		}
		$('weekviewheader').firstChild.nodeValue = _("CW") + " " + 
				formatDateTime("w yyyy", new Date(getKeyDayOfWeek(new Date(Date.UTC(activeYear,activeMonth,activeDay))) * 864e5));
		
	},
	goToNextWeek: function(nDirection) {		
		this.clearContentFields();
		this.getData = 1;
		this.nDirection = nDirection;
		this.oDateLoc.setUTCDate(this.oDateLoc.getUTCDate()+(nDirection*7));	
		activeMonth = this.oDateLoc.getUTCMonth();
		activeYear = this.oDateLoc.getUTCFullYear();		
		activeDay = this.oDateLoc.getUTCDate();	
		this.setKW();
		this.setDays();
		var Self = this;
		
		Self.calendarWeekTimer  = window.setTimeout(
									function() 	{
											Self.goToNextWeek(Self.nDirection); 
											Self.switchedToNext++;
									}, Self.timer);
		if (this.switchedToNext>4) {
			Self.timer = 400;
		} else {
			Self.timer = 700;
		}
		
		if (oMiniCalendar) oMiniCalendar.setSelectedByDate(activeYear, activeMonth, activeDay);
	},
	clearTimer: function() {
		this.switchedToNext = 0;
		window.clearTimeout(this.calendarWeekTimer);
		if(this.getData >0) {
			this.getData = 0;
			this.getStorageCache();
		}		
	}, 
	clearContentFields: function() {
		for (var t = 1; t< 8; t++)
		 		$("imageList"+t).style.display = "none";
		allcalendarWeekObjects = {};
		this.weekAppointments = new Array();
		for(var i=0; i<7;i++) {
			removeChildNodes($('weekDayContent'+i));
		}
	},
	updateView: function() {
		this.clearContentFields();
		this.iterateStorage();
		lastUpdateOfCalendarTimestamp = storageCache.current.timestamp;
		calendar_HeaderUpdate(weekStorage.ids.length);
	},
	showAllAppointments: function(mybool) {
		this.showallappointments=mybool;
		configSetKey("gui.calendar.allfolders",mybool)
		this.getStorageCache();
	},
	getStorageCache: function() {
		if (weekStorage) {
            weekStorage.events.unregister("Changed", function() {
                Self.updateView();
            });
        }
		this.clearContentFields();
		var start = new Date(getDayInSameWeek(this.oDateLoc, weekStart) * 864e5);		
		var end = new Date(start.getTime());
		end.setUTCDate(start.getUTCDate()+7);
		this.starttime=start.getTime();
		var endtime=end.getTime();
		var Self = this;
		Self.params=new Object();
		Self.params.action="all";
		Self.params.start=Self.starttime;
		Self.params.end=endtime;		
		Self.params.columns="1,20,207";
		if(!Self.showallappointments) {
			Self.params.folder= activefolder;
		}
		storageCache.get(AjaxRoot + "/calendar",
		 Self.params ,
		 function(storage) {
		 	Self.weekAppointments = new Array()
		 	for (var t = 0; t< Self.displayedWeekLinks.length; t++)
		 		Self.displayedWeekLinks[t].style.display = "none";
		 		
		 		
		 	storageCache.current = storage;		 	
			calendarSelection.setStorage(storage);
		 	storageCache.update();
		 	Self.iterateStorage(storage);
			lastUpdateOfCalendarTimestamp = storage.timestamp;
		 },
		 Self.calendarwfields.toString(),
		 true,
		 function(x) { return x.folder + "." + x.id + "." + x.recurrence_position; } ,
		 function(x) { return { id: x[0], folder: x[1], recurrence_position : x[2] } }
		);
	},
	iterateStorage: function(storage) {		
		var Self = this;
		Self.displayedWeekLinks = [];
		this.appendetFirstChilds = [];
		if (storage) {
			weekStorage = storage;
			storage.events.register("Changed", function() {
				Self.updateView();
			});
		}
		weekStorage.newIterate(weekStorage.ids, emptyFunction, function(x,y) {
			Self.buildAppointments(x,y);
		},function(){
			Self.sortAppointments();
		});
	}, 
	buildAppointments: function(index, object) {
		this.weekAppointments[index] = object;
		object["indxxx"]  = index;
	},
	sortAppointments: function() {
		this.weekAppointments.sort(
			function(a, b){
				return (a[9] || a[4] < b[4] ? -1 : 1); 
			}
		);
		for(var i=0; i<this.weekAppointments.length;i++) {
			this.generateCalendarObjects(this.weekAppointments[i]["indxxx"],this.weekAppointments[i]);
		}
	},
	getAppointmentDiv:function (object) {
		var appointmentDate = new Date(object.start_date);
		var appointmentDateEnd = new Date(object.end_date);
		if(!object.title) {
			object.title="";
		}
		var textToView = formatDate(appointmentDate, "time") + " - " + formatDate(appointmentDateEnd, "time") + "  " + object.title;
		var shownDiv = getShownDiv(object.shown_as);
		
		var divIconParticipant = newnode("div", {},{ className: "showAppointmentSeqWeek"});
		if(object.participants && object.participants.length > 1) {
			var imageGroupFlag=document.createElement("img");
			imageGroupFlag.setAttribute("src",getFullImgSrc("img/calendar/group.gif"));
			divIconParticipant.appendChild(imageGroupFlag);
		}
		
		var divIconPrivate = newnode("div", {},{ className: "showAppointmentSeqWeek"});
		if(object.private_flag) {
			var imagePrivatFlag=document.createElement("img");
			imagePrivatFlag.setAttribute("src",getFullImgSrc("img/private_flag.gif"));
			divIconPrivate.appendChild(imagePrivatFlag);
		}
		
		var divIconTag = newnode("div", {}, { className: "showAppointmentSeqWeek"});
		if(object.color_label && object.color_label > 0) {
			var imageColorLabel=document.createElement("img");
			imageColorLabel.setAttribute("src",getFullImgSrc("img/menu/tag_" + object.color_label + ".gif"));
			divIconTag.appendChild(imageColorLabel);
		}		
		
		var textDiv;
		shownDiv.appendChild(document.createTextNode(" "));
		var datDIV = newnode('div');
		if (object.full_time) {
			datDIV.className = 'textFullTimeWeek wholeDayBackground border-color-design borderDefault';
		} else {
			datDIV.className = 'appInWeekView borderDefault background-color-content border-color-design';
		}
		textDiv = newnode('div');
		textDiv.className = "showAppointmentTextWeek mozSelected";
		
		textDiv.appendChild(document.createTextNode(textToView));
		datDIV.appendChild(shownDiv);	
		datDIV.appendChild(textDiv);
		
		if (divIconParticipant.firstChild) datDIV.appendChild(divIconParticipant);
		if (divIconPrivate.firstChild) datDIV.appendChild(divIconPrivate);
		if (divIconTag.firstChild) datDIV.appendChild(divIconTag);
		
		if (object.recurrence_type > 0) {
			var image=document.createElement("img");
			image.setAttribute("src",getFullImgSrc("img/calendar/calendar_series.gif"));
			var seqDiv = newnode('div', {}, { className: "showAppointmentSeqWeek"});
			seqDiv.className = "showAppointmentSeqWeek";	
			seqDiv.appendChild(image);
			datDIV.appendChild(seqDiv);	
		}
		
		return datDIV;						
	},
	generateCalendarObjects: function(index, object) {		
		this.index = index;
		this.object = object;
		var id = this.object[0];
		var folder_id = this.object[1];
		var nRecPosition = this.object[2];
		var nRecId = this.object[14];
		var created_by = this.object[15];
		this.object.index = index;
		allcalendarWeekObjects[""+folder_id+"-"+id + "-"+nRecPosition]=this.object;
		
		var selectedAppointmentWeek = { 
			id:id, folder:folder_id, recurrence_position:nRecPosition, 
			recurrence_id:nRecId, created_by:created_by };
		var divAppId = 'appInWeekView' + folder_id + "_" + id+"_"+nRecPosition;
		
		// additional clean-up? hack to fix bug #11406
		if ($2(divAppId) && $2(divAppId).parentNode) {
			$2(divAppId).parentNode.removeChild($2(divAppId));
		}
		
		var title = this.object[3];
		var start_date = this.object[4];
		var end_date = this.object[5];
		var note = this.object[6];
		var recurrence_type = this.object[7];
		
		// consolidate users and participants (field 220 and 221)
		for (var i in object[16]) {
			var a1 = object[16][i];
			for (var ia in object[8]) {
				if (object[8][ia].id == a1.id) {
					a1.confirmation = object[8][ia].confirmation;
					if (object[8][ia].confirmmessage) a1.confirmmessage = object[8][ia].confirmmessage;
					break;
				}
			}
		}
		
		var participants = this.object[8];
		var full_time = this.object[9];
		var shown_as = this.object[10];
		var color_label = this.object[11];
		var location = this.object[12];
		var private_flag = this.object[13];
		var appointmentDate = new Date(start_date);
		var appointmentDateEnd = new Date(end_date);''	
		var timeWeekpart1 = formatDate(appointmentDate, "time");
		var timeWeekpart2 = formatDate(appointmentDateEnd, "time");
		
		if (!title) title = "";		
		var textToView = timeWeekpart1 + " - " + timeWeekpart2 + "  " + title;
		if (full_time) {
			textToView = title; 
		}
		var divId = "weekDayContent" + appointmentDate.getUTCDay();
		
		this.aAppointmentAssoc[divAppId] = this.object;
		var divIconFlags = newnode("div", {flt:"right", marginLeft : "5px"});
       		
		var divIconParticipant = newnode("div", {},{ className: "showAppointmentSeqWeek"});
		if(participants && participants.length > 1) {
			var imageGroupFlag=document.createElement("img");
			imageGroupFlag.setAttribute("src",getFullImgSrc("img/calendar/group.gif"));
			divIconParticipant.appendChild(imageGroupFlag);
		}
		
		var divIconPrivate = newnode("div", {},{ className: "showAppointmentSeqWeek"});
		if(private_flag) {
			var imagePrivatFlag=document.createElement("img");
			imagePrivatFlag.setAttribute("src",getFullImgSrc("img/private_flag.gif"));
			divIconPrivate.appendChild(imagePrivatFlag);
		}
		
		var divIconTag = newnode("div", {}, { className: "showAppointmentSeqWeek"});
		if(color_label && color_label > 0) {
			var imageColorLabel=document.createElement("img");
			imageColorLabel.setAttribute("src",getFullImgSrc("img/menu/tag_" + color_label + ".gif"));
			divIconTag.appendChild(imageColorLabel);
		}
		
		var oneday=1000*60*60*24;
		var showappointment=true;		
		var fulltimeStartBefore = false;
		var fulltimeStartBeforeDiff = 0;
		
		// Setze Startzeit des Termins um , falls er vor der Anzeige beginnt
		if (start_date < this.starttime) {
			fulltimeStartBeforeDiff = (this.starttime - start_date)  / 1000 / 60 / 60 / 24;
			start_date=this.starttime;
			fulltimeStartBefore = true;
		}
		
		if (fulltimeStartBefore && !full_time) {
			end_date = end_date + oneday;
		}
		
		// Setze Endzeit des Termins um , falls er nach der Anzeige endet
		if (end_date > this.starttime + (7 * oneday)) {
			end_date = this.starttime + (7 * oneday)
		}
		
		/*
		 * ms / d
		 * calculating appointment duration in days
		 */
		var d_end = Math.ceil(end_date / 864e5);
        var d_start = Math.floor(start_date / 864e5);
		days = d_end-d_start;
		
		var titlesMultidays = [];
		if (days > 1) {
			var timeA = formatDate(new Date(Date.UTC(1970,1,1,0,0,0)), "time");
			var timeB = formatDate(new Date(Date.UTC(1970,1,1,23,59,59)), "time");			
			textToView = titlesMultidays[0] = timeWeekpart1 + " - " + timeB + "  " + title;
			if (full_time) {
				textToView = titlesMultidays[0] = title;
			}
			titlesMultidays[1] = timeA + " - " + timeB + "  " + title;
			if (full_time) {
				titlesMultidays[1] = title; 
			}
			titlesMultidays[2] = timeA + " - " + timeWeekpart2 + "  " + title;
			if (full_time) {
				titlesMultidays[2] = title;
			}
		}

		var Self = this;
		
		var shownDiv = newnode('div');
		switch (shown_as) {
			case 1:
				shownDiv.className = "appointmentRESERVED";
				break;
			case 2:
				shownDiv.className = "appointmentTEMPORARY";
				break;
			case 3:
				shownDiv.className = "appointmentABSENT";
				break;
			case 4:
				shownDiv.className = "appointmentFREE";
				break;
			default:
				shownDiv.className = "appointmentNONE";
				break;
		}
		shownDiv.className += ' showAppointmentShownWeek';
		
		var	paramObj = { id: Self.object[0], folder: Self.object[1], 
		                 folder_id: Self.object[1], recurrence_position: (Self.object[7]) ? Self.object[7] : 0
		               };

		function disabled(e) {
			var iconNode=$2("cww_node_dd");
			if (iconNode == undefined) {
				iconNode = newnode("img", 
				        { position:"absolute", left:(e.clientX + iconOffset) + "px", 
				          top:(e.clientY + iconOffset) + "px" },
			            { src: getFullImgSrc("img/dnd_disabled.gif"), alt: "" });
				iconNode.id="cww_node_dd";
				body.appendChild(iconNode);
				addTMPId(iconNode);
			}
			iconNode.style.left = (e.clientX + iconOffset) + "px";
			iconNode.style.top = (e.clientY + iconOffset)+ "px";
			stopEvent(e);
			return false;
		}
		
		function disabledremove(e,targetNode) {
			if ($2("cww_node_dd")) {
				$2("cww_node_dd").parentNode.removeChild($2("cww_node_dd"));
			}
		}

		shownDiv.appendChild(document.createTextNode(" "));
		var cName = "appInWeekView background-color-content border-color-design borderDefault";
		if (full_time) {
			cName = "textFullTimeWeek wholeDayBackground border-color-design borderDefault";
		}
		
		var datDIV = newnode("div", 0, { className: cName });
		if (selectedAppointmentWeekView == divAppId) {
			datDIV.className = removeClass(datDIV.className,"borderDefault");
			datDIV.className += ' selectedAppMonth border-color-selection-elements';
		}
		datDIV.id = divAppId;
		datDIV.oxcalobj = "" + folder_id + "-" + id + "-" + nRecPosition;
		var textDiv = newnode('div');
		textDiv.className = "showAppointmentTextWeek";
		
		if (location) {
			textToView += " - " + location;
		}
		
		// setting confirmation styles
		if (participants.length) {
			for (var i in participants) {
				if (participants[i].id == config.identifier) {
					switch(participants[i].confirmation) {
						case 0:
							datDIV.style.opacity = ".6";
							textDiv.style.fontStyle = "italic";
							textToView += " (" + _("waiting") + ")";
							break;
						case 2:
							textDiv.style.textDecoration = "line-through";
							break;
						case 3:											
							textDiv.style.fontStyle = "italic";
							textToView += " (" + _("tentative") + ")";
							break;
					}
				}
			}
		}
		
		textDiv.appendChild(document.createTextNode(textToView));
		datDIV.appendChild(shownDiv);	
		datDIV.appendChild(textDiv);
		
		if (divIconParticipant.firstChild) {
			datDIV.appendChild(divIconParticipant);
		}
        if (divIconPrivate.firstChild) {
        	datDIV.appendChild(divIconPrivate);
        }
        if (divIconTag.firstChild) {
        	datDIV.appendChild(divIconTag);
        }
		
		if (recurrence_type > 0) {
			var image=document.createElement("img");
			image.setAttribute("src",getFullImgSrc("img/calendar/calendar_series.gif"));
			var seqDiv = newnode('div');
			seqDiv.className = "showAppointmentSeqWeek";	
			seqDiv.appendChild(image);
			datDIV.appendChild(seqDiv);	
		}			
					
		registerSource(datDIV, "calendar",
		     function () {
		     	objMoveCopy.begin("calendar",
		     	[ new CalendarWorkWeekObject(
		     	       object[0], object[1], object[2], object[14], object[15], object[3], object[4], 
		     	       object[5], null, object[7], null, object[9], object[10],
		     	       null,null,null)
		     	]);
			    return objMoveCopy;
			     
			 }, null,null, calendardefaultdisabled, defaultdisabledremove);
		
	 	var oInplEdit  = new inplaceEdit(textDiv, title, 
	 	                    function (sStr,oObj) {
	 	                    	editAppointment(paramObj,{title:sStr});
							});

		//fixed bug with auto selection													
		if (calendarSelection) {
			calendarSelection.reset();
		}
			
		addDOMEvent(datDIV, "click", 
			function(e) {
				var SelObj = calendarSelection.getID();
				if (SelObj && SelObj.id == paramObj.id &&
							SelObj.folder == paramObj.folder_id &&
									SelObj.recurrence_position == paramObj.recurrence_position) {
					if (window.event) {
						e = window.event;
					}
					oInplEdit.on();
				} else {					
					setSelectionWeekView(selectedAppointmentWeek);
					selectAppointmentWeekView(divAppId, (full_time ? 2 : 1));
				}
				stopEvent(e);
			}
		);
        addDOMEvent(datDIV, "contextmenu", 
        	function(e) {            	
        		stopEvent(e);
            	globalContextMenus.calendar.display(e.clientX, e.clientY, calendarSelection);
        	}
        );
		addDOMEvent(datDIV, "mousedown", 
			function(e) {
				setSelectionWeekView(selectedAppointmentWeek);
				selectAppointmentWeekView(divAppId, (full_time ? 2 : 1));
												
			}
		);									
		addDOMEvent(datDIV, "dblclick", 
			function(e) {
				setActivatedWeekView(selectedAppointmentWeek, e);
				selectAppointmentWeekView(divAppId, (full_time ? 2 : 1));
			}
		);
		
		/*
		 * draw the first appointment
		 */																		
		if (!fulltimeStartBefore) {
			$(divId).appendChild(datDIV);
			addTMPId(datDIV);
		}
		
		/*
		 * if appointment lasts longer then 1 day draw the rest
		 */
		for (var i=1; i<days; i++) {
			var ccweek_template_app_time = datDIV.cloneNode(true);
			if (i+1 == days) {
				ccweek_template_app_time.childNodes[1].firstChild.nodeValue = titlesMultidays[2];
			} else {
				ccweek_template_app_time.childNodes[1].firstChild.nodeValue = titlesMultidays[1];
			}

			var	paramObj = { id:Self.object[0],folder:Self.object[1], folder_id: Self.object[1],
											recurrence_position:(Self.object[7]) ? Self.object[7] : 0 };
			if (!fulltimeStartBefore) {
				start_date += oneday;
			}
			
			var newId = new Date(start_date).getUTCDay();
			divId = "weekDayContent" + newId;

			ccweek_template_app_time.id = datDIV.id + "-" + i;
			ccweek_template_app_time.oxcalobj= "" + folder_id + "-" + id + "-" + nRecPosition;

			// remove if already exists!
			if ($2(ccweek_template_app_time.id) && $2(ccweek_template_app_time.id).parentNode) {
				$2(ccweek_template_app_time.id).parentNode.removeChild($2(ccweek_template_app_time.id));
			}
			
			addDOMEvent(ccweek_template_app_time, "click", function(e) {					
				var SelObj = calendarSelection.getID();
				if (SelObj && SelObj.id == paramObj.id &&
				    SelObj.folder == paramObj.folder_id &&
					SelObj.recurrence_position == paramObj.recurrence_position) {
					if (window.event) {
						e = window.event;
					}
					oInplEdit.on();
				} else {
					setSelectionWeekView(selectedAppointmentWeek);
					selectAppointmentWeekView(divAppId, (full_time ? 2 : 1));
				}
				stopEvent(e);
			});	
			
            addDOMEvent(ccweek_template_app_time, "contextmenu", 
	            	function(e) {
		                if (!calendarSelection.get(index)) {
		                    calendarSelection.click(index, e);
		                }
		                stopEvent(e);
		                globalContextMenus.calendar.display(e.clientX, e.clientY, calendarSelection);
	            	}
            );
			addDOMEvent(ccweek_template_app_time, "mousedown", 
					function(e) {
						setSelectionWeekView(selectedAppointmentWeek);
						selectAppointmentWeekView(divAppId, (full_time ? 2 : 1));
					}
			);
			addDOMEvent(ccweek_template_app_time, "dblclick", 
					function(e) {
						setActivatedWeekView(selectedAppointmentWeek);
						selectAppointmentWeekView(divAppId, (full_time ? 2 : 1));
					}
			);
			$(divId).appendChild(ccweek_template_app_time);	
			addTMPId(ccweek_template_app_time);
			if (fulltimeStartBefore) {
				start_date += oneday;
			}
		}
	}
}

function setSelectionWeekView(selectedAppointmentWeek) {
	selectedAppointment = selectedAppointmentWeek;
	var arrId = new Array();
	arrId.push(selectedAppointment);
	var arrayIndex = selectedAppointmentWeek.folder + "-" + selectedAppointmentWeek.id + "-" + selectedAppointmentWeek.recurrence_position;
	calendarSelection.click(allcalendarWeekObjects[arrayIndex].index);
	triggerEvent("Selected", arrId);
}

function setActivatedWeekView(selectedAppointmentWeek, evt) {
	selectedAppointment = selectedAppointmentWeek;
	openAppointmentPopup(evt);
}

var selectedAppointmentWeekView = '';
var selectedAppointmentWeekViewModus = 0;
function selectAppointmentWeekView(divAppId, app) {
	if ($2(divAppId)) {
		$2(divAppId).className = removeClass($2(divAppId).className, "border-color-selection-elements");
		$2(divAppId).className = removeClass($2(divAppId).className, "borderDefault");
		$2(divAppId).className += " selectedAppMonth border-color-selection-elements";
	}
	
	for (var i=0; i<=7; i++) {
		if($2(divAppId + "-" + i)) {
			$2(divAppId + "-" + i).className = removeClass($2(divAppId + "-" + i).className, "border-color-selection-elements");
			$2(divAppId + "-" + i).className = removeClass($2(divAppId + "-" + i).className, "borderDefault");			
			$2(divAppId + "-" + i).className += " selectedAppMonth border-color-selection-elements";
		}
	}
	
				
	if (selectedAppointmentWeekView != "" && $2(selectedAppointmentWeekView) && selectedAppointmentWeekView != divAppId) {
		var cName = "textFullTimeWeek wholeDayBackground border-color-design borderDefault";
		if (selectedAppointmentWeekViewModus == 1) {
			cName = "appInWeekView background-color-content border-color-design borderDefault";
		}		
		$2(selectedAppointmentWeekView).className = cName;
	}
	
	if (selectedAppointmentWeekView != "" && selectedAppointmentWeekView != divAppId) {
		for (var i=0; i<=7; i++) {
			if ($2(selectedAppointmentWeekView + "-" + i)) {
				var cName = "textFullTimeWeek wholeDayBackground border-color-design borderDefault";
				if (selectedAppointmentWeekViewModus == 1) {
					cName = "appInWeekView background-color-content border-color-design borderDefault";
				}
				$2(selectedAppointmentWeekView + "-" + i).className = cName;
			}
		}
	}
	
	if (divAppId) {
		selectedAppointmentWeekView = divAppId;
		selectedAppointmentWeekViewModus = app;
	}
}

var objWeekCalendarTool = new WeekCalendarTool();
fileloaded();
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Suphi Basdemir <suphi.basdemir@open-xchange.com>
 * @author Markus Wagner <markus.wagner@open-xchange.com>
 * 
 */

var completedetailappointment;
var aDetailFieldsToClear = {};
var idsAppParticipantForSelected = new Array();
var a_d_timestamp = 0;
var a_d_participantGrid = null;	
var objAppDetailParticipants = null;
var refAppDetailParticipantTab = null;
var gridatt = null;
var currentAppointmentDetailTab = 'appointment';
var aPrintDetailApp = new Array();
function caldetail_change_Confirm() {
	var calconfirm=new AcceptDeny(true);
	calconfirm.showConfirmation([new AcceptDenyObject("calendar",completedetailappointment)]);
}
/********************************** Tab Appointment *****************************************************************/
register("OX_Calendar_Switch_Detail",function(param) { menuswitchCalendarDetail(param) });
function detailAppointmentOnload(callback)
{
	/********************************** Tabs *****************************************************************/

	/**
	 * global vars for detail appointment view
	 */
	a_d_tabsList = new Array('tabAppointmentDetail1','tabAppointmentDetail2', 'tabAppointmentDetail3');
	a_d_panelsList = new Array('panelAppointmentDetail1','panelAppointmentDetail2', 'panelAppointmentDetail3');
	a_d_eventList =
			new Array(	['OX_Calendar_Switch_Detail','overview'],
						['OX_Calendar_Switch_Detail','participant'],
						['OX_Calendar_Switch_Detail','attachment']);
	
	//register tabs
	setTabLists(a_d_tabsList, a_d_panelsList, a_d_eventList);
	//var eventList =	null;
	//changeTab('tabAppointmentDetail1', 'panelAppointmentDetail1');
	

//	/* Grid for Attachments */
//	gridatt = makeDetailAppointmentAttachmentGrid();

	var json = new JSON();
	var recurrencePosition = (selectedAppointment.recurrence_position) ? selectedAppointment.recurrence_position : 0;
	
	json.get(AjaxRoot + "/calendar?action=get&session=" + session 
									+"&id="+selectedAppointment.id
									+"&folder="+selectedAppointment.folder
									+"&recurrence_position="+recurrencePosition+"&"
		,
		null,
		function(fieldsToGet) {
			if (fieldsToGet) {				
				objAppDetailParticipants = new AppDetailParticipants();
				if(!refAppDetailParticipantTab) {
					refAppDetailParticipantTab = new ObjAppDetailParticipantTab();
					refAppDetailParticipantTab.makeGrid();
				}
				if(storageCache && storageCache.current)
					storageCache.current.timestamp = fieldsToGet.timestamp;
				completedetailappointment=fieldsToGet.data;
				triggerEvent("Confirmation_Changed",0);
				for(var i=0;i<completedetailappointment.users.length;i++) {
					if(completedetailappointment.users[i].id==configGetKey("identifier")) {
						triggerEvent("Confirmation_Changed",1);
					}
				}
				//clear all fields
				var tmpDivToClear = null;
				for (idDetailFieldsToClear in aDetailFieldsToClear) 
				{
					if(aDetailFieldsToClear[idDetailFieldsToClear] == "set" && $(idDetailFieldsToClear).firstChild)
						$(idDetailFieldsToClear).firstChild.data = " "; 
					else if(aDetailFieldsToClear[idDetailFieldsToClear] == "setInput")	
						$(idDetailFieldsToClear).value = " "; 	
				}
				removeChildNodes($("a_d_note")); // clean-up notes
				
				var participantTabDiv = $("a_d_count_participants");
				
				if(!fieldsToGet.data['participants']) {
					participantTabDiv.firstChild.data = "0";
				}
				
				//hide next button for participants
				$("a_d_participants_more").style.display = 'none';
				
				//reset fields
				aDetailFieldsToClear = {};
		
				a_d_timestamp = fieldsToGet.timestamp;
				lastUpdateOfCalendarTimestamp = fieldsToGet.timestamp;
				aPrintDetailApp = [];
				if (fieldsToGet.data.number_of_attachments)
				{
					$('a_d_count_attachment').firstChild.nodeValue = fieldsToGet.data.number_of_attachments;
				} else {
					$('a_d_count_attachment').firstChild.nodeValue = "0";
				}
				
				if (fieldsToGet.data.color_label) {
					$("tagImgAppointmentDetail").src = getFullImgSrc("img/menu/tag_"+fieldsToGet.data.color_label+".gif");
				} else {
					$("tagImgAppointmentDetail").src = getFullImgSrc("img/menu/tag_0.gif");
				}
				if (fieldsToGet.data.private_flag) {
					$("privateflagImgAppointmentDetail").src = getFullImgSrc("img/private_flag.gif");
				} else {
					$("privateflagImgAppointmentDetail").src = getFullImgSrc("img/dummy.gif");
				}
				var myseries=new SeriesObject(fieldsToGet.data,true);
				$("a_d_series").firstChild.data = myseries.getSeriesString();
				if (fieldsToGet.data.recurrence_type > 0) {
					$("seriesflagImgAppointmentDetail").src = getFullImgSrc("img/calendar/calendar_series.gif");		
				} else {
					$("seriesflagImgAppointmentDetail").src = getFullImgSrc("img/dummy.gif");
				}
				
				var sText = "";
				var nText = "";
				for(a_d_data in fieldsToGet.data){
					switch (a_d_data) {
						case 'title':
							$("a_d_title").firstChild.data = fieldsToGet.data[a_d_data];	
							aDetailFieldsToClear["a_d_title"] = "set"; //for deleting the next time, when show detail								
							aPrintDetailApp[0] = Print.createTitle(fieldsToGet.data[a_d_data]);
							aPrintDetailApp[1] = Print.createLine("5px");
							aPrintDetailApp[2] = Print.createEmpty();
							break;
						case 'location':
							$("a_d_location").firstChild.data = fieldsToGet.data[a_d_data];
							aDetailFieldsToClear["a_d_location"] = "set";						
							aPrintDetailApp[3] = Print.createContent(_("Location:"),fieldsToGet.data[a_d_data]);
							break;
						case 'note':
						    removeChildNodes($("a_d_note"));
                            var nText = fieldsToGet.data[a_d_data] ? fieldsToGet.data[a_d_data].split("\n") : "";
			                for (var i in nText) {
			                    $("a_d_note").appendChild(document.createTextNode(nText[i]));
			                    $("a_d_note").appendChild(newnode("br"));
			                }
			                nText = null;
							aDetailFieldsToClear["a_d_note"] = "set";	
							aPrintDetailApp[6] = Print.createContent(_("Description:"),fieldsToGet.data[a_d_data]);
							break;
						case 'start_date':	
							var startDate = new Date(fieldsToGet.data[a_d_data]);
							this.start = startDate.getUTCHours() + "." + (parseInt((startDate.getUTCMinutes()/15), 10)*25);	// startdate for calculating
							
							if (fieldsToGet.data['full_time']) {
								this.starthuman = formatDate(startDate, "date");
							} else {
								this.starthuman = formatDate(startDate, "datetime");//startDate.getUTCHours() + ":" + parseInt(startDate.getUTCMinutes()/15, 10)*15;	// human readable start time
								if((this.starthuman.length - (this.starthuman.lastIndexOf(":"))) < 3)
									this.starthuman += "0";
							}
							
							sText = this.starthuman;
							$("a_d_begins").firstChild.data = sText;
							aPrintDetailApp[4] = Print.createContent(_("Start date:"),sText);
							var sStartText = (fieldsToGet.data["title"] ? fieldsToGet.data["title"] : "") + " - " + this.starthuman;
							aDetailFieldsToClear["a_d_begins"] = "set";
							
							activeYear = startDate.getUTCFullYear();
							activeMonth = startDate.getUTCMonth();
							activeDay = startDate.getUTCDate();
							if (oMiniCalendar) oMiniCalendar.setSelectedByDate(activeYear, activeMonth, activeDay);

							break;
						case 'end_date':
							var endDate = new Date(fieldsToGet.data[a_d_data]);
							
							if(fieldsToGet.data['full_time']) {
								var endDateNew = endDate - 86400000;
								this.endhuman = formatDate(endDateNew, "date");
							} else {
								this.endhuman = formatDate(endDate, "datetime");//endDate.getUTCHours() + ":" + parseInt(endDate.getUTCMinutes()/15)*15;	// human readable end time
							}
							var sText = this.endhuman;
							$("a_d_ends").firstChild.data = sText;	
							aPrintDetailApp[5] = Print.createContent(_("End:"),this.endhuman);
							
							var sEndText = " "  + _("to") + " " + this.endhuman;
							
							aDetailFieldsToClear["a_d_ends"] = "set";	
							aDetailFieldsToClear["a_d_headline"] = "set";	
							break;
							
						case 'participants':
							(callback) ? objAppDetailParticipants.callback = callback : objAppDetailParticipants.callback = null;
							objAppDetailParticipants.oParticipantResources = fieldsToGet.data[a_d_data];	
							objAppDetailParticipants.writeCountParticipants();
							objAppDetailParticipants.pushArray();
							var ids=clone(objAppDetailParticipants.idUserArr);
							internalCache.getUsers(ids,transit_a_d_fnSetUserColumn);															
							break;
							
						case 'folder_id':
						
							function cb_fdrpath(sText)
							{
								$("a_d_folder").firstChild.data = sText;
							}
							oMainFolderTree.get_folder_path(fieldsToGet.data[a_d_data],cb_fdrpath);
							break;
							
						case 'shown_as':
							var newClassName = "";
							var sShownAs = "";
							switch (fieldsToGet.data[a_d_data]) {
								case 1	:	newClassName = "appointmentRESERVED";
											sShownAs = _("reserved");
											aPrintDetailApp[7] = Print.createContent(_("Display as:"),_("reserved"));
											break;
											
								case 2	:	newClassName = "appointmentTEMPORARY";
											sShownAs = _("temporary");
											aPrintDetailApp[7] = Print.createContent(_("Display as:"),_("temporary"));
											break;
											
								case 3	:	newClassName = "appointmentABSENT";
											sShownAs = _("absent on business");
											aPrintDetailApp[7] = Print.createContent(_("Display as:"),_("absent on business"));
											break;
											
								case 4	:	newClassName = "appointmentFREE";
											sShownAs = _("free");
											aPrintDetailApp[7] = Print.createContent(_("Display as:"),_("free"));
											break;			
									
								default	:	newClassName = "appointmentNONE";
											break;		
											
							}
							
							$("a_d_show_as_color").className = newClassName;
							$("a_d_show_as").firstChild.data = sShownAs;	
							aDetailFieldsToClear["a_d_show_as"] = "set";	
							break;
							
						case 'alarm':
							if(fieldsToGet.data[a_d_data] >= 0) {									
								var sConvertedTime = getInterval(fieldsToGet.data[a_d_data] * 60 * 1000);
								$("a_d_alarm").firstChild.data = sConvertedTime;		
								aDetailFieldsToClear["a_d_alarm"] = "set";
							}
							break;
							
						case 'categories':
							$("a_d_categories").firstChild.data = fieldsToGet.data[a_d_data];
							aPrintDetailApp[8] = Print.createContent(_("Tags:"),fieldsToGet.data[a_d_data]);	
							aDetailFieldsToClear["a_d_categories"] = "set";
							break;
							
						case 'users':
							objAppDetailParticipants.usersD = fieldsToGet.data[a_d_data];
							break;
						case 'created_by':
							internalCache.getUsers([fieldsToGet.data[a_d_data]],	function (arg)
							{
								if(fieldsToGet.data[a_d_data] && arg[fieldsToGet.data[a_d_data]]) {
									$("a_d_created_by").firstChild.data = arg[fieldsToGet.data[a_d_data]].display_name;	
									aDetailFieldsToClear["a_d_created_by"] = "set";
								} else if(arg) {
									for(i in arg) {
										$("a_d_created_by").firstChild.data = arg[i].display_name;	
										aDetailFieldsToClear["a_d_created_by"] = "set";
									}
								}
							});
							break;
							
						default:
							break;
					}
				}

				writeBottomString(fieldsToGet.data,"app_detail_bot");
				$("a_d_headline").firstChild.data = sStartText + sEndText;
			}
			
		},
		function(error) {
			triggerEvent("OX_Switch_View", configGetKey("gui.calendar.view"));
			return true;
		}
	); 
}

/* Participant object */
function AppDetailParticipants() {
	this.oParticipantResources = null;
	this.idUserArr = new Array();
	this.idGroupArr = new Array();
	this.idResourceArr = new Array();
	this.sText = "";
	this.nText = null;
	this.nPart = 0;
	this.gridData = new Array();
	this.gridIndex = 0;
	this.idForExternal = 1;
	this.usersD = null;
	this.callback = null;
}

AppDetailParticipants.prototype = {
	writeCountParticipants : function () {
		this.sText = this.oParticipantResources.length;
		$("a_d_count_participants").firstChild.data = this.sText;
	},
	pushArray : function () {
		for(i=0; i < this.oParticipantResources.length; i++)
		{
				/*
				 * type 1 user
				 * type 2 user group
				 * type > 2 resource
				 * */
			switch (this.oParticipantResources[i].type) {
				case 1:		this.idUserArr.push(this.oParticipantResources[i].id);
							break;
							
				case 2: 	this.idGroupArr.push(this.oParticipantResources[i].id);
							break;
							
				case 3: 	this.idResourceArr.push(this.oParticipantResources[i].id);
							break;
							
				case 5: 	this.idUserArr.push(this.oParticipantResources[i]);
							break;
			}						
		}
	},
	writeMoreButton : function() {
		$("a_d_participants_more").style.display = 'inline';
		$("a_d_participants_more").style.display = 'block';
	},
	writeMoreButtonResource : function() {
		$("a_d_participants_more_resources").style.display = 'inline';
		$("a_d_participants_more_resources").style.display = 'block';
	},
	a_d_fnSetUserColumn : function (arg) {
		
		for(var i=0; i < this.idUserArr.length; i++) {
			if(this.nPart < 6 ) {	
				if(arg[this.idUserArr[i]]) {
					this.sText = arg[this.idUserArr[i]].display_name;										
					$("a_d_participants_" + (this.nPart+1)).firstChild.data = this.sText										
					aDetailFieldsToClear["a_d_participants_" + (this.nPart+1)] = "set";						
					this.nPart++;				
				} else if(typeof this.idUserArr[i] == "object") {	
					this.sText = this.idUserArr[i].display_name || this.idUserArr[i].mail || _("unkown");										
					$("a_d_participants_" + (this.nPart+1)).firstChild.data = this.sText										
					aDetailFieldsToClear["a_d_participants_" + (this.nPart+1)] = "set";	
					this.nPart++;
				}					
			} else {
				this.writeMoreButton();
			}
			
			if(typeof this.idUserArr[i] == "number") {
				this.gridData[this.gridIndex] = new Array();
				this.gridData[this.gridIndex].push(arg[this.idUserArr[i]].id);
				this.gridData[this.gridIndex].push("user");
				this.gridData[this.gridIndex].push(arg[this.idUserArr[i]].display_name);		
				this.gridIndex++;
				aPrintDetailApp.push(Print.createContent(_("Participant:"),arg[this.idUserArr[i]].display_name || arg[this.idUserArr[i]].mail));	
			} else {
				this.gridData[this.gridIndex] = new Array();
				this.gridData[this.gridIndex].push(this.idForExternal + "ext");
				this.gridData[this.gridIndex].push("user_extern");
				this.gridData[this.gridIndex].push(this.idUserArr[i].display_name || this.idUserArr[i].mail);		
				this.gridIndex++;
				this.idForExternal ++;
				aPrintDetailApp.push(Print.createContent(_("Participant:"),this.idUserArr[i].display_name || this.idUserArr[i].mail));
			}			
		}

		this.gridData.sort(function(a,b){
			if (a[2] && b[2]) {
				return a[2].toLowerCase() < b[2].toLowerCase() ? -1 : 1;
			} else {
				return -1;
			}
		});
		
		var Self = this;
		
		setTimeout(
			function() {
				if(Self.usersD)
				refAppDetailParticipantTab.users= Self.usersD;
				//refAppDetailParticipantTab.appendToStorage(); Moved to enableGrid
			},20
		);
		setTimeout(
			function(){
				internalCache.getGroups(objAppDetailParticipants.idGroupArr,transit_a_d_fnSetGroupColumn)
			}, 0
		);
	},
	a_d_fnSetGroupColumn : function (arg)
	{
		var tmp_grouplength = new Array();
		var tmp_userToGet = new Array();
		for(var i=0; i < this.idGroupArr.length; i++)
		{
			if(this.nPart < 6 ) 
			{										
				this.sText = arg[this.idGroupArr[i]].display_name;								
				$("a_d_participants_" + (this.nPart+1)).firstChild.data = this.sText										
				aDetailFieldsToClear["a_d_participants_" + (this.nPart+1)] = "set";		
				this.nPart++;									
			}
			else
			{
				this.writeMoreButton();
			}
			this.gridData[this.gridIndex] = new Array();
			this.gridData[this.gridIndex].push(arg[this.idGroupArr[i]].id);
			this.gridData[this.gridIndex].push("group");
			this.gridData[this.gridIndex].push(arg[this.idGroupArr[i]].display_name);	
			this.gridData[this.gridIndex].push("group");
			this.gridData[this.gridIndex].push("");
			tmp_grouplength[this.idGroupArr[i]] = this.gridIndex + 1;
			this.gridIndex += (arg[this.idGroupArr[i]].members.length + 1);
			for(tz in arg[this.idGroupArr[i]].members)
				tmp_userToGet.push({id : arg[this.idGroupArr[i]].members[tz], type : 1});
		}
		var Self = this;
		internalCache.getObjects(tmp_userToGet, function(cb){
			for(gr in tmp_grouplength){
				var tmp_members = new Array();
				var start = tmp_grouplength[gr];
				for(me in arg[gr].members){
					tmp_members[tmp_grouplength[gr]] = new Array();
					tmp_members[tmp_grouplength[gr]].push(arg[gr].members[me] + "gm" + arg[gr].id);
					tmp_members[tmp_grouplength[gr]].push("groupMember");
					tmp_members[tmp_grouplength[gr]].push(cb[arg[gr].members[me]].display_name);
					tmp_grouplength[gr]++;
				}
				tmp_members.sort(function(a,b){
					if (a[2] && b[2]) {
						return a[2].toLowerCase() < b[2].toLowerCase() ? -1 : 1;
					} else {
						return -1;
					}
				});
				for(var z=0; z<tmp_members.length;z++){
					if(tmp_members[z]) {
						Self.gridData[start] = tmp_members[z];
						start++;
					}
				}
			}
		});
		setTimeout(
			function(){
				if(objAppDetailParticipants && objAppDetailParticipants.idResourceArr) {
					internalCache.getResources(objAppDetailParticipants.idResourceArr,transit_a_d_fnSetResourceColumn)
				}
			}, 0
		);
	},
	a_d_fnSetResourceColumn : function (arg)
	{
		for(var i=0; i < this.idResourceArr.length; i++)
		{
			if(i < 2 ) 
			{										
				this.sText = arg[this.idResourceArr[i]].display_name;									
				$("a_d_resources_" + (i+1)).firstChild.data = this.sText										
				aDetailFieldsToClear["a_d_resources_" + (i+1)] = "set";		
			}
			else
			{
				this.writeMoreButtonResource();
			}
		this.gridData[this.gridIndex] = new Array();
		this.gridData[this.gridIndex].push(arg[this.idResourceArr[i]].id);
		this.gridData[this.gridIndex].push("ressourcen");	
		this.gridData[this.gridIndex].push(arg[this.idResourceArr[i]].display_name);	
		this.gridData[this.gridIndex].push("ressourcen");	
		this.gridData[this.gridIndex].push("");
		this.gridIndex++;
		}
		var Self = this;
		if(Self.callback) {
			setTimeout(
				function() {
					Self.gridIndex = 0;
					Self.callback();
				},50
			);
		}
	}
}

function transit_a_d_fnSetUserColumn(arg) {
	objAppDetailParticipants.a_d_fnSetUserColumn(arg);
}
function transit_a_d_fnSetGroupColumn(arg) {
	objAppDetailParticipants.a_d_fnSetGroupColumn(arg);
}
function transit_a_d_fnSetResourceColumn(arg) {
	objAppDetailParticipants.a_d_fnSetResourceColumn(arg);
}

/********************************** Tab Participants *****************************************************************/

function ObjAppDetailParticipantTab() {
	this.users = null;
	this.list = $("a_d_listParticipants");
	this.listHeader = $("a_d_listParticipantsHeader");
	this.myAppPStorage = null;
	this.myAppPStorage = new Storage(0, []);
}

ObjAppDetailParticipantTab.prototype = {
	
	makeGrid : function () {		
	
		if (a_d_participantGrid) {
			removeChildNodes(this.list);
			a_d_participantGrid = null;
		}
		
		a_d_participantGrid = new LiveGrid([
					{	
						text: "&#160;",
						sortable: false,
						width: "16px",
						style: {padding: 0},
						set: function(div,text) 
							{
								if (div) {
									removeChildNodes(div);
									if(text != "groupMember") {
										div.appendChild(newnode("img", 0, {src : getFullImgSrc("img/calendar/" + text + ".gif"), alt : "participant"}));
									}
									else
										div.appendChild(newnode("img", 0, {src : getFullImgSrc("img/dummy.gif"), alt : "participants", width: "16", height: "16"}));
								}
							},
						clear: LiveGrid.makeClear("..."),
						index:1
					},{	
						text: _("Participants"),
						sortable: false,
						width: "30%",
						set: function(div, text){
							if(div){
								if(text[1] == "groupMember"){
									removeChildNodes(div);
									div.appendChild(newnode("img", 0, {src: getFullImgSrc("img/calendar/user.gif"), alt: "Group Member"}));
									div.appendChild(document.createTextNode("\u00a0" + text[2]));
								} else {
									removeChildNodes(div);
									div.appendChild(document.createTextNode(text[2]));
								}
							}
						},
						clear: LiveGrid.makeClear("...")
					}, {
						text: _("Status"),
						sortable: false,
						width: "10%",
						set: function(div,text) {
							if (div) {
								removeChildNodes(div);
								switch(text) {
									case 0	:	div.appendChild(document.createTextNode(_("waiting")));
													break;
													
									case 1	:	div.appendChild(newnode("span", 0, {className : "appointmentACCEPTED"}, [document.createTextNode(_("accepted"))]));
													break;
													
									case 2	:	div.appendChild(newnode("span", 0, {className : "appointmentDECLINED"}, [document.createTextNode(_("declined"))])); 
													break;
													
									case 3	:	div.appendChild(newnode("span", 0, {className : "appointmentTENTATIVE"}, [document.createTextNode(_("tentative"))]));
													break;
													
									default	:	div.appendChild(document.createTextNode("\u00a0"));
													break;
								}
							}
						},
						clear: LiveGrid.makeClear("..."),
						index: 3
					}, {
						text: _("Comment"),
						sortable: false,
						set: LiveGrid.defaultSet,
						clear: LiveGrid.makeClear("..."),
						index: 4
					}
		], new Selection());

		this.listHeader.appendChild(a_d_participantGrid.getHeader());
		a_d_participantGrid.getTable(this.list);
	},
	
	appendToStorage : function() {	
		for(var i = 0; i < objAppDetailParticipants.gridData.length; i++) {
			for (var t = 0; t < this.users.length; t++) {
				
				if(objAppDetailParticipants.gridData[i][0].toString().indexOf("gm") < 0){
					if(this.users[t].id == objAppDetailParticipants.gridData[i][0]) {
						objAppDetailParticipants.gridData[i].push(this.users[t].confirmation);
						objAppDetailParticipants.gridData[i].push(this.users[t].confirmmessage || "");	
					}
				} else {
					if(this.users[t].id == objAppDetailParticipants.gridData[i][0].split("gm")[0]) {
						objAppDetailParticipants.gridData[i].push(this.users[t].confirmation);
						objAppDetailParticipants.gridData[i].push(this.users[t].confirmmessage || "");	
					}
				}
			}			
		}
	},
	
	enableGrid : function () {
		this.appendToStorage();
		// copy to an array, because the gridData is a hashmap!!!
		var tmp_participants = new Array();
		for(var i=0; i<objAppDetailParticipants.gridData.length;i++) {			
			tmp_participants[i] = objAppDetailParticipants.gridData[i];
		}
		this.myAppPStorage.append(tmp_participants);
		//enable the grid
		a_d_participantGrid.enable(this.myAppPStorage); 
	},
	
	disableGrid	: function() {
		this.myAppPStorage.remove(0,objAppDetailParticipants.gridData.length);
		a_d_participantGrid.disable(); 
	}
}

var gridAttachmentAppDetail = null;
function makeDetailAppointmentAttachmentGrid() {
		var nId = selectedAppointment.id;
		var nFolder = selectedAppointment.folder;
		gridatt = new attachmentGrid("a_d_Attachments",nId,nFolder,1);	
}

/*
 * Print Detail Appointment
 * 
 * */
function calendar_printDetail(){
	var title = aPrintDetailApp[0] && aPrintDetailApp[0].title ? aPrintDetailApp[0].title : "";
	var detailCalendarPrint = new Print(title);
	detailCalendarPrint.printDetail(aPrintDetailApp);
}

fileloaded();
